/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

'use strict';

var util = require('util');
var azureCommon = require('azure-common');
var Service = azureCommon.Service;
var WebResource = azureCommon.WebResource;

var KeyVaultManagementClient = ( /** @lends KeyVaultManagementClient */ function() {
  /**
   * @class
   * The Windows Azure management API provides a RESTful set of web services
   * that interact with Azure Key Vault.
   * Initializes a new instance of the KeyVaultManagementClient class.
   * @constructor
   * 
   * @param {SubscriptionCloudCredentials} credentials Gets subscription
   * credentials which uniquely identify Microsoft Azure subscription. The
   * subscription ID forms part of the URI for every service call.
   * 
   * @param {string} [credentials.subscriptionId]
   * 
   * @param {string} [baseUri] Gets the URI used as the base for all cloud
   * service requests.
   * 
   * @param {Array} filters
   */
  function KeyVaultManagementClient(credentials, baseUri, filters) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    KeyVaultManagementClient['super_'].call(this, credentials, filters);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.azure.com';
    }
    if (this.apiVersion === null || this.apiVersion === undefined) {
      this.apiVersion = '2014-04-01';
    }
    if (this.longRunningOperationInitialTimeout === null || this.longRunningOperationInitialTimeout === undefined) {
      this.longRunningOperationInitialTimeout = -1;
    }
    if (this.longRunningOperationRetryTimeout === null || this.longRunningOperationRetryTimeout === undefined) {
      this.longRunningOperationRetryTimeout = -1;
    }
    /**
     * Provides an instance of the [VaultOperations](-VaultOperations.html)
     * object.
     * @type {object}
     */
    this.vaults = new VaultOperations(this);
  }
  
  util.inherits(KeyVaultManagementClient, Service);
  
  
  return KeyVaultManagementClient;
})();
exports.KeyVaultManagementClient = KeyVaultManagementClient;

var VaultOperations = ( /** @lends VaultOperations */ function() {
  /**
   * @class
   * Vault operations
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [KeyVaultManagementClient] {@link
   * KeyVaultManagementClient~KeyVaultManagementClient}.
   * See [vaults] {@link
   * KeyVaultManagementClient~KeyVaultManagementClient#vaults}.
   * Initializes a new instance of the VaultOperations class.
   * @constructor
   * 
   * @param {KeyVaultManagementClient} client Reference to the service client.
   */
  function VaultOperations(client) {
    this.client = client;
  }
  
  /**
   * Creates a new Azure key vault.
   * 
   * @param {string} resourceGroupName The name of the Resource Group to which
   * the server belongs.
   * 
   * @param {string} vaultName
   * 
   * @param {VaultCreateOrUpdateParameters} parameters
   * 
   * @param {VaultProperties} parameters.properties Gets or sets the location
   * of the resource.
   * 
   * @param {string} [parameters.properties.vaultUri]
   * 
   * @param {Guid} [parameters.properties.tenantId]
   * 
   * @param {Sku} [parameters.properties.sku]
   * 
   * @param {string} [parameters.properties.sku.family]
   * 
   * @param {string} [parameters.properties.sku.name]
   * 
   * @param {AccessPolicyEntry[]} [parameters.properties.accessPolicies]
   * 
   * @param {boolean} [parameters.properties.enabledForDeployment]
   * 
   * @param {string} parameters.location Gets or sets the location of the
   * resource.
   * 
   * @param {object} parameters.tags Gets or sets the tags associated with the
   * request.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VaultOperations.prototype.createOrUpdate = function(resourceGroupName, vaultName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vaultName === null || vaultName === undefined) {
      return callback(new Error('vaultName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.location === null || parameters.location === undefined) {
      return callback(new Error('parameters.location cannot be null.'));
    }
    if (parameters.properties === null || parameters.properties === undefined) {
      return callback(new Error('parameters.properties cannot be null.'));
    }
    if (parameters.tags === null || parameters.tags === undefined) {
      return callback(new Error('parameters.tags cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.KeyVault';
    url2 = url2 + '/vaults/';
    url2 = url2 + encodeURIComponent(vaultName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-19-PREVIEW');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = null;
    
    var vaultCreateOrUpdateParametersValue = {};
    requestDoc = vaultCreateOrUpdateParametersValue;
    
    var propertiesValue = {};
    vaultCreateOrUpdateParametersValue['properties'] = propertiesValue;
    
    if (parameters.properties.vaultUri !== null && parameters.properties.vaultUri !== undefined) {
      propertiesValue['vaultUri'] = parameters.properties.vaultUri;
    }
    
    if (parameters.properties.tenantId !== null && parameters.properties.tenantId !== undefined) {
      propertiesValue['tenantId'] = parameters.properties.tenantId;
    }
    
    if (parameters.properties.sku !== null && parameters.properties.sku !== undefined) {
      var skuValue = {};
      propertiesValue['sku'] = skuValue;
      
      if (parameters.properties.sku.family !== null && parameters.properties.sku.family !== undefined) {
        skuValue['family'] = parameters.properties.sku.family;
      }
      
      if (parameters.properties.sku.name !== null && parameters.properties.sku.name !== undefined) {
        skuValue['name'] = parameters.properties.sku.name;
      }
    }
    
    if (parameters.properties.accessPolicies !== null && parameters.properties.accessPolicies !== undefined) {
      var accessPoliciesArray = [];
      for (var loweredIndex1 = 0; loweredIndex1 < parameters.properties.accessPolicies.length; loweredIndex1 = loweredIndex1 + 1) {
        var accessPoliciesItem = parameters.properties.accessPolicies[loweredIndex1];
        var accessPolicyEntryValue = {};
        accessPoliciesArray.push(accessPolicyEntryValue);
        
        if (accessPoliciesItem.tenantId !== null && accessPoliciesItem.tenantId !== undefined) {
          accessPolicyEntryValue['tenantId'] = accessPoliciesItem.tenantId;
        }
        
        if (accessPoliciesItem.objectId !== null && accessPoliciesItem.objectId !== undefined) {
          accessPolicyEntryValue['objectId'] = accessPoliciesItem.objectId;
        }
        
        if (accessPoliciesItem.permissions !== null && accessPoliciesItem.permissions !== undefined) {
          accessPolicyEntryValue['permissions'] = accessPoliciesItem.permissions;
        }
      }
      propertiesValue['accessPolicies'] = accessPoliciesArray;
    }
    
    if (parameters.properties.enabledForDeployment !== null && parameters.properties.enabledForDeployment !== undefined) {
      propertiesValue['enabledForDeployment'] = parameters.properties.enabledForDeployment;
    }
    
    vaultCreateOrUpdateParametersValue['location'] = parameters.location;
    
    if (parameters.tags !== null && parameters.tags !== undefined) {
      var tagsDictionary = {};
      for (var tagsKey in parameters.tags) {
        var tagsValue = parameters.tags[tagsKey];
        tagsDictionary[tagsKey] = tagsValue;
      }
      vaultCreateOrUpdateParametersValue['tags'] = tagsDictionary;
    }
    
    requestContent = JSON.stringify(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200 || statusCode === 201) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var vaultInstance = { tags: {} };
          result.vault = vaultInstance;
          
          var propertiesValue2 = responseDoc['properties'];
          if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
            var propertiesInstance = { accessPolicies: [] };
            vaultInstance.properties = propertiesInstance;
            
            var vaultUriValue = propertiesValue2['vaultUri'];
            if (vaultUriValue !== null && vaultUriValue !== undefined) {
              var vaultUriInstance = vaultUriValue;
              propertiesInstance.vaultUri = vaultUriInstance;
            }
            
            var tenantIdValue = propertiesValue2['tenantId'];
            if (tenantIdValue !== null && tenantIdValue !== undefined) {
              var tenantIdInstance = tenantIdValue;
              propertiesInstance.tenantId = tenantIdInstance;
            }
            
            var skuValue2 = propertiesValue2['sku'];
            if (skuValue2 !== null && skuValue2 !== undefined) {
              var skuInstance = {};
              propertiesInstance.sku = skuInstance;
              
              var familyValue = skuValue2['family'];
              if (familyValue !== null && familyValue !== undefined) {
                var familyInstance = familyValue;
                skuInstance.family = familyInstance;
              }
              
              var nameValue = skuValue2['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                skuInstance.name = nameInstance;
              }
            }
            
            var accessPoliciesArray2 = propertiesValue2['accessPolicies'];
            if (accessPoliciesArray2 !== null && accessPoliciesArray2 !== undefined) {
              for (var loweredIndex2 = 0; loweredIndex2 < accessPoliciesArray2.length; loweredIndex2 = loweredIndex2 + 1) {
                var accessPoliciesValue = accessPoliciesArray2[loweredIndex2];
                var accessPolicyEntryInstance = {};
                propertiesInstance.accessPolicies.push(accessPolicyEntryInstance);
                
                var tenantIdValue2 = accessPoliciesValue['tenantId'];
                if (tenantIdValue2 !== null && tenantIdValue2 !== undefined) {
                  var tenantIdInstance2 = tenantIdValue2;
                  accessPolicyEntryInstance.tenantId = tenantIdInstance2;
                }
                
                var objectIdValue = accessPoliciesValue['objectId'];
                if (objectIdValue !== null && objectIdValue !== undefined) {
                  var objectIdInstance = objectIdValue;
                  accessPolicyEntryInstance.objectId = objectIdInstance;
                }
                
                var permissionsValue = accessPoliciesValue['permissions'];
                if (permissionsValue !== null && permissionsValue !== undefined) {
                  var permissionsInstance = permissionsValue;
                  accessPolicyEntryInstance.permissions = permissionsInstance;
                }
              }
            }
            
            var enabledForDeploymentValue = propertiesValue2['enabledForDeployment'];
            if (enabledForDeploymentValue !== null && enabledForDeploymentValue !== undefined) {
              var enabledForDeploymentInstance = enabledForDeploymentValue;
              propertiesInstance.enabledForDeployment = enabledForDeploymentInstance;
            }
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            vaultInstance.id = idInstance;
          }
          
          var nameValue2 = responseDoc['name'];
          if (nameValue2 !== null && nameValue2 !== undefined) {
            var nameInstance2 = nameValue2;
            vaultInstance.name = nameInstance2;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            vaultInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey2 = property;
              var tagsValue2 = tagsSequenceElement[property];
              vaultInstance.tags[tagsKey2] = tagsValue2;
            }
          }
          
          var typeValue = responseDoc['type'];
          if (typeValue !== null && typeValue !== undefined) {
            var typeInstance = typeValue;
            vaultInstance.type = typeInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Creates a new Azure key vault.
   * 
   * @param {string} resourceGroupName The name of the Resource Group to which
   * the server belongs.
   * 
   * @param {string} vaultName
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VaultOperations.prototype.deleteMethod = function(resourceGroupName, vaultName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vaultName === null || vaultName === undefined) {
      return callback(new Error('vaultName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.KeyVault';
    url2 = url2 + '/vaults/';
    url2 = url2 + encodeURIComponent(vaultName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-19-PREVIEW');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Creates a new Azure key vault.
   * 
   * @param {string} resourceGroupName The name of the Resource Group to which
   * the vault belongs.
   * 
   * @param {string} vaultName
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VaultOperations.prototype.get = function(resourceGroupName, vaultName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vaultName === null || vaultName === undefined) {
      return callback(new Error('vaultName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.KeyVault';
    url2 = url2 + '/vaults/';
    url2 = url2 + encodeURIComponent(vaultName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-19-PREVIEW');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var vaultInstance = { tags: {} };
          result.vault = vaultInstance;
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var propertiesInstance = { accessPolicies: [] };
            vaultInstance.properties = propertiesInstance;
            
            var vaultUriValue = propertiesValue['vaultUri'];
            if (vaultUriValue !== null && vaultUriValue !== undefined) {
              var vaultUriInstance = vaultUriValue;
              propertiesInstance.vaultUri = vaultUriInstance;
            }
            
            var tenantIdValue = propertiesValue['tenantId'];
            if (tenantIdValue !== null && tenantIdValue !== undefined) {
              var tenantIdInstance = tenantIdValue;
              propertiesInstance.tenantId = tenantIdInstance;
            }
            
            var skuValue = propertiesValue['sku'];
            if (skuValue !== null && skuValue !== undefined) {
              var skuInstance = {};
              propertiesInstance.sku = skuInstance;
              
              var familyValue = skuValue['family'];
              if (familyValue !== null && familyValue !== undefined) {
                var familyInstance = familyValue;
                skuInstance.family = familyInstance;
              }
              
              var nameValue = skuValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                skuInstance.name = nameInstance;
              }
            }
            
            var accessPoliciesArray = propertiesValue['accessPolicies'];
            if (accessPoliciesArray !== null && accessPoliciesArray !== undefined) {
              for (var loweredIndex1 = 0; loweredIndex1 < accessPoliciesArray.length; loweredIndex1 = loweredIndex1 + 1) {
                var accessPoliciesValue = accessPoliciesArray[loweredIndex1];
                var accessPolicyEntryInstance = {};
                propertiesInstance.accessPolicies.push(accessPolicyEntryInstance);
                
                var tenantIdValue2 = accessPoliciesValue['tenantId'];
                if (tenantIdValue2 !== null && tenantIdValue2 !== undefined) {
                  var tenantIdInstance2 = tenantIdValue2;
                  accessPolicyEntryInstance.tenantId = tenantIdInstance2;
                }
                
                var objectIdValue = accessPoliciesValue['objectId'];
                if (objectIdValue !== null && objectIdValue !== undefined) {
                  var objectIdInstance = objectIdValue;
                  accessPolicyEntryInstance.objectId = objectIdInstance;
                }
                
                var permissionsValue = accessPoliciesValue['permissions'];
                if (permissionsValue !== null && permissionsValue !== undefined) {
                  var permissionsInstance = permissionsValue;
                  accessPolicyEntryInstance.permissions = permissionsInstance;
                }
              }
            }
            
            var enabledForDeploymentValue = propertiesValue['enabledForDeployment'];
            if (enabledForDeploymentValue !== null && enabledForDeploymentValue !== undefined) {
              var enabledForDeploymentInstance = enabledForDeploymentValue;
              propertiesInstance.enabledForDeployment = enabledForDeploymentInstance;
            }
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            vaultInstance.id = idInstance;
          }
          
          var nameValue2 = responseDoc['name'];
          if (nameValue2 !== null && nameValue2 !== undefined) {
            var nameInstance2 = nameValue2;
            vaultInstance.name = nameInstance2;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            vaultInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey = property;
              var tagsValue = tagsSequenceElement[property];
              vaultInstance.tags[tagsKey] = tagsValue;
            }
          }
          
          var typeValue = responseDoc['type'];
          if (typeValue !== null && typeValue !== undefined) {
            var typeInstance = typeValue;
            vaultInstance.type = typeInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Creates a new Azure key vault.
   * 
   * @param {string} resourceGroupName The name of the Resource Group to which
   * the vault belongs.
   * 
   * @param {Number} [top]
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VaultOperations.prototype.list = function(resourceGroupName, top, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/resources';
    var queryParameters = [];
    var odataFilter = [];
    odataFilter.push('resourceType eq \'Microsoft.KeyVault/vaults\' ');
    if (odataFilter.length > 0) {
      queryParameters.push('$filter=' + odataFilter.join(null));
    }
    if (top !== null && top !== undefined) {
      queryParameters.push('$top=' + encodeURIComponent(top.toString()));
    }
    queryParameters.push('api-version=2014-04-01');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { vaults: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var vaultInstance = { tags: {} };
              result.vaults.push(vaultInstance);
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var propertiesInstance = { accessPolicies: [] };
                vaultInstance.properties = propertiesInstance;
                
                var vaultUriValue = propertiesValue['vaultUri'];
                if (vaultUriValue !== null && vaultUriValue !== undefined) {
                  var vaultUriInstance = vaultUriValue;
                  propertiesInstance.vaultUri = vaultUriInstance;
                }
                
                var tenantIdValue = propertiesValue['tenantId'];
                if (tenantIdValue !== null && tenantIdValue !== undefined) {
                  var tenantIdInstance = tenantIdValue;
                  propertiesInstance.tenantId = tenantIdInstance;
                }
                
                var skuValue = propertiesValue['sku'];
                if (skuValue !== null && skuValue !== undefined) {
                  var skuInstance = {};
                  propertiesInstance.sku = skuInstance;
                  
                  var familyValue = skuValue['family'];
                  if (familyValue !== null && familyValue !== undefined) {
                    var familyInstance = familyValue;
                    skuInstance.family = familyInstance;
                  }
                  
                  var nameValue = skuValue['name'];
                  if (nameValue !== null && nameValue !== undefined) {
                    var nameInstance = nameValue;
                    skuInstance.name = nameInstance;
                  }
                }
                
                var accessPoliciesArray = propertiesValue['accessPolicies'];
                if (accessPoliciesArray !== null && accessPoliciesArray !== undefined) {
                  for (var loweredIndex2 = 0; loweredIndex2 < accessPoliciesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                    var accessPoliciesValue = accessPoliciesArray[loweredIndex2];
                    var accessPolicyEntryInstance = {};
                    propertiesInstance.accessPolicies.push(accessPolicyEntryInstance);
                    
                    var tenantIdValue2 = accessPoliciesValue['tenantId'];
                    if (tenantIdValue2 !== null && tenantIdValue2 !== undefined) {
                      var tenantIdInstance2 = tenantIdValue2;
                      accessPolicyEntryInstance.tenantId = tenantIdInstance2;
                    }
                    
                    var objectIdValue = accessPoliciesValue['objectId'];
                    if (objectIdValue !== null && objectIdValue !== undefined) {
                      var objectIdInstance = objectIdValue;
                      accessPolicyEntryInstance.objectId = objectIdInstance;
                    }
                    
                    var permissionsValue = accessPoliciesValue['permissions'];
                    if (permissionsValue !== null && permissionsValue !== undefined) {
                      var permissionsInstance = permissionsValue;
                      accessPolicyEntryInstance.permissions = permissionsInstance;
                    }
                  }
                }
                
                var enabledForDeploymentValue = propertiesValue['enabledForDeployment'];
                if (enabledForDeploymentValue !== null && enabledForDeploymentValue !== undefined) {
                  var enabledForDeploymentInstance = enabledForDeploymentValue;
                  propertiesInstance.enabledForDeployment = enabledForDeploymentInstance;
                }
              }
              
              var idValue = valueValue['id'];
              if (idValue !== null && idValue !== undefined) {
                var idInstance = idValue;
                vaultInstance.id = idInstance;
              }
              
              var nameValue2 = valueValue['name'];
              if (nameValue2 !== null && nameValue2 !== undefined) {
                var nameInstance2 = nameValue2;
                vaultInstance.name = nameInstance2;
              }
              
              var locationValue = valueValue['location'];
              if (locationValue !== null && locationValue !== undefined) {
                var locationInstance = locationValue;
                vaultInstance.location = locationInstance;
              }
              
              var tagsSequenceElement = valueValue['tags'];
              if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                for (var property in tagsSequenceElement) {
                  var tagsKey = property;
                  var tagsValue = tagsSequenceElement[property];
                  vaultInstance.tags[tagsKey] = tagsValue;
                }
              }
              
              var typeValue = valueValue['type'];
              if (typeValue !== null && typeValue !== undefined) {
                var typeInstance = typeValue;
                vaultInstance.type = typeInstance;
              }
            }
          }
          
          var nextLinkValue = responseDoc['nextLink'];
          if (nextLinkValue !== null && nextLinkValue !== undefined) {
            var nextLinkInstance = nextLinkValue;
            result.nextLink = nextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Creates a new Azure key vault.
   * 
   * @param {string} nextLink Next link.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VaultOperations.prototype.listNext = function(nextLink, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (nextLink === null || nextLink === undefined) {
      return callback(new Error('nextLink cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + nextLink;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { vaults: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var vaultInstance = { tags: {} };
              result.vaults.push(vaultInstance);
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var propertiesInstance = { accessPolicies: [] };
                vaultInstance.properties = propertiesInstance;
                
                var vaultUriValue = propertiesValue['vaultUri'];
                if (vaultUriValue !== null && vaultUriValue !== undefined) {
                  var vaultUriInstance = vaultUriValue;
                  propertiesInstance.vaultUri = vaultUriInstance;
                }
                
                var tenantIdValue = propertiesValue['tenantId'];
                if (tenantIdValue !== null && tenantIdValue !== undefined) {
                  var tenantIdInstance = tenantIdValue;
                  propertiesInstance.tenantId = tenantIdInstance;
                }
                
                var skuValue = propertiesValue['sku'];
                if (skuValue !== null && skuValue !== undefined) {
                  var skuInstance = {};
                  propertiesInstance.sku = skuInstance;
                  
                  var familyValue = skuValue['family'];
                  if (familyValue !== null && familyValue !== undefined) {
                    var familyInstance = familyValue;
                    skuInstance.family = familyInstance;
                  }
                  
                  var nameValue = skuValue['name'];
                  if (nameValue !== null && nameValue !== undefined) {
                    var nameInstance = nameValue;
                    skuInstance.name = nameInstance;
                  }
                }
                
                var accessPoliciesArray = propertiesValue['accessPolicies'];
                if (accessPoliciesArray !== null && accessPoliciesArray !== undefined) {
                  for (var loweredIndex2 = 0; loweredIndex2 < accessPoliciesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                    var accessPoliciesValue = accessPoliciesArray[loweredIndex2];
                    var accessPolicyEntryInstance = {};
                    propertiesInstance.accessPolicies.push(accessPolicyEntryInstance);
                    
                    var tenantIdValue2 = accessPoliciesValue['tenantId'];
                    if (tenantIdValue2 !== null && tenantIdValue2 !== undefined) {
                      var tenantIdInstance2 = tenantIdValue2;
                      accessPolicyEntryInstance.tenantId = tenantIdInstance2;
                    }
                    
                    var objectIdValue = accessPoliciesValue['objectId'];
                    if (objectIdValue !== null && objectIdValue !== undefined) {
                      var objectIdInstance = objectIdValue;
                      accessPolicyEntryInstance.objectId = objectIdInstance;
                    }
                    
                    var permissionsValue = accessPoliciesValue['permissions'];
                    if (permissionsValue !== null && permissionsValue !== undefined) {
                      var permissionsInstance = permissionsValue;
                      accessPolicyEntryInstance.permissions = permissionsInstance;
                    }
                  }
                }
                
                var enabledForDeploymentValue = propertiesValue['enabledForDeployment'];
                if (enabledForDeploymentValue !== null && enabledForDeploymentValue !== undefined) {
                  var enabledForDeploymentInstance = enabledForDeploymentValue;
                  propertiesInstance.enabledForDeployment = enabledForDeploymentInstance;
                }
              }
              
              var idValue = valueValue['id'];
              if (idValue !== null && idValue !== undefined) {
                var idInstance = idValue;
                vaultInstance.id = idInstance;
              }
              
              var nameValue2 = valueValue['name'];
              if (nameValue2 !== null && nameValue2 !== undefined) {
                var nameInstance2 = nameValue2;
                vaultInstance.name = nameInstance2;
              }
              
              var locationValue = valueValue['location'];
              if (locationValue !== null && locationValue !== undefined) {
                var locationInstance = locationValue;
                vaultInstance.location = locationInstance;
              }
              
              var tagsSequenceElement = valueValue['tags'];
              if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                for (var property in tagsSequenceElement) {
                  var tagsKey = property;
                  var tagsValue = tagsSequenceElement[property];
                  vaultInstance.tags[tagsKey] = tagsValue;
                }
              }
              
              var typeValue = valueValue['type'];
              if (typeValue !== null && typeValue !== undefined) {
                var typeInstance = typeValue;
                vaultInstance.type = typeInstance;
              }
            }
          }
          
          var nextLinkValue = responseDoc['nextLink'];
          if (nextLinkValue !== null && nextLinkValue !== undefined) {
            var nextLinkInstance = nextLinkValue;
            result.nextLink = nextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return VaultOperations;
})();
