/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

'use strict';

var util = require('util');
var _ = require('underscore');
var azureCommon = require('azure-common');
var Service = azureCommon.Service;
var WebResource = azureCommon.WebResource;

var ComputeManagementClient = ( /** @lends ComputeManagementClient */ function() {
  /**
   * @class
   * The Compute Management Client.
   * Initializes a new instance of the ComputeManagementClient class.
   * @constructor
   * 
   * @param {SubscriptionCloudCredentials} credentials Gets subscription
   * credentials which uniquely identify Microsoft Azure subscription. The
   * subscription ID forms part of the URI for every service call.
   * 
   * @param {string} [credentials.subscriptionId]
   * 
   * @param {string} [baseUri] Gets the URI used as the base for all cloud
   * service requests.
   * 
   * @param {Array} filters
   */
  function ComputeManagementClient(credentials, baseUri, filters) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    ComputeManagementClient['super_'].call(this, credentials, filters);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.azure.com';
    }
    if (this.apiVersion === null || this.apiVersion === undefined) {
      this.apiVersion = '2014-12-01-preview';
    }
    if (this.longRunningOperationInitialTimeout === null || this.longRunningOperationInitialTimeout === undefined) {
      this.longRunningOperationInitialTimeout = -1;
    }
    if (this.longRunningOperationRetryTimeout === null || this.longRunningOperationRetryTimeout === undefined) {
      this.longRunningOperationRetryTimeout = -1;
    }
    /**
     * Provides an instance of the
     * [AvailabilitySetOperations](-AvailabilitySetOperations.html) object.
     * @type {object}
     */
    this.availabilitySets = new AvailabilitySetOperations(this);
    /**
     * Provides an instance of the
     * [VirtualMachineExtensionOperations](-VirtualMachineExtensionOperations.html)
     * object.
     * @type {object}
     */
    this.virtualMachineExtensions = new VirtualMachineExtensionOperations(this);
    /**
     * Provides an instance of the
     * [VirtualMachineOperations](-VirtualMachineOperations.html) object.
     * @type {object}
     */
    this.virtualMachines = new VirtualMachineOperations(this);
    /**
     * Provides an instance of the
     * [VirtualMachineSizeOperations](-VirtualMachineSizeOperations.html)
     * object.
     * @type {object}
     */
    this.virtualMachineSizes = new VirtualMachineSizeOperations(this);
  }
  
  util.inherits(ComputeManagementClient, Service);
  
  /**
   * The Get Operation Status operation returns the status of the specified
   * operation. After calling an asynchronous operation, you can call Get
   * Operation Status to determine whether the operation has succeeded,
   * failed, or is still in progress.
   * 
   * @param {string} operationStatusLink Location value returned by the Begin
   * operation.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ComputeManagementClient.prototype.getLongRunningOperationStatus = function(operationStatusLink, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (operationStatusLink === null || operationStatusLink === undefined) {
      return callback(new Error('operationStatusLink cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + operationStatusLink;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2014-12-01-preview';
    
    // Send Request
    return this.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200 || statusCode === 202) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            result.trackingOperationId = idInstance;
          }
          
          var statusValue = responseDoc['status'];
          if (statusValue !== null && statusValue !== undefined) {
            var statusInstance = statusValue;
            result.status = statusInstance;
          }
          
          var startTimeValue = responseDoc['startTime'];
          if (startTimeValue !== null && startTimeValue !== undefined) {
            var startTimeInstance = startTimeValue;
            result.startTime = startTimeInstance;
          }
          
          var endTimeValue = responseDoc['endTime'];
          if (endTimeValue !== null && endTimeValue !== undefined) {
            var endTimeInstance = endTimeValue;
            result.endTime = endTimeInstance;
          }
          
          var errorValue = responseDoc['error'];
          if (errorValue !== null && errorValue !== undefined) {
            var errorInstance = { details: [] };
            result.error = errorInstance;
            
            var detailsArray = errorValue['details'];
            if (detailsArray !== null && detailsArray !== undefined) {
              for (var loweredIndex1 = 0; loweredIndex1 < detailsArray.length; loweredIndex1 = loweredIndex1 + 1) {
                var detailsValue = detailsArray[loweredIndex1];
                var apiErrorBaseInstance = {};
                errorInstance.details.push(apiErrorBaseInstance);
                
                var codeValue = detailsValue['code'];
                if (codeValue !== null && codeValue !== undefined) {
                  var codeInstance = codeValue;
                  apiErrorBaseInstance.code = codeInstance;
                }
                
                var targetValue = detailsValue['target'];
                if (targetValue !== null && targetValue !== undefined) {
                  var targetInstance = targetValue;
                  apiErrorBaseInstance.target = targetInstance;
                }
                
                var messageValue = detailsValue['message'];
                if (messageValue !== null && messageValue !== undefined) {
                  var messageInstance = messageValue;
                  apiErrorBaseInstance.message = messageInstance;
                }
              }
            }
            
            var innererrorValue = errorValue['innererror'];
            if (innererrorValue !== null && innererrorValue !== undefined) {
              var innererrorInstance = {};
              errorInstance.innerError = innererrorInstance;
              
              var exceptiontypeValue = innererrorValue['exceptiontype'];
              if (exceptiontypeValue !== null && exceptiontypeValue !== undefined) {
                var exceptiontypeInstance = exceptiontypeValue;
                innererrorInstance.exceptionType = exceptiontypeInstance;
              }
              
              var errordetailValue = innererrorValue['errordetail'];
              if (errordetailValue !== null && errordetailValue !== undefined) {
                var errordetailInstance = errordetailValue;
                innererrorInstance.errorDetail = errordetailInstance;
              }
            }
            
            var codeValue2 = errorValue['code'];
            if (codeValue2 !== null && codeValue2 !== undefined) {
              var codeInstance2 = codeValue2;
              errorInstance.code = codeInstance2;
            }
            
            var targetValue2 = errorValue['target'];
            if (targetValue2 !== null && targetValue2 !== undefined) {
              var targetInstance2 = targetValue2;
              errorInstance.target = targetInstance2;
            }
            
            var messageValue2 = errorValue['message'];
            if (messageValue2 !== null && messageValue2 !== undefined) {
              var messageInstance2 = messageValue2;
              errorInstance.message = messageInstance2;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return ComputeManagementClient;
})();
exports.ComputeManagementClient = ComputeManagementClient;

var AvailabilitySetOperations = ( /** @lends AvailabilitySetOperations */ function() {
  /**
   * @class
   * Operations for managing the availability sets in compute management.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ComputeManagementClient] {@link
   * ComputeManagementClient~ComputeManagementClient}.
   * See [availabilitySets] {@link
   * ComputeManagementClient~ComputeManagementClient#availabilitySets}.
   * Initializes a new instance of the AvailabilitySetOperations class.
   * @constructor
   * 
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  function AvailabilitySetOperations(client) {
    this.client = client;
  }
  
  /**
   * The operation to create the availability set.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {AvailabilitySet} parameters Parameters supplied to the Create
   * Availability Set operation.
   * 
   * @param {number} [parameters.platformUpdateDomainCount] Gets or sets Update
   * Domain count.
   * 
   * @param {number} [parameters.platformFaultDomainCount] Gets or sets Fault
   * Domain count.
   * 
   * @param {array} [parameters.virtualMachinesReferences] Gets or sets a list
   * containing reference to all Virtual Machines  created under this
   * Availability Set.
   * 
   * @param {array} [parameters.statuses] Gets or sets the resource status
   * information.
   * 
   * @param {string} [parameters.id] Gets or sets the ID of the resource.
   * 
   * @param {string} [parameters.name] Gets or sets the name of the resource.
   * 
   * @param {string} [parameters.type] Gets or sets the type of the resource.
   * 
   * @param {string} parameters.location Gets or sets the location of the
   * resource.
   * 
   * @param {object} [parameters.tags] Gets or sets the tags attached to the
   * resource.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  AvailabilitySetOperations.prototype.createOrUpdate = function(resourceGroupName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.location === null || parameters.location === undefined) {
      return callback(new Error('parameters.location cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/availabilitySets/';
    if (parameters.name !== null && parameters.name !== undefined) {
      url2 = url2 + encodeURIComponent(parameters.name);
    }
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = null;
    
    var availabilitySetJsonValue = {};
    requestDoc = availabilitySetJsonValue;
    
    var propertiesValue = {};
    availabilitySetJsonValue['properties'] = propertiesValue;
    
    if (parameters.platformUpdateDomainCount !== null && parameters.platformUpdateDomainCount !== undefined) {
      propertiesValue['platformUpdateDomainCount'] = parameters.platformUpdateDomainCount;
    }
    
    if (parameters.platformFaultDomainCount !== null && parameters.platformFaultDomainCount !== undefined) {
      propertiesValue['platformFaultDomainCount'] = parameters.platformFaultDomainCount;
    }
    
    if (parameters.virtualMachinesReferences !== null && parameters.virtualMachinesReferences !== undefined) {
      var virtualMachinesArray = [];
      for (var loweredIndex1 = 0; loweredIndex1 < parameters.virtualMachinesReferences.length; loweredIndex1 = loweredIndex1 + 1) {
        var virtualMachinesItem = parameters.virtualMachinesReferences[loweredIndex1];
        var virtualMachineReferenceValue = {};
        virtualMachinesArray.push(virtualMachineReferenceValue);
        
        if (virtualMachinesItem.referenceUri !== null && virtualMachinesItem.referenceUri !== undefined) {
          virtualMachineReferenceValue['id'] = virtualMachinesItem.referenceUri;
        }
      }
      propertiesValue['virtualMachines'] = virtualMachinesArray;
    }
    
    if (parameters.statuses !== null && parameters.statuses !== undefined) {
      var statusesArray = [];
      for (var loweredIndex2 = 0; loweredIndex2 < parameters.statuses.length; loweredIndex2 = loweredIndex2 + 1) {
        var statusesItem = parameters.statuses[loweredIndex2];
        
        if (statusesItem.code !== null && statusesItem.code !== undefined) {
          requestDoc = statusesItem.code;
        }
        
        if (statusesItem.level !== null && statusesItem.level !== undefined) {
          requestDoc = statusesItem.level;
        }
        
        if (statusesItem.displayStatus !== null && statusesItem.displayStatus !== undefined) {
          requestDoc = statusesItem.displayStatus;
        }
        
        if (statusesItem.message !== null && statusesItem.message !== undefined) {
          requestDoc = statusesItem.message;
        }
        
        if (statusesItem.time !== null && statusesItem.time !== undefined) {
          requestDoc = statusesItem.time;
        }
      }
      propertiesValue['statuses'] = statusesArray;
    }
    
    if (parameters.id !== null && parameters.id !== undefined) {
      availabilitySetJsonValue['id'] = parameters.id;
    }
    
    if (parameters.name !== null && parameters.name !== undefined) {
      availabilitySetJsonValue['name'] = parameters.name;
    }
    
    if (parameters.type !== null && parameters.type !== undefined) {
      availabilitySetJsonValue['type'] = parameters.type;
    }
    
    availabilitySetJsonValue['location'] = parameters.location;
    
    if (parameters.tags !== null && parameters.tags !== undefined) {
      var tagsDictionary = {};
      for (var tagsKey in parameters.tags) {
        var tagsValue = parameters.tags[tagsKey];
        tagsDictionary[tagsKey] = tagsValue;
      }
      availabilitySetJsonValue['tags'] = tagsDictionary;
    }
    
    requestContent = JSON.stringify(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var availabilitySetInstance = { statuses: [], tags: {}, virtualMachinesReferences: [] };
          result.availabilitySet = availabilitySetInstance;
          
          var propertiesValue2 = responseDoc['properties'];
          if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
            var platformUpdateDomainCountValue = propertiesValue2['platformUpdateDomainCount'];
            if (platformUpdateDomainCountValue !== null && platformUpdateDomainCountValue !== undefined) {
              var platformUpdateDomainCountInstance = platformUpdateDomainCountValue;
              availabilitySetInstance.platformUpdateDomainCount = platformUpdateDomainCountInstance;
            }
            
            var platformFaultDomainCountValue = propertiesValue2['platformFaultDomainCount'];
            if (platformFaultDomainCountValue !== null && platformFaultDomainCountValue !== undefined) {
              var platformFaultDomainCountInstance = platformFaultDomainCountValue;
              availabilitySetInstance.platformFaultDomainCount = platformFaultDomainCountInstance;
            }
            
            var virtualMachinesArray2 = propertiesValue2['virtualMachines'];
            if (virtualMachinesArray2 !== null && virtualMachinesArray2 !== undefined) {
              for (var loweredIndex3 = 0; loweredIndex3 < virtualMachinesArray2.length; loweredIndex3 = loweredIndex3 + 1) {
                var virtualMachinesValue = virtualMachinesArray2[loweredIndex3];
                var virtualMachineReferenceInstance = {};
                availabilitySetInstance.virtualMachinesReferences.push(virtualMachineReferenceInstance);
                
                var idValue = virtualMachinesValue['id'];
                if (idValue !== null && idValue !== undefined) {
                  var idInstance = idValue;
                  virtualMachineReferenceInstance.referenceUri = idInstance;
                }
              }
            }
            
            var statusesArray2 = propertiesValue2['statuses'];
            if (statusesArray2 !== null && statusesArray2 !== undefined) {
              for (var loweredIndex4 = 0; loweredIndex4 < statusesArray2.length; loweredIndex4 = loweredIndex4 + 1) {
                var statusesValue = statusesArray2[loweredIndex4];
                var instanceViewStatusInstance = {};
                availabilitySetInstance.statuses.push(instanceViewStatusInstance);
                
                var codeValue = statusesValue['code'];
                if (codeValue !== null && codeValue !== undefined) {
                  var codeInstance = codeValue;
                  instanceViewStatusInstance.code = codeInstance;
                }
                
                var levelValue = statusesValue['level'];
                if (levelValue !== null && levelValue !== undefined) {
                  var levelInstance = levelValue;
                  instanceViewStatusInstance.level = levelInstance;
                }
                
                var displayStatusValue = statusesValue['displayStatus'];
                if (displayStatusValue !== null && displayStatusValue !== undefined) {
                  var displayStatusInstance = displayStatusValue;
                  instanceViewStatusInstance.displayStatus = displayStatusInstance;
                }
                
                var messageValue = statusesValue['message'];
                if (messageValue !== null && messageValue !== undefined) {
                  var messageInstance = messageValue;
                  instanceViewStatusInstance.message = messageInstance;
                }
                
                var timeValue = statusesValue['time'];
                if (timeValue !== null && timeValue !== undefined) {
                  var timeInstance = timeValue;
                  instanceViewStatusInstance.time = timeInstance;
                }
              }
            }
          }
          
          var idValue2 = responseDoc['id'];
          if (idValue2 !== null && idValue2 !== undefined) {
            var idInstance2 = idValue2;
            availabilitySetInstance.id = idInstance2;
          }
          
          var nameValue = responseDoc['name'];
          if (nameValue !== null && nameValue !== undefined) {
            var nameInstance = nameValue;
            availabilitySetInstance.name = nameInstance;
          }
          
          var typeValue = responseDoc['type'];
          if (typeValue !== null && typeValue !== undefined) {
            var typeInstance = typeValue;
            availabilitySetInstance.type = typeInstance;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            availabilitySetInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey2 = property;
              var tagsValue2 = tagsSequenceElement[property];
              availabilitySetInstance.tags[tagsKey2] = tagsValue2;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to delete the availability set.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} availabilitySetName The name of the availability set.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  AvailabilitySetOperations.prototype.deleteMethod = function(resourceGroupName, availabilitySetName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (availabilitySetName === null || availabilitySetName === undefined) {
      return callback(new Error('availabilitySetName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/availabilitySets/';
    url2 = url2 + encodeURIComponent(availabilitySetName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to get the availability set.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} availabilitySetName The name of the availability set.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  AvailabilitySetOperations.prototype.get = function(resourceGroupName, availabilitySetName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (availabilitySetName === null || availabilitySetName === undefined) {
      return callback(new Error('availabilitySetName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/availabilitySets/';
    url2 = url2 + encodeURIComponent(availabilitySetName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var availabilitySetInstance = { statuses: [], tags: {}, virtualMachinesReferences: [] };
          result.availabilitySet = availabilitySetInstance;
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var platformUpdateDomainCountValue = propertiesValue['platformUpdateDomainCount'];
            if (platformUpdateDomainCountValue !== null && platformUpdateDomainCountValue !== undefined) {
              var platformUpdateDomainCountInstance = platformUpdateDomainCountValue;
              availabilitySetInstance.platformUpdateDomainCount = platformUpdateDomainCountInstance;
            }
            
            var platformFaultDomainCountValue = propertiesValue['platformFaultDomainCount'];
            if (platformFaultDomainCountValue !== null && platformFaultDomainCountValue !== undefined) {
              var platformFaultDomainCountInstance = platformFaultDomainCountValue;
              availabilitySetInstance.platformFaultDomainCount = platformFaultDomainCountInstance;
            }
            
            var virtualMachinesArray = propertiesValue['virtualMachines'];
            if (virtualMachinesArray !== null && virtualMachinesArray !== undefined) {
              for (var loweredIndex1 = 0; loweredIndex1 < virtualMachinesArray.length; loweredIndex1 = loweredIndex1 + 1) {
                var virtualMachinesValue = virtualMachinesArray[loweredIndex1];
                var virtualMachineReferenceInstance = {};
                availabilitySetInstance.virtualMachinesReferences.push(virtualMachineReferenceInstance);
                
                var idValue = virtualMachinesValue['id'];
                if (idValue !== null && idValue !== undefined) {
                  var idInstance = idValue;
                  virtualMachineReferenceInstance.referenceUri = idInstance;
                }
              }
            }
            
            var statusesArray = propertiesValue['statuses'];
            if (statusesArray !== null && statusesArray !== undefined) {
              for (var loweredIndex2 = 0; loweredIndex2 < statusesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                var statusesValue = statusesArray[loweredIndex2];
                var instanceViewStatusInstance = {};
                availabilitySetInstance.statuses.push(instanceViewStatusInstance);
                
                var codeValue = statusesValue['code'];
                if (codeValue !== null && codeValue !== undefined) {
                  var codeInstance = codeValue;
                  instanceViewStatusInstance.code = codeInstance;
                }
                
                var levelValue = statusesValue['level'];
                if (levelValue !== null && levelValue !== undefined) {
                  var levelInstance = levelValue;
                  instanceViewStatusInstance.level = levelInstance;
                }
                
                var displayStatusValue = statusesValue['displayStatus'];
                if (displayStatusValue !== null && displayStatusValue !== undefined) {
                  var displayStatusInstance = displayStatusValue;
                  instanceViewStatusInstance.displayStatus = displayStatusInstance;
                }
                
                var messageValue = statusesValue['message'];
                if (messageValue !== null && messageValue !== undefined) {
                  var messageInstance = messageValue;
                  instanceViewStatusInstance.message = messageInstance;
                }
                
                var timeValue = statusesValue['time'];
                if (timeValue !== null && timeValue !== undefined) {
                  var timeInstance = timeValue;
                  instanceViewStatusInstance.time = timeInstance;
                }
              }
            }
          }
          
          var idValue2 = responseDoc['id'];
          if (idValue2 !== null && idValue2 !== undefined) {
            var idInstance2 = idValue2;
            availabilitySetInstance.id = idInstance2;
          }
          
          var nameValue = responseDoc['name'];
          if (nameValue !== null && nameValue !== undefined) {
            var nameInstance = nameValue;
            availabilitySetInstance.name = nameInstance;
          }
          
          var typeValue = responseDoc['type'];
          if (typeValue !== null && typeValue !== undefined) {
            var typeInstance = typeValue;
            availabilitySetInstance.type = typeInstance;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            availabilitySetInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey = property;
              var tagsValue = tagsSequenceElement[property];
              availabilitySetInstance.tags[tagsKey] = tagsValue;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to list the availability sets.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  AvailabilitySetOperations.prototype.list = function(resourceGroupName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/availabilitySets';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { availabilitySets: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var availabilitySetJsonInstance = { statuses: [], tags: {}, virtualMachinesReferences: [] };
              result.availabilitySets.push(availabilitySetJsonInstance);
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var platformUpdateDomainCountValue = propertiesValue['platformUpdateDomainCount'];
                if (platformUpdateDomainCountValue !== null && platformUpdateDomainCountValue !== undefined) {
                  var platformUpdateDomainCountInstance = platformUpdateDomainCountValue;
                  availabilitySetJsonInstance.platformUpdateDomainCount = platformUpdateDomainCountInstance;
                }
                
                var platformFaultDomainCountValue = propertiesValue['platformFaultDomainCount'];
                if (platformFaultDomainCountValue !== null && platformFaultDomainCountValue !== undefined) {
                  var platformFaultDomainCountInstance = platformFaultDomainCountValue;
                  availabilitySetJsonInstance.platformFaultDomainCount = platformFaultDomainCountInstance;
                }
                
                var virtualMachinesArray = propertiesValue['virtualMachines'];
                if (virtualMachinesArray !== null && virtualMachinesArray !== undefined) {
                  for (var loweredIndex2 = 0; loweredIndex2 < virtualMachinesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                    var virtualMachinesValue = virtualMachinesArray[loweredIndex2];
                    var virtualMachineReferenceInstance = {};
                    availabilitySetJsonInstance.virtualMachinesReferences.push(virtualMachineReferenceInstance);
                    
                    var idValue = virtualMachinesValue['id'];
                    if (idValue !== null && idValue !== undefined) {
                      var idInstance = idValue;
                      virtualMachineReferenceInstance.referenceUri = idInstance;
                    }
                  }
                }
                
                var statusesArray = propertiesValue['statuses'];
                if (statusesArray !== null && statusesArray !== undefined) {
                  for (var loweredIndex3 = 0; loweredIndex3 < statusesArray.length; loweredIndex3 = loweredIndex3 + 1) {
                    var statusesValue = statusesArray[loweredIndex3];
                    var instanceViewStatusInstance = {};
                    availabilitySetJsonInstance.statuses.push(instanceViewStatusInstance);
                    
                    var codeValue = statusesValue['code'];
                    if (codeValue !== null && codeValue !== undefined) {
                      var codeInstance = codeValue;
                      instanceViewStatusInstance.code = codeInstance;
                    }
                    
                    var levelValue = statusesValue['level'];
                    if (levelValue !== null && levelValue !== undefined) {
                      var levelInstance = levelValue;
                      instanceViewStatusInstance.level = levelInstance;
                    }
                    
                    var displayStatusValue = statusesValue['displayStatus'];
                    if (displayStatusValue !== null && displayStatusValue !== undefined) {
                      var displayStatusInstance = displayStatusValue;
                      instanceViewStatusInstance.displayStatus = displayStatusInstance;
                    }
                    
                    var messageValue = statusesValue['message'];
                    if (messageValue !== null && messageValue !== undefined) {
                      var messageInstance = messageValue;
                      instanceViewStatusInstance.message = messageInstance;
                    }
                    
                    var timeValue = statusesValue['time'];
                    if (timeValue !== null && timeValue !== undefined) {
                      var timeInstance = timeValue;
                      instanceViewStatusInstance.time = timeInstance;
                    }
                  }
                }
              }
              
              var idValue2 = valueValue['id'];
              if (idValue2 !== null && idValue2 !== undefined) {
                var idInstance2 = idValue2;
                availabilitySetJsonInstance.id = idInstance2;
              }
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                availabilitySetJsonInstance.name = nameInstance;
              }
              
              var typeValue = valueValue['type'];
              if (typeValue !== null && typeValue !== undefined) {
                var typeInstance = typeValue;
                availabilitySetJsonInstance.type = typeInstance;
              }
              
              var locationValue = valueValue['location'];
              if (locationValue !== null && locationValue !== undefined) {
                var locationInstance = locationValue;
                availabilitySetJsonInstance.location = locationInstance;
              }
              
              var tagsSequenceElement = valueValue['tags'];
              if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                for (var property in tagsSequenceElement) {
                  var tagsKey = property;
                  var tagsValue = tagsSequenceElement[property];
                  availabilitySetJsonInstance.tags[tagsKey] = tagsValue;
                }
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Lists virtual-machine-sizes available to be used for an availability set.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} availabilitySetName The name of the availability set.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  AvailabilitySetOperations.prototype.listAvailableSizes = function(resourceGroupName, availabilitySetName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (availabilitySetName === null || availabilitySetName === undefined) {
      return callback(new Error('availabilitySetName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/availabilitySets/';
    url2 = url2 + encodeURIComponent(availabilitySetName);
    url2 = url2 + '/vmSizes';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachineSizes: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineSizeInstance = {};
              result.virtualMachineSizes.push(virtualMachineSizeInstance);
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                virtualMachineSizeInstance.name = nameInstance;
              }
              
              var numberOfCoresValue = valueValue['numberOfCores'];
              if (numberOfCoresValue !== null && numberOfCoresValue !== undefined) {
                var numberOfCoresInstance = numberOfCoresValue;
                virtualMachineSizeInstance.numberOfCores = numberOfCoresInstance;
              }
              
              var osDiskSizeInMBValue = valueValue['osDiskSizeInMB'];
              if (osDiskSizeInMBValue !== null && osDiskSizeInMBValue !== undefined) {
                var osDiskSizeInMBInstance = osDiskSizeInMBValue;
                virtualMachineSizeInstance.oSDiskSizeInMB = osDiskSizeInMBInstance;
              }
              
              var resourceDiskSizeInMBValue = valueValue['resourceDiskSizeInMB'];
              if (resourceDiskSizeInMBValue !== null && resourceDiskSizeInMBValue !== undefined) {
                var resourceDiskSizeInMBInstance = resourceDiskSizeInMBValue;
                virtualMachineSizeInstance.resourceDiskSizeInMB = resourceDiskSizeInMBInstance;
              }
              
              var memoryInMBValue = valueValue['memoryInMB'];
              if (memoryInMBValue !== null && memoryInMBValue !== undefined) {
                var memoryInMBInstance = memoryInMBValue;
                virtualMachineSizeInstance.memoryInMB = memoryInMBInstance;
              }
              
              var maxDataDiskCountValue = valueValue['maxDataDiskCount'];
              if (maxDataDiskCountValue !== null && maxDataDiskCountValue !== undefined) {
                var maxDataDiskCountInstance = maxDataDiskCountValue;
                virtualMachineSizeInstance.maxDataDiskCount = maxDataDiskCountInstance;
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return AvailabilitySetOperations;
})();

var VirtualMachineExtensionOperations = ( /** @lends VirtualMachineExtensionOperations */ function() {
  /**
   * @class
   * Operations for managing the virtual machine extensions in compute
   * management.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ComputeManagementClient] {@link
   * ComputeManagementClient~ComputeManagementClient}.
   * See [virtualMachineExtensions] {@link
   * ComputeManagementClient~ComputeManagementClient#virtualMachineExtensions}.
   * Initializes a new instance of the VirtualMachineExtensionOperations class.
   * @constructor
   * 
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  function VirtualMachineExtensionOperations(client) {
    this.client = client;
  }
  
  /**
   * The operation to create the extension.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine where the extension
   * should be create or updated.
   * 
   * @param {VirtualMachineExtension} extensionParameters Parameters supplied
   * to the Create Virtual Machine Extension operation.
   * 
   * @param {string} [extensionParameters.publisher] Gets or sets the name of
   * the extension handler publisher.
   * 
   * @param {string} [extensionParameters.extensionType] Gets or sets the type
   * of the extension handler.
   * 
   * @param {string} [extensionParameters.typeHandlerVersion] Gets or sets the
   * type version of the extension handler.
   * 
   * @param {string} [extensionParameters.settings] Gets or sets Json formatted
   * public settings for the extension.
   * 
   * @param {string} [extensionParameters.protectedSettings] Gets or sets Json
   * formatted protected settings for the extension.
   * 
   * @param {string} [extensionParameters.provisioningState] Gets or sets the
   * provisioning state, which only appears in the response.
   * 
   * @param {VirtualMachineExtensionInstanceView}
   * [extensionParameters.instanceView] Gets or sets the virtual machine
   * extension instance view.
   * 
   * @param {string} [extensionParameters.instanceView.name] Gets or sets the
   * virtual machine extension name.
   * 
   * @param {string} [extensionParameters.instanceView.extensionType] Gets or
   * sets the full type of the extension handler which includes both publisher
   * and type.
   * 
   * @param {string} [extensionParameters.instanceView.typeHandlerVersion] Gets
   * or sets the type version of the extension handler.
   * 
   * @param {array} [extensionParameters.instanceView.subStatuses] Gets or sets
   * the resource status information.
   * 
   * @param {array} [extensionParameters.instanceView.statuses] Gets or sets
   * the resource status information.
   * 
   * @param {string} [extensionParameters.id] Gets or sets the ID of the
   * resource.
   * 
   * @param {string} [extensionParameters.name] Gets or sets the name of the
   * resource.
   * 
   * @param {string} [extensionParameters.type] Gets or sets the type of the
   * resource.
   * 
   * @param {string} extensionParameters.location Gets or sets the location of
   * the resource.
   * 
   * @param {object} [extensionParameters.tags] Gets or sets the tags attached
   * to the resource.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.beginCreatingOrUpdating = function(resourceGroupName, vmName, extensionParameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    if (extensionParameters === null || extensionParameters === undefined) {
      return callback(new Error('extensionParameters cannot be null.'));
    }
    if (extensionParameters.location === null || extensionParameters.location === undefined) {
      return callback(new Error('extensionParameters.location cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/extensions/';
    if (extensionParameters.name !== null && extensionParameters.name !== undefined) {
      url2 = url2 + encodeURIComponent(extensionParameters.name);
    }
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = null;
    
    var virtualMachineExtensionJsonValue = {};
    requestDoc = virtualMachineExtensionJsonValue;
    
    var propertiesValue = {};
    virtualMachineExtensionJsonValue['properties'] = propertiesValue;
    
    if (extensionParameters.publisher !== null && extensionParameters.publisher !== undefined) {
      propertiesValue['publisher'] = extensionParameters.publisher;
    }
    
    if (extensionParameters.extensionType !== null && extensionParameters.extensionType !== undefined) {
      propertiesValue['type'] = extensionParameters.extensionType;
    }
    
    if (extensionParameters.typeHandlerVersion !== null && extensionParameters.typeHandlerVersion !== undefined) {
      propertiesValue['typeHandlerVersion'] = extensionParameters.typeHandlerVersion;
    }
    
    if (extensionParameters.settings !== null && extensionParameters.settings !== undefined) {
      propertiesValue['settings'] = extensionParameters.settings;
    }
    
    if (extensionParameters.protectedSettings !== null && extensionParameters.protectedSettings !== undefined) {
      propertiesValue['protectedSettings'] = extensionParameters.protectedSettings;
    }
    
    if (extensionParameters.provisioningState !== null && extensionParameters.provisioningState !== undefined) {
      propertiesValue['provisioningState'] = extensionParameters.provisioningState;
    }
    
    if (extensionParameters.instanceView !== null && extensionParameters.instanceView !== undefined) {
      var instanceViewValue = {};
      propertiesValue['instanceView'] = instanceViewValue;
      
      if (extensionParameters.instanceView.name !== null && extensionParameters.instanceView.name !== undefined) {
        instanceViewValue['name'] = extensionParameters.instanceView.name;
      }
      
      if (extensionParameters.instanceView.extensionType !== null && extensionParameters.instanceView.extensionType !== undefined) {
        instanceViewValue['type'] = extensionParameters.instanceView.extensionType;
      }
      
      if (extensionParameters.instanceView.typeHandlerVersion !== null && extensionParameters.instanceView.typeHandlerVersion !== undefined) {
        instanceViewValue['typeHandlerVersion'] = extensionParameters.instanceView.typeHandlerVersion;
      }
      
      if (extensionParameters.instanceView.subStatuses !== null && extensionParameters.instanceView.subStatuses !== undefined) {
        var substatusesArray = [];
        for (var loweredIndex1 = 0; loweredIndex1 < extensionParameters.instanceView.subStatuses.length; loweredIndex1 = loweredIndex1 + 1) {
          var substatusesItem = extensionParameters.instanceView.subStatuses[loweredIndex1];
          
          if (substatusesItem.code !== null && substatusesItem.code !== undefined) {
            requestDoc = substatusesItem.code;
          }
          
          if (substatusesItem.level !== null && substatusesItem.level !== undefined) {
            requestDoc = substatusesItem.level;
          }
          
          if (substatusesItem.displayStatus !== null && substatusesItem.displayStatus !== undefined) {
            requestDoc = substatusesItem.displayStatus;
          }
          
          if (substatusesItem.message !== null && substatusesItem.message !== undefined) {
            requestDoc = substatusesItem.message;
          }
          
          if (substatusesItem.time !== null && substatusesItem.time !== undefined) {
            requestDoc = substatusesItem.time;
          }
        }
        instanceViewValue['substatuses'] = substatusesArray;
      }
      
      if (extensionParameters.instanceView.statuses !== null && extensionParameters.instanceView.statuses !== undefined) {
        var statusesArray = [];
        for (var loweredIndex2 = 0; loweredIndex2 < extensionParameters.instanceView.statuses.length; loweredIndex2 = loweredIndex2 + 1) {
          var statusesItem = extensionParameters.instanceView.statuses[loweredIndex2];
          
          if (statusesItem.code !== null && statusesItem.code !== undefined) {
            requestDoc = statusesItem.code;
          }
          
          if (statusesItem.level !== null && statusesItem.level !== undefined) {
            requestDoc = statusesItem.level;
          }
          
          if (statusesItem.displayStatus !== null && statusesItem.displayStatus !== undefined) {
            requestDoc = statusesItem.displayStatus;
          }
          
          if (statusesItem.message !== null && statusesItem.message !== undefined) {
            requestDoc = statusesItem.message;
          }
          
          if (statusesItem.time !== null && statusesItem.time !== undefined) {
            requestDoc = statusesItem.time;
          }
        }
        instanceViewValue['statuses'] = statusesArray;
      }
    }
    
    if (extensionParameters.id !== null && extensionParameters.id !== undefined) {
      virtualMachineExtensionJsonValue['id'] = extensionParameters.id;
    }
    
    if (extensionParameters.name !== null && extensionParameters.name !== undefined) {
      virtualMachineExtensionJsonValue['name'] = extensionParameters.name;
    }
    
    if (extensionParameters.type !== null && extensionParameters.type !== undefined) {
      virtualMachineExtensionJsonValue['type'] = extensionParameters.type;
    }
    
    virtualMachineExtensionJsonValue['location'] = extensionParameters.location;
    
    if (extensionParameters.tags !== null && extensionParameters.tags !== undefined) {
      var tagsDictionary = {};
      for (var tagsKey in extensionParameters.tags) {
        var tagsValue = extensionParameters.tags[tagsKey];
        tagsDictionary[tagsKey] = tagsValue;
      }
      virtualMachineExtensionJsonValue['tags'] = tagsDictionary;
    }
    
    requestContent = JSON.stringify(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200 || statusCode === 201) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var virtualMachineExtensionInstance = { tags: {} };
          result.virtualMachineExtension = virtualMachineExtensionInstance;
          
          var propertiesValue2 = responseDoc['properties'];
          if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
            var publisherValue = propertiesValue2['publisher'];
            if (publisherValue !== null && publisherValue !== undefined) {
              var publisherInstance = publisherValue;
              virtualMachineExtensionInstance.publisher = publisherInstance;
            }
            
            var typeValue = propertiesValue2['type'];
            if (typeValue !== null && typeValue !== undefined) {
              var typeInstance = typeValue;
              virtualMachineExtensionInstance.extensionType = typeInstance;
            }
            
            var typeHandlerVersionValue = propertiesValue2['typeHandlerVersion'];
            if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
              var typeHandlerVersionInstance = typeHandlerVersionValue;
              virtualMachineExtensionInstance.typeHandlerVersion = typeHandlerVersionInstance;
            }
            
            var settingsValue = propertiesValue2['settings'];
            if (settingsValue !== null && settingsValue !== undefined) {
              var settingsInstance = settingsValue;
              virtualMachineExtensionInstance.settings = settingsInstance;
            }
            
            var protectedSettingsValue = propertiesValue2['protectedSettings'];
            if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
              var protectedSettingsInstance = protectedSettingsValue;
              virtualMachineExtensionInstance.protectedSettings = protectedSettingsInstance;
            }
            
            var provisioningStateValue = propertiesValue2['provisioningState'];
            if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
              var provisioningStateInstance = provisioningStateValue;
              virtualMachineExtensionInstance.provisioningState = provisioningStateInstance;
            }
            
            var instanceViewValue2 = propertiesValue2['instanceView'];
            if (instanceViewValue2 !== null && instanceViewValue2 !== undefined) {
              var instanceViewInstance = { statuses: [], subStatuses: [] };
              virtualMachineExtensionInstance.instanceView = instanceViewInstance;
              
              var nameValue = instanceViewValue2['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                instanceViewInstance.name = nameInstance;
              }
              
              var typeValue2 = instanceViewValue2['type'];
              if (typeValue2 !== null && typeValue2 !== undefined) {
                var typeInstance2 = typeValue2;
                instanceViewInstance.extensionType = typeInstance2;
              }
              
              var typeHandlerVersionValue2 = instanceViewValue2['typeHandlerVersion'];
              if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
              }
              
              var substatusesArray2 = instanceViewValue2['substatuses'];
              if (substatusesArray2 !== null && substatusesArray2 !== undefined) {
                for (var loweredIndex3 = 0; loweredIndex3 < substatusesArray2.length; loweredIndex3 = loweredIndex3 + 1) {
                  var substatusesValue = substatusesArray2[loweredIndex3];
                  var instanceViewStatusInstance = {};
                  instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                  
                  var codeValue = substatusesValue['code'];
                  if (codeValue !== null && codeValue !== undefined) {
                    var codeInstance = codeValue;
                    instanceViewStatusInstance.code = codeInstance;
                  }
                  
                  var levelValue = substatusesValue['level'];
                  if (levelValue !== null && levelValue !== undefined) {
                    var levelInstance = levelValue;
                    instanceViewStatusInstance.level = levelInstance;
                  }
                  
                  var displayStatusValue = substatusesValue['displayStatus'];
                  if (displayStatusValue !== null && displayStatusValue !== undefined) {
                    var displayStatusInstance = displayStatusValue;
                    instanceViewStatusInstance.displayStatus = displayStatusInstance;
                  }
                  
                  var messageValue = substatusesValue['message'];
                  if (messageValue !== null && messageValue !== undefined) {
                    var messageInstance = messageValue;
                    instanceViewStatusInstance.message = messageInstance;
                  }
                  
                  var timeValue = substatusesValue['time'];
                  if (timeValue !== null && timeValue !== undefined) {
                    var timeInstance = timeValue;
                    instanceViewStatusInstance.time = timeInstance;
                  }
                }
              }
              
              var statusesArray2 = instanceViewValue2['statuses'];
              if (statusesArray2 !== null && statusesArray2 !== undefined) {
                for (var loweredIndex4 = 0; loweredIndex4 < statusesArray2.length; loweredIndex4 = loweredIndex4 + 1) {
                  var statusesValue = statusesArray2[loweredIndex4];
                  var instanceViewStatusInstance2 = {};
                  instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                  
                  var codeValue2 = statusesValue['code'];
                  if (codeValue2 !== null && codeValue2 !== undefined) {
                    var codeInstance2 = codeValue2;
                    instanceViewStatusInstance2.code = codeInstance2;
                  }
                  
                  var levelValue2 = statusesValue['level'];
                  if (levelValue2 !== null && levelValue2 !== undefined) {
                    var levelInstance2 = levelValue2;
                    instanceViewStatusInstance2.level = levelInstance2;
                  }
                  
                  var displayStatusValue2 = statusesValue['displayStatus'];
                  if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                    var displayStatusInstance2 = displayStatusValue2;
                    instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                  }
                  
                  var messageValue2 = statusesValue['message'];
                  if (messageValue2 !== null && messageValue2 !== undefined) {
                    var messageInstance2 = messageValue2;
                    instanceViewStatusInstance2.message = messageInstance2;
                  }
                  
                  var timeValue2 = statusesValue['time'];
                  if (timeValue2 !== null && timeValue2 !== undefined) {
                    var timeInstance2 = timeValue2;
                    instanceViewStatusInstance2.time = timeInstance2;
                  }
                }
              }
            }
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            virtualMachineExtensionInstance.id = idInstance;
          }
          
          var nameValue2 = responseDoc['name'];
          if (nameValue2 !== null && nameValue2 !== undefined) {
            var nameInstance2 = nameValue2;
            virtualMachineExtensionInstance.name = nameInstance2;
          }
          
          var typeValue3 = responseDoc['type'];
          if (typeValue3 !== null && typeValue3 !== undefined) {
            var typeInstance3 = typeValue3;
            virtualMachineExtensionInstance.type = typeInstance3;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            virtualMachineExtensionInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey2 = property;
              var tagsValue2 = tagsSequenceElement[property];
              virtualMachineExtensionInstance.tags[tagsKey2] = tagsValue2;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to delete the extension.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine where the extension
   * should be deleted.
   * 
   * @param {string} vmExtensionName The name of the virtual machine extension.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.beginDeleting = function(resourceGroupName, vmName, vmExtensionName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    if (vmExtensionName === null || vmExtensionName === undefined) {
      return callback(new Error('vmExtensionName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/extensions/';
    url2 = url2 + encodeURIComponent(vmExtensionName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine where the extension
   * should be create or updated.
   * 
   * @param {VirtualMachineExtension} extensionParameters Parameters supplied
   * to the Create Virtual Machine Extension operation.
   * 
   * @param {string} [extensionParameters.publisher] Gets or sets the name of
   * the extension handler publisher.
   * 
   * @param {string} [extensionParameters.extensionType] Gets or sets the type
   * of the extension handler.
   * 
   * @param {string} [extensionParameters.typeHandlerVersion] Gets or sets the
   * type version of the extension handler.
   * 
   * @param {string} [extensionParameters.settings] Gets or sets Json formatted
   * public settings for the extension.
   * 
   * @param {string} [extensionParameters.protectedSettings] Gets or sets Json
   * formatted protected settings for the extension.
   * 
   * @param {string} [extensionParameters.provisioningState] Gets or sets the
   * provisioning state, which only appears in the response.
   * 
   * @param {VirtualMachineExtensionInstanceView}
   * [extensionParameters.instanceView] Gets or sets the virtual machine
   * extension instance view.
   * 
   * @param {string} [extensionParameters.instanceView.name] Gets or sets the
   * virtual machine extension name.
   * 
   * @param {string} [extensionParameters.instanceView.extensionType] Gets or
   * sets the full type of the extension handler which includes both publisher
   * and type.
   * 
   * @param {string} [extensionParameters.instanceView.typeHandlerVersion] Gets
   * or sets the type version of the extension handler.
   * 
   * @param {array} [extensionParameters.instanceView.subStatuses] Gets or sets
   * the resource status information.
   * 
   * @param {array} [extensionParameters.instanceView.statuses] Gets or sets
   * the resource status information.
   * 
   * @param {string} [extensionParameters.id] Gets or sets the ID of the
   * resource.
   * 
   * @param {string} [extensionParameters.name] Gets or sets the name of the
   * resource.
   * 
   * @param {string} [extensionParameters.type] Gets or sets the type of the
   * resource.
   * 
   * @param {string} extensionParameters.location Gets or sets the location of
   * the resource.
   * 
   * @param {object} [extensionParameters.tags] Gets or sets the tags attached
   * to the resource.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.createOrUpdate = function(resourceGroupName, vmName, extensionParameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachineExtensions.beginCreatingOrUpdating(resourceGroupName, vmName, extensionParameters, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine where the extension
   * should be deleted.
   * 
   * @param {string} vmExtensionName The name of the virtual machine extension.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.deleteMethod = function(resourceGroupName, vmName, vmExtensionName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachineExtensions.beginDeleting(resourceGroupName, vmName, vmExtensionName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * The operation to get the extension.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine containing the
   * extension.
   * 
   * @param {string} vmExtensionName The name of the virtual machine extension.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.get = function(resourceGroupName, vmName, vmExtensionName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    if (vmExtensionName === null || vmExtensionName === undefined) {
      return callback(new Error('vmExtensionName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/extensions/';
    url2 = url2 + encodeURIComponent(vmExtensionName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var virtualMachineExtensionInstance = { tags: {} };
          result.virtualMachineExtension = virtualMachineExtensionInstance;
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var publisherValue = propertiesValue['publisher'];
            if (publisherValue !== null && publisherValue !== undefined) {
              var publisherInstance = publisherValue;
              virtualMachineExtensionInstance.publisher = publisherInstance;
            }
            
            var typeValue = propertiesValue['type'];
            if (typeValue !== null && typeValue !== undefined) {
              var typeInstance = typeValue;
              virtualMachineExtensionInstance.extensionType = typeInstance;
            }
            
            var typeHandlerVersionValue = propertiesValue['typeHandlerVersion'];
            if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
              var typeHandlerVersionInstance = typeHandlerVersionValue;
              virtualMachineExtensionInstance.typeHandlerVersion = typeHandlerVersionInstance;
            }
            
            var settingsValue = propertiesValue['settings'];
            if (settingsValue !== null && settingsValue !== undefined) {
              var settingsInstance = settingsValue;
              virtualMachineExtensionInstance.settings = settingsInstance;
            }
            
            var protectedSettingsValue = propertiesValue['protectedSettings'];
            if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
              var protectedSettingsInstance = protectedSettingsValue;
              virtualMachineExtensionInstance.protectedSettings = protectedSettingsInstance;
            }
            
            var provisioningStateValue = propertiesValue['provisioningState'];
            if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
              var provisioningStateInstance = provisioningStateValue;
              virtualMachineExtensionInstance.provisioningState = provisioningStateInstance;
            }
            
            var instanceViewValue = propertiesValue['instanceView'];
            if (instanceViewValue !== null && instanceViewValue !== undefined) {
              var instanceViewInstance = { statuses: [], subStatuses: [] };
              virtualMachineExtensionInstance.instanceView = instanceViewInstance;
              
              var nameValue = instanceViewValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                instanceViewInstance.name = nameInstance;
              }
              
              var typeValue2 = instanceViewValue['type'];
              if (typeValue2 !== null && typeValue2 !== undefined) {
                var typeInstance2 = typeValue2;
                instanceViewInstance.extensionType = typeInstance2;
              }
              
              var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
              if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
              }
              
              var substatusesArray = instanceViewValue['substatuses'];
              if (substatusesArray !== null && substatusesArray !== undefined) {
                for (var loweredIndex1 = 0; loweredIndex1 < substatusesArray.length; loweredIndex1 = loweredIndex1 + 1) {
                  var substatusesValue = substatusesArray[loweredIndex1];
                  var instanceViewStatusInstance = {};
                  instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                  
                  var codeValue = substatusesValue['code'];
                  if (codeValue !== null && codeValue !== undefined) {
                    var codeInstance = codeValue;
                    instanceViewStatusInstance.code = codeInstance;
                  }
                  
                  var levelValue = substatusesValue['level'];
                  if (levelValue !== null && levelValue !== undefined) {
                    var levelInstance = levelValue;
                    instanceViewStatusInstance.level = levelInstance;
                  }
                  
                  var displayStatusValue = substatusesValue['displayStatus'];
                  if (displayStatusValue !== null && displayStatusValue !== undefined) {
                    var displayStatusInstance = displayStatusValue;
                    instanceViewStatusInstance.displayStatus = displayStatusInstance;
                  }
                  
                  var messageValue = substatusesValue['message'];
                  if (messageValue !== null && messageValue !== undefined) {
                    var messageInstance = messageValue;
                    instanceViewStatusInstance.message = messageInstance;
                  }
                  
                  var timeValue = substatusesValue['time'];
                  if (timeValue !== null && timeValue !== undefined) {
                    var timeInstance = timeValue;
                    instanceViewStatusInstance.time = timeInstance;
                  }
                }
              }
              
              var statusesArray = instanceViewValue['statuses'];
              if (statusesArray !== null && statusesArray !== undefined) {
                for (var loweredIndex2 = 0; loweredIndex2 < statusesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                  var statusesValue = statusesArray[loweredIndex2];
                  var instanceViewStatusInstance2 = {};
                  instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                  
                  var codeValue2 = statusesValue['code'];
                  if (codeValue2 !== null && codeValue2 !== undefined) {
                    var codeInstance2 = codeValue2;
                    instanceViewStatusInstance2.code = codeInstance2;
                  }
                  
                  var levelValue2 = statusesValue['level'];
                  if (levelValue2 !== null && levelValue2 !== undefined) {
                    var levelInstance2 = levelValue2;
                    instanceViewStatusInstance2.level = levelInstance2;
                  }
                  
                  var displayStatusValue2 = statusesValue['displayStatus'];
                  if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                    var displayStatusInstance2 = displayStatusValue2;
                    instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                  }
                  
                  var messageValue2 = statusesValue['message'];
                  if (messageValue2 !== null && messageValue2 !== undefined) {
                    var messageInstance2 = messageValue2;
                    instanceViewStatusInstance2.message = messageInstance2;
                  }
                  
                  var timeValue2 = statusesValue['time'];
                  if (timeValue2 !== null && timeValue2 !== undefined) {
                    var timeInstance2 = timeValue2;
                    instanceViewStatusInstance2.time = timeInstance2;
                  }
                }
              }
            }
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            virtualMachineExtensionInstance.id = idInstance;
          }
          
          var nameValue2 = responseDoc['name'];
          if (nameValue2 !== null && nameValue2 !== undefined) {
            var nameInstance2 = nameValue2;
            virtualMachineExtensionInstance.name = nameInstance2;
          }
          
          var typeValue3 = responseDoc['type'];
          if (typeValue3 !== null && typeValue3 !== undefined) {
            var typeInstance3 = typeValue3;
            virtualMachineExtensionInstance.type = typeInstance3;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            virtualMachineExtensionInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey = property;
              var tagsValue = tagsSequenceElement[property];
              virtualMachineExtensionInstance.tags[tagsKey] = tagsValue;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to get the instance view of the extension.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine containing the
   * extension.
   * 
   * @param {string} vmExtensionName The name of the virtual machine extension.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineExtensionOperations.prototype.getInstanceView = function(resourceGroupName, vmName, vmExtensionName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    if (vmExtensionName === null || vmExtensionName === undefined) {
      return callback(new Error('vmExtensionName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/extensions/';
    url2 = url2 + encodeURIComponent(vmExtensionName);
    url2 = url2 + '/InstanceView';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var virtualMachineExtensionInstance = { tags: {} };
          result.virtualMachineExtension = virtualMachineExtensionInstance;
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var publisherValue = propertiesValue['publisher'];
            if (publisherValue !== null && publisherValue !== undefined) {
              var publisherInstance = publisherValue;
              virtualMachineExtensionInstance.publisher = publisherInstance;
            }
            
            var typeValue = propertiesValue['type'];
            if (typeValue !== null && typeValue !== undefined) {
              var typeInstance = typeValue;
              virtualMachineExtensionInstance.extensionType = typeInstance;
            }
            
            var typeHandlerVersionValue = propertiesValue['typeHandlerVersion'];
            if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
              var typeHandlerVersionInstance = typeHandlerVersionValue;
              virtualMachineExtensionInstance.typeHandlerVersion = typeHandlerVersionInstance;
            }
            
            var settingsValue = propertiesValue['settings'];
            if (settingsValue !== null && settingsValue !== undefined) {
              var settingsInstance = settingsValue;
              virtualMachineExtensionInstance.settings = settingsInstance;
            }
            
            var protectedSettingsValue = propertiesValue['protectedSettings'];
            if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
              var protectedSettingsInstance = protectedSettingsValue;
              virtualMachineExtensionInstance.protectedSettings = protectedSettingsInstance;
            }
            
            var provisioningStateValue = propertiesValue['provisioningState'];
            if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
              var provisioningStateInstance = provisioningStateValue;
              virtualMachineExtensionInstance.provisioningState = provisioningStateInstance;
            }
            
            var instanceViewValue = propertiesValue['instanceView'];
            if (instanceViewValue !== null && instanceViewValue !== undefined) {
              var instanceViewInstance = { statuses: [], subStatuses: [] };
              virtualMachineExtensionInstance.instanceView = instanceViewInstance;
              
              var nameValue = instanceViewValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                instanceViewInstance.name = nameInstance;
              }
              
              var typeValue2 = instanceViewValue['type'];
              if (typeValue2 !== null && typeValue2 !== undefined) {
                var typeInstance2 = typeValue2;
                instanceViewInstance.extensionType = typeInstance2;
              }
              
              var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
              if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
              }
              
              var substatusesArray = instanceViewValue['substatuses'];
              if (substatusesArray !== null && substatusesArray !== undefined) {
                for (var loweredIndex1 = 0; loweredIndex1 < substatusesArray.length; loweredIndex1 = loweredIndex1 + 1) {
                  var substatusesValue = substatusesArray[loweredIndex1];
                  var instanceViewStatusInstance = {};
                  instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                  
                  var codeValue = substatusesValue['code'];
                  if (codeValue !== null && codeValue !== undefined) {
                    var codeInstance = codeValue;
                    instanceViewStatusInstance.code = codeInstance;
                  }
                  
                  var levelValue = substatusesValue['level'];
                  if (levelValue !== null && levelValue !== undefined) {
                    var levelInstance = levelValue;
                    instanceViewStatusInstance.level = levelInstance;
                  }
                  
                  var displayStatusValue = substatusesValue['displayStatus'];
                  if (displayStatusValue !== null && displayStatusValue !== undefined) {
                    var displayStatusInstance = displayStatusValue;
                    instanceViewStatusInstance.displayStatus = displayStatusInstance;
                  }
                  
                  var messageValue = substatusesValue['message'];
                  if (messageValue !== null && messageValue !== undefined) {
                    var messageInstance = messageValue;
                    instanceViewStatusInstance.message = messageInstance;
                  }
                  
                  var timeValue = substatusesValue['time'];
                  if (timeValue !== null && timeValue !== undefined) {
                    var timeInstance = timeValue;
                    instanceViewStatusInstance.time = timeInstance;
                  }
                }
              }
              
              var statusesArray = instanceViewValue['statuses'];
              if (statusesArray !== null && statusesArray !== undefined) {
                for (var loweredIndex2 = 0; loweredIndex2 < statusesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                  var statusesValue = statusesArray[loweredIndex2];
                  var instanceViewStatusInstance2 = {};
                  instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                  
                  var codeValue2 = statusesValue['code'];
                  if (codeValue2 !== null && codeValue2 !== undefined) {
                    var codeInstance2 = codeValue2;
                    instanceViewStatusInstance2.code = codeInstance2;
                  }
                  
                  var levelValue2 = statusesValue['level'];
                  if (levelValue2 !== null && levelValue2 !== undefined) {
                    var levelInstance2 = levelValue2;
                    instanceViewStatusInstance2.level = levelInstance2;
                  }
                  
                  var displayStatusValue2 = statusesValue['displayStatus'];
                  if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                    var displayStatusInstance2 = displayStatusValue2;
                    instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                  }
                  
                  var messageValue2 = statusesValue['message'];
                  if (messageValue2 !== null && messageValue2 !== undefined) {
                    var messageInstance2 = messageValue2;
                    instanceViewStatusInstance2.message = messageInstance2;
                  }
                  
                  var timeValue2 = statusesValue['time'];
                  if (timeValue2 !== null && timeValue2 !== undefined) {
                    var timeInstance2 = timeValue2;
                    instanceViewStatusInstance2.time = timeInstance2;
                  }
                }
              }
            }
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            virtualMachineExtensionInstance.id = idInstance;
          }
          
          var nameValue2 = responseDoc['name'];
          if (nameValue2 !== null && nameValue2 !== undefined) {
            var nameInstance2 = nameValue2;
            virtualMachineExtensionInstance.name = nameInstance2;
          }
          
          var typeValue3 = responseDoc['type'];
          if (typeValue3 !== null && typeValue3 !== undefined) {
            var typeInstance3 = typeValue3;
            virtualMachineExtensionInstance.type = typeInstance3;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            virtualMachineExtensionInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey = property;
              var tagsValue = tagsSequenceElement[property];
              virtualMachineExtensionInstance.tags[tagsKey] = tagsValue;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return VirtualMachineExtensionOperations;
})();

var VirtualMachineOperations = ( /** @lends VirtualMachineOperations */ function() {
  /**
   * @class
   * Operations for managing the virtual machines in compute management.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ComputeManagementClient] {@link
   * ComputeManagementClient~ComputeManagementClient}.
   * See [virtualMachines] {@link
   * ComputeManagementClient~ComputeManagementClient#virtualMachines}.
   * Initializes a new instance of the VirtualMachineOperations class.
   * @constructor
   * 
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  function VirtualMachineOperations(client) {
    this.client = client;
  }
  
  /**
   * Captures the VM by copying VirtualHardDisks of the VM and outputs a
   * template that can be used to create similar VM.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {VirtualMachineCaptureParameters} parameters Parameters supplied to
   * the Capture Virtual Machine operation.
   * 
   * @param {string} parameters.virtualHardDiskNamePrefix Gets or sets the
   * captured VirtualHardDisk's name prefix.
   * 
   * @param {string} parameters.destinationContainerName Gets or sets the
   * destination container name.
   * 
   * @param {boolean} parameters.overwrite Gets or sets whether it overwrites
   * destination VirtualHardDisk if true, in case of conflict.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginCapturing = function(resourceGroupName, vmName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.destinationContainerName === null || parameters.destinationContainerName === undefined) {
      return callback(new Error('parameters.destinationContainerName cannot be null.'));
    }
    if (parameters.overwrite === null || parameters.overwrite === undefined) {
      return callback(new Error('parameters.overwrite cannot be null.'));
    }
    if (parameters.virtualHardDiskNamePrefix === null || parameters.virtualHardDiskNamePrefix === undefined) {
      return callback(new Error('parameters.virtualHardDiskNamePrefix cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/capture';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = null;
    
    var virtualMachineCaptureParametersValue = {};
    requestDoc = virtualMachineCaptureParametersValue;
    
    virtualMachineCaptureParametersValue['vhdPrefix'] = parameters.virtualHardDiskNamePrefix;
    
    virtualMachineCaptureParametersValue['destinationContainerName'] = parameters.destinationContainerName;
    
    virtualMachineCaptureParametersValue['overwriteVhds'] = parameters.overwrite;
    
    requestContent = JSON.stringify(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200 || statusCode === 202) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var capturedTemplateInstance = responseDoc;
          result.capturedTemplate = capturedTemplateInstance;
        }
        
      }
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to create a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {VirtualMachine} parameters Parameters supplied to the Create
   * Virtual Machine operation.
   * 
   * @param {Plan} [parameters.plan] Gets or sets the purchase plan when
   * deploying virtual machine from VM Marketplace images.
   * 
   * @param {string} [parameters.plan.name] Gets or sets the plan ID.
   * 
   * @param {string} [parameters.plan.publisher] Gets or sets the publisher ID.
   * 
   * @param {string} [parameters.plan.product] Gets or sets the offer ID.
   * 
   * @param {string} [parameters.plan.promotionCode] Gets or sets the promotion
   * code.
   * 
   * @param {HardwareProfile} [parameters.hardwareProfile] Gets or sets the
   * hardware profile.
   * 
   * @param {string} [parameters.hardwareProfile.virtualMachineSize] The
   * virtual machine size name.
   * 
   * @param {StorageProfile} [parameters.storageProfile] Gets or sets the
   * storage profile.
   * 
   * @param {SourceImageReference} [parameters.storageProfile.sourceImage] Gets
   * or sets the source image reference.
   * 
   * @param {string} [parameters.storageProfile.sourceImage.referenceUri] Gets
   * or sets the relative URL in the previous Service Management API's
   * namespace. For Example:
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   * 
   * @param {string} [parameters.storageProfile.destinationVhdsContainer] Gets
   * or sets the destination container for VHDs.
   * 
   * @param {OSDisk} [parameters.storageProfile.oSDisk] Gets or sets the OS
   * disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.operatingSystemType]
   * Gets or sets the Operating System type.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.name] Gets or sets the
   * disk name.
   * 
   * @param {VirtualHardDisk}
   * [parameters.storageProfile.oSDisk.virtualHardDisk] Gets or sets the
   * Virtual Hard Disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.virtualHardDisk.uri]
   * Gets or sets the virtual hard disk's uri. It should be a valid Uri to a
   * virtual hard disk.
   * 
   * @param {VirtualHardDisk} [parameters.storageProfile.oSDisk.sourceImage]
   * Gets or sets the Source User Image VirtualHardDisk. This VirtualHardDisk
   * will be copied before using it to attach to the Virtual Machine.If
   * SourceImage is provided, the destination VirtualHardDisk should not exist.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.sourceImage.uri] Gets or
   * sets the virtual hard disk's uri. It should be a valid Uri to a virtual
   * hard disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.caching] Gets or sets
   * the caching type.
   * 
   * @param {array} [parameters.storageProfile.dataDisks] Gets or sets the data
   * disks.
   * 
   * @param {OSProfile} [parameters.oSProfile] Gets or sets the OS profile.
   * 
   * @param {string} [parameters.oSProfile.computerName] Gets or sets the
   * computer name.
   * 
   * @param {string} [parameters.oSProfile.adminUsername] Gets or sets the
   * admin user name.
   * 
   * @param {string} [parameters.oSProfile.adminPassword] Gets or sets the
   * admin user password.
   * 
   * @param {string} [parameters.oSProfile.customData] Gets or sets a base-64
   * encoded string of custom data.
   * 
   * @param {WindowsConfiguration} [parameters.oSProfile.windowsConfiguration]
   * Gets or sets the Windows Configuration of the OS profile.
   * 
   * @param {boolean}
   * [parameters.oSProfile.windowsConfiguration.provisionVMAgent] Gets or sets
   * whether VM Agent should be provisioned on the Virtual Machine.
   * 
   * @param {boolean}
   * [parameters.oSProfile.windowsConfiguration.enableAutomaticUpdates] Gets
   * or sets whether Windows updates are automatically installed on the VM
   * 
   * @param {string} [parameters.oSProfile.windowsConfiguration.timeZone] Gets
   * or sets the Time Zone of the VM
   * 
   * @param {array}
   * [parameters.oSProfile.windowsConfiguration.additionalUnattendContents]
   * Gets or sets the additional base-64 encoded XML formatted information
   * that can be included in the Unattend.xml file.
   * 
   * @param {WinRMConfiguration}
   * [parameters.oSProfile.windowsConfiguration.winRMConfiguration] Gets or
   * sets the Windows Remote Management configuration of the VM
   * 
   * @param {array}
   * [parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners]
   * Gets or sets the list of Windows Remote Management listeners
   * 
   * @param {LinuxConfiguration} [parameters.oSProfile.linuxConfiguration] Gets
   * or sets the Linux Configuration of the OS profile.
   * 
   * @param {boolean}
   * [parameters.oSProfile.linuxConfiguration.disablePasswordAuthentication]
   * Gets or sets whether Authentication using user name and password is
   * allowed or not
   * 
   * @param {SshConfiguration}
   * [parameters.oSProfile.linuxConfiguration.sshConfiguration] Gets or sets
   * the SSH configuration for linux VMs
   * 
   * @param {array}
   * [parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys]
   * Gets or sets the list of SSH public keys used to authenticate with linux
   * based VMs
   * 
   * @param {array} [parameters.oSProfile.secrets] Gets or sets the List of
   * certificates for addition to the VM.
   * 
   * @param {NetworkProfile} [parameters.networkProfile] Gets or sets the
   * network profile.
   * 
   * @param {array} [parameters.networkProfile.networkInterfaces] Gets or sets
   * the network interfaces.
   * 
   * @param {AvailabilitySetReference} [parameters.availabilitySetReference]
   * Gets or sets the reference Id of the availailbity set to which this
   * virtual machine belongs.
   * 
   * @param {string} [parameters.availabilitySetReference.referenceUri] Gets or
   * sets the relative URL in the previous Service Management API's namespace.
   * For Example:
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   * 
   * @param {string} [parameters.provisioningState] Gets or sets the
   * provisioning state, which only appears in the response.
   * 
   * @param {VirtualMachineChildResources} [parameters.resources] Gets or sets
   * the virtual machine child resources.
   * 
   * @param {array} [parameters.resources.extensions] Gets or sets the virtual
   * machine child extension resources.
   * 
   * @param {string} [parameters.id] Gets or sets the ID of the resource.
   * 
   * @param {string} [parameters.name] Gets or sets the name of the resource.
   * 
   * @param {string} [parameters.type] Gets or sets the type of the resource.
   * 
   * @param {string} parameters.location Gets or sets the location of the
   * resource.
   * 
   * @param {object} [parameters.tags] Gets or sets the tags attached to the
   * resource.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginCreatingOrUpdating = function(resourceGroupName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.location === null || parameters.location === undefined) {
      return callback(new Error('parameters.location cannot be null.'));
    }
    if (parameters.resources !== null && parameters.resources !== undefined) {
      if (parameters.resources.extensions !== null && parameters.resources.extensions !== undefined) {
        for (var loweredIndex1 = 0; loweredIndex1 < parameters.resources.extensions.length; loweredIndex1 = loweredIndex1 + 1) {
          var extensionsParameterItem = parameters.resources.extensions[loweredIndex1];
          if (extensionsParameterItem.location === null || extensionsParameterItem.location === undefined) {
            return callback(new Error('parameters.resources.extensions.location cannot be null.'));
          }
        }
      }
    }
    if (parameters.storageProfile !== null && parameters.storageProfile !== undefined) {
      if (parameters.storageProfile.dataDisks !== null && parameters.storageProfile.dataDisks !== undefined) {
        for (var loweredIndex2 = 0; loweredIndex2 < parameters.storageProfile.dataDisks.length; loweredIndex2 = loweredIndex2 + 1) {
          var dataDisksParameterItem = parameters.storageProfile.dataDisks[loweredIndex2];
          if (dataDisksParameterItem.name === null || dataDisksParameterItem.name === undefined) {
            return callback(new Error('parameters.storageProfile.dataDisks.name cannot be null.'));
          }
        }
      }
      if (parameters.storageProfile.oSDisk !== null && parameters.storageProfile.oSDisk !== undefined) {
        if (parameters.storageProfile.oSDisk.name === null || parameters.storageProfile.oSDisk.name === undefined) {
          return callback(new Error('parameters.storageProfile.oSDisk.name cannot be null.'));
        }
      }
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    if (parameters.name !== null && parameters.name !== undefined) {
      url2 = url2 + encodeURIComponent(parameters.name);
    }
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = null;
    
    var virtualMachineJsonValue = {};
    requestDoc = virtualMachineJsonValue;
    
    if (parameters.plan !== null && parameters.plan !== undefined) {
      var planValue = {};
      virtualMachineJsonValue['plan'] = planValue;
      
      if (parameters.plan.name !== null && parameters.plan.name !== undefined) {
        planValue['name'] = parameters.plan.name;
      }
      
      if (parameters.plan.publisher !== null && parameters.plan.publisher !== undefined) {
        planValue['publisher'] = parameters.plan.publisher;
      }
      
      if (parameters.plan.product !== null && parameters.plan.product !== undefined) {
        planValue['product'] = parameters.plan.product;
      }
      
      if (parameters.plan.promotionCode !== null && parameters.plan.promotionCode !== undefined) {
        planValue['promotionCode'] = parameters.plan.promotionCode;
      }
    }
    
    var propertiesValue = {};
    virtualMachineJsonValue['properties'] = propertiesValue;
    
    if (parameters.hardwareProfile !== null && parameters.hardwareProfile !== undefined) {
      var hardwareProfileValue = {};
      propertiesValue['hardwareProfile'] = hardwareProfileValue;
      
      if (parameters.hardwareProfile.virtualMachineSize !== null && parameters.hardwareProfile.virtualMachineSize !== undefined) {
        hardwareProfileValue['vmSize'] = parameters.hardwareProfile.virtualMachineSize;
      }
    }
    
    if (parameters.storageProfile !== null && parameters.storageProfile !== undefined) {
      var storageProfileValue = {};
      propertiesValue['storageProfile'] = storageProfileValue;
      
      if (parameters.storageProfile.sourceImage !== null && parameters.storageProfile.sourceImage !== undefined) {
        var sourceImageValue = {};
        storageProfileValue['sourceImage'] = sourceImageValue;
        
        if (parameters.storageProfile.sourceImage.referenceUri !== null && parameters.storageProfile.sourceImage.referenceUri !== undefined) {
          sourceImageValue['id'] = parameters.storageProfile.sourceImage.referenceUri;
        }
      }
      
      if (parameters.storageProfile.destinationVhdsContainer !== null && parameters.storageProfile.destinationVhdsContainer !== undefined) {
        storageProfileValue['destinationVhdsContainer'] = parameters.storageProfile.destinationVhdsContainer;
      }
      
      if (parameters.storageProfile.oSDisk !== null && parameters.storageProfile.oSDisk !== undefined) {
        var osDiskValue = {};
        storageProfileValue['osDisk'] = osDiskValue;
        
        if (parameters.storageProfile.oSDisk.operatingSystemType !== null && parameters.storageProfile.oSDisk.operatingSystemType !== undefined) {
          osDiskValue['osType'] = parameters.storageProfile.oSDisk.operatingSystemType;
        }
        
        osDiskValue['name'] = parameters.storageProfile.oSDisk.name;
        
        if (parameters.storageProfile.oSDisk.virtualHardDisk !== null && parameters.storageProfile.oSDisk.virtualHardDisk !== undefined) {
          var vhdValue = {};
          osDiskValue['vhd'] = vhdValue;
          
          if (parameters.storageProfile.oSDisk.virtualHardDisk.uri !== null && parameters.storageProfile.oSDisk.virtualHardDisk.uri !== undefined) {
            vhdValue['uri'] = parameters.storageProfile.oSDisk.virtualHardDisk.uri;
          }
        }
        
        if (parameters.storageProfile.oSDisk.sourceImage !== null && parameters.storageProfile.oSDisk.sourceImage !== undefined) {
          var imageValue = {};
          osDiskValue['image'] = imageValue;
          
          if (parameters.storageProfile.oSDisk.sourceImage.uri !== null && parameters.storageProfile.oSDisk.sourceImage.uri !== undefined) {
            imageValue['uri'] = parameters.storageProfile.oSDisk.sourceImage.uri;
          }
        }
        
        if (parameters.storageProfile.oSDisk.caching !== null && parameters.storageProfile.oSDisk.caching !== undefined) {
          osDiskValue['caching'] = parameters.storageProfile.oSDisk.caching;
        }
      }
      
      if (parameters.storageProfile.dataDisks !== null && parameters.storageProfile.dataDisks !== undefined) {
        var dataDisksArray = [];
        for (var loweredIndex3 = 0; loweredIndex3 < parameters.storageProfile.dataDisks.length; loweredIndex3 = loweredIndex3 + 1) {
          var dataDisksItem = parameters.storageProfile.dataDisks[loweredIndex3];
          var dataDiskValue = {};
          dataDisksArray.push(dataDiskValue);
          
          if (dataDisksItem.lun !== null && dataDisksItem.lun !== undefined) {
            dataDiskValue['lun'] = dataDisksItem.lun;
          }
          
          if (dataDisksItem.diskSizeGB !== null && dataDisksItem.diskSizeGB !== undefined) {
            dataDiskValue['diskSizeGB'] = dataDisksItem.diskSizeGB;
          }
          
          dataDiskValue['name'] = dataDisksItem.name;
          
          if (dataDisksItem.virtualHardDisk !== null && dataDisksItem.virtualHardDisk !== undefined) {
            var vhdValue2 = {};
            dataDiskValue['vhd'] = vhdValue2;
            
            if (dataDisksItem.virtualHardDisk.uri !== null && dataDisksItem.virtualHardDisk.uri !== undefined) {
              vhdValue2['uri'] = dataDisksItem.virtualHardDisk.uri;
            }
          }
          
          if (dataDisksItem.sourceImage !== null && dataDisksItem.sourceImage !== undefined) {
            var imageValue2 = {};
            dataDiskValue['image'] = imageValue2;
            
            if (dataDisksItem.sourceImage.uri !== null && dataDisksItem.sourceImage.uri !== undefined) {
              imageValue2['uri'] = dataDisksItem.sourceImage.uri;
            }
          }
          
          if (dataDisksItem.caching !== null && dataDisksItem.caching !== undefined) {
            dataDiskValue['caching'] = dataDisksItem.caching;
          }
        }
        storageProfileValue['dataDisks'] = dataDisksArray;
      }
    }
    
    if (parameters.oSProfile !== null && parameters.oSProfile !== undefined) {
      var osProfileValue = {};
      propertiesValue['osProfile'] = osProfileValue;
      
      if (parameters.oSProfile.computerName !== null && parameters.oSProfile.computerName !== undefined) {
        osProfileValue['computerName'] = parameters.oSProfile.computerName;
      }
      
      if (parameters.oSProfile.adminUsername !== null && parameters.oSProfile.adminUsername !== undefined) {
        osProfileValue['adminUsername'] = parameters.oSProfile.adminUsername;
      }
      
      if (parameters.oSProfile.adminPassword !== null && parameters.oSProfile.adminPassword !== undefined) {
        osProfileValue['adminPassword'] = parameters.oSProfile.adminPassword;
      }
      
      if (parameters.oSProfile.customData !== null && parameters.oSProfile.customData !== undefined) {
        osProfileValue['customData'] = parameters.oSProfile.customData;
      }
      
      if (parameters.oSProfile.windowsConfiguration !== null && parameters.oSProfile.windowsConfiguration !== undefined) {
        var windowsConfigurationValue = {};
        osProfileValue['windowsConfiguration'] = windowsConfigurationValue;
        
        if (parameters.oSProfile.windowsConfiguration.provisionVMAgent !== null && parameters.oSProfile.windowsConfiguration.provisionVMAgent !== undefined) {
          windowsConfigurationValue['provisionVMAgent'] = parameters.oSProfile.windowsConfiguration.provisionVMAgent;
        }
        
        if (parameters.oSProfile.windowsConfiguration.enableAutomaticUpdates !== null && parameters.oSProfile.windowsConfiguration.enableAutomaticUpdates !== undefined) {
          windowsConfigurationValue['enableAutomaticUpdates'] = parameters.oSProfile.windowsConfiguration.enableAutomaticUpdates;
        }
        
        if (parameters.oSProfile.windowsConfiguration.timeZone !== null && parameters.oSProfile.windowsConfiguration.timeZone !== undefined) {
          windowsConfigurationValue['timeZone'] = parameters.oSProfile.windowsConfiguration.timeZone;
        }
        
        if (parameters.oSProfile.windowsConfiguration.additionalUnattendContents !== null && parameters.oSProfile.windowsConfiguration.additionalUnattendContents !== undefined) {
          var additionalUnattendContentArray = [];
          for (var loweredIndex4 = 0; loweredIndex4 < parameters.oSProfile.windowsConfiguration.additionalUnattendContents.length; loweredIndex4 = loweredIndex4 + 1) {
            var additionalUnattendContentItem = parameters.oSProfile.windowsConfiguration.additionalUnattendContents[loweredIndex4];
            var additionalUnattendContentValue = {};
            additionalUnattendContentArray.push(additionalUnattendContentValue);
            
            if (additionalUnattendContentItem.passName !== null && additionalUnattendContentItem.passName !== undefined) {
              additionalUnattendContentValue['passName'] = additionalUnattendContentItem.passName;
            }
            
            if (additionalUnattendContentItem.componentName !== null && additionalUnattendContentItem.componentName !== undefined) {
              additionalUnattendContentValue['componentName'] = additionalUnattendContentItem.componentName;
            }
            
            if (additionalUnattendContentItem.settingName !== null && additionalUnattendContentItem.settingName !== undefined) {
              additionalUnattendContentValue['settingName'] = additionalUnattendContentItem.settingName;
            }
            
            if (additionalUnattendContentItem.content !== null && additionalUnattendContentItem.content !== undefined) {
              additionalUnattendContentValue['content'] = additionalUnattendContentItem.content;
            }
          }
          windowsConfigurationValue['additionalUnattendContent'] = additionalUnattendContentArray;
        }
        
        if (parameters.oSProfile.windowsConfiguration.winRMConfiguration !== null && parameters.oSProfile.windowsConfiguration.winRMConfiguration !== undefined) {
          var winRMValue = {};
          windowsConfigurationValue['winRM'] = winRMValue;
          
          if (parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners !== null && parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners !== undefined) {
            var listenersArray = [];
            for (var loweredIndex5 = 0; loweredIndex5 < parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners.length; loweredIndex5 = loweredIndex5 + 1) {
              var listenersItem = parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners[loweredIndex5];
              var winRMListenerValue = {};
              listenersArray.push(winRMListenerValue);
              
              if (listenersItem.protocol !== null && listenersItem.protocol !== undefined) {
                winRMListenerValue['protocol'] = listenersItem.protocol;
              }
              
              if (listenersItem.certificateUrl !== null && listenersItem.certificateUrl !== undefined) {
                winRMListenerValue['certificateUrl'] = listenersItem.certificateUrl;
              }
            }
            winRMValue['listeners'] = listenersArray;
          }
        }
      }
      
      if (parameters.oSProfile.linuxConfiguration !== null && parameters.oSProfile.linuxConfiguration !== undefined) {
        var linuxConfigurationValue = {};
        osProfileValue['linuxConfiguration'] = linuxConfigurationValue;
        
        if (parameters.oSProfile.linuxConfiguration.disablePasswordAuthentication !== null && parameters.oSProfile.linuxConfiguration.disablePasswordAuthentication !== undefined) {
          linuxConfigurationValue['disablePasswordAuthentication'] = parameters.oSProfile.linuxConfiguration.disablePasswordAuthentication;
        }
        
        if (parameters.oSProfile.linuxConfiguration.sshConfiguration !== null && parameters.oSProfile.linuxConfiguration.sshConfiguration !== undefined) {
          var sshValue = {};
          linuxConfigurationValue['ssh'] = sshValue;
          
          if (parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys !== null && parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys !== undefined) {
            var publicKeysArray = [];
            for (var loweredIndex6 = 0; loweredIndex6 < parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys.length; loweredIndex6 = loweredIndex6 + 1) {
              var publicKeysItem = parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys[loweredIndex6];
              var sshPublicKeyValue = {};
              publicKeysArray.push(sshPublicKeyValue);
              
              if (publicKeysItem.path !== null && publicKeysItem.path !== undefined) {
                sshPublicKeyValue['path'] = publicKeysItem.path;
              }
              
              if (publicKeysItem.keyData !== null && publicKeysItem.keyData !== undefined) {
                sshPublicKeyValue['keyData'] = publicKeysItem.keyData;
              }
            }
            sshValue['publicKeys'] = publicKeysArray;
          }
        }
      }
      
      if (parameters.oSProfile.secrets !== null && parameters.oSProfile.secrets !== undefined) {
        var secretsArray = [];
        for (var loweredIndex7 = 0; loweredIndex7 < parameters.oSProfile.secrets.length; loweredIndex7 = loweredIndex7 + 1) {
          var secretsItem = parameters.oSProfile.secrets[loweredIndex7];
          var vaultSecretGroupValue = {};
          secretsArray.push(vaultSecretGroupValue);
          
          if (secretsItem.sourceVault !== null && secretsItem.sourceVault !== undefined) {
            var sourceVaultValue = {};
            vaultSecretGroupValue['sourceVault'] = sourceVaultValue;
            
            if (secretsItem.sourceVault.referenceUri !== null && secretsItem.sourceVault.referenceUri !== undefined) {
              sourceVaultValue['id'] = secretsItem.sourceVault.referenceUri;
            }
          }
          
          if (secretsItem.vaultCertificates !== null && secretsItem.vaultCertificates !== undefined) {
            var vaultCertificatesArray = [];
            for (var loweredIndex8 = 0; loweredIndex8 < secretsItem.vaultCertificates.length; loweredIndex8 = loweredIndex8 + 1) {
              var vaultCertificatesItem = secretsItem.vaultCertificates[loweredIndex8];
              var vaultCertificateValue = {};
              vaultCertificatesArray.push(vaultCertificateValue);
              
              if (vaultCertificatesItem.certificateUrl !== null && vaultCertificatesItem.certificateUrl !== undefined) {
                vaultCertificateValue['certificateUrl'] = vaultCertificatesItem.certificateUrl;
              }
              
              if (vaultCertificatesItem.certificateStore !== null && vaultCertificatesItem.certificateStore !== undefined) {
                vaultCertificateValue['certificateStore'] = vaultCertificatesItem.certificateStore;
              }
            }
            vaultSecretGroupValue['vaultCertificates'] = vaultCertificatesArray;
          }
        }
        osProfileValue['secrets'] = secretsArray;
      }
    }
    
    if (parameters.networkProfile !== null && parameters.networkProfile !== undefined) {
      var networkProfileValue = {};
      propertiesValue['networkProfile'] = networkProfileValue;
      
      if (parameters.networkProfile.networkInterfaces !== null && parameters.networkProfile.networkInterfaces !== undefined) {
        var networkInterfacesArray = [];
        for (var loweredIndex9 = 0; loweredIndex9 < parameters.networkProfile.networkInterfaces.length; loweredIndex9 = loweredIndex9 + 1) {
          var networkInterfacesItem = parameters.networkProfile.networkInterfaces[loweredIndex9];
          var networkInterfaceReferenceJsonValue = {};
          networkInterfacesArray.push(networkInterfaceReferenceJsonValue);
          
          var propertiesValue2 = {};
          networkInterfaceReferenceJsonValue['properties'] = propertiesValue2;
          
          if (networkInterfacesItem.primary !== null && networkInterfacesItem.primary !== undefined) {
            propertiesValue2['primary'] = networkInterfacesItem.primary;
          }
          
          if (networkInterfacesItem.referenceUri !== null && networkInterfacesItem.referenceUri !== undefined) {
            networkInterfaceReferenceJsonValue['id'] = networkInterfacesItem.referenceUri;
          }
        }
        networkProfileValue['networkInterfaces'] = networkInterfacesArray;
      }
    }
    
    if (parameters.availabilitySetReference !== null && parameters.availabilitySetReference !== undefined) {
      var availabilitySetValue = {};
      propertiesValue['availabilitySet'] = availabilitySetValue;
      
      if (parameters.availabilitySetReference.referenceUri !== null && parameters.availabilitySetReference.referenceUri !== undefined) {
        availabilitySetValue['id'] = parameters.availabilitySetReference.referenceUri;
      }
    }
    
    if (parameters.provisioningState !== null && parameters.provisioningState !== undefined) {
      propertiesValue['provisioningState'] = parameters.provisioningState;
    }
    
    if (parameters.resources !== null && parameters.resources !== undefined) {
      var resourcesValue = {};
      virtualMachineJsonValue['resources'] = resourcesValue;
      
      if (parameters.resources.extensions !== null && parameters.resources.extensions !== undefined) {
        var extensionsArray = [];
        for (var loweredIndex10 = 0; loweredIndex10 < parameters.resources.extensions.length; loweredIndex10 = loweredIndex10 + 1) {
          var extensionsItem = parameters.resources.extensions[loweredIndex10];
          var virtualMachineExtensionJsonValue = {};
          extensionsArray.push(virtualMachineExtensionJsonValue);
          
          var propertiesValue3 = {};
          virtualMachineExtensionJsonValue['properties'] = propertiesValue3;
          
          if (extensionsItem.publisher !== null && extensionsItem.publisher !== undefined) {
            propertiesValue3['publisher'] = extensionsItem.publisher;
          }
          
          if (extensionsItem.extensionType !== null && extensionsItem.extensionType !== undefined) {
            propertiesValue3['type'] = extensionsItem.extensionType;
          }
          
          if (extensionsItem.typeHandlerVersion !== null && extensionsItem.typeHandlerVersion !== undefined) {
            propertiesValue3['typeHandlerVersion'] = extensionsItem.typeHandlerVersion;
          }
          
          if (extensionsItem.settings !== null && extensionsItem.settings !== undefined) {
            propertiesValue3['settings'] = extensionsItem.settings;
          }
          
          if (extensionsItem.protectedSettings !== null && extensionsItem.protectedSettings !== undefined) {
            propertiesValue3['protectedSettings'] = extensionsItem.protectedSettings;
          }
          
          if (extensionsItem.provisioningState !== null && extensionsItem.provisioningState !== undefined) {
            propertiesValue3['provisioningState'] = extensionsItem.provisioningState;
          }
          
          if (extensionsItem.instanceView !== null && extensionsItem.instanceView !== undefined) {
            var instanceViewValue = {};
            propertiesValue3['instanceView'] = instanceViewValue;
            
            if (extensionsItem.instanceView.name !== null && extensionsItem.instanceView.name !== undefined) {
              instanceViewValue['name'] = extensionsItem.instanceView.name;
            }
            
            if (extensionsItem.instanceView.extensionType !== null && extensionsItem.instanceView.extensionType !== undefined) {
              instanceViewValue['type'] = extensionsItem.instanceView.extensionType;
            }
            
            if (extensionsItem.instanceView.typeHandlerVersion !== null && extensionsItem.instanceView.typeHandlerVersion !== undefined) {
              instanceViewValue['typeHandlerVersion'] = extensionsItem.instanceView.typeHandlerVersion;
            }
            
            if (extensionsItem.instanceView.subStatuses !== null && extensionsItem.instanceView.subStatuses !== undefined) {
              var substatusesArray = [];
              for (var loweredIndex11 = 0; loweredIndex11 < extensionsItem.instanceView.subStatuses.length; loweredIndex11 = loweredIndex11 + 1) {
                var substatusesItem = extensionsItem.instanceView.subStatuses[loweredIndex11];
                
                if (substatusesItem.code !== null && substatusesItem.code !== undefined) {
                  requestDoc = substatusesItem.code;
                }
                
                if (substatusesItem.level !== null && substatusesItem.level !== undefined) {
                  requestDoc = substatusesItem.level;
                }
                
                if (substatusesItem.displayStatus !== null && substatusesItem.displayStatus !== undefined) {
                  requestDoc = substatusesItem.displayStatus;
                }
                
                if (substatusesItem.message !== null && substatusesItem.message !== undefined) {
                  requestDoc = substatusesItem.message;
                }
                
                if (substatusesItem.time !== null && substatusesItem.time !== undefined) {
                  requestDoc = substatusesItem.time;
                }
              }
              instanceViewValue['substatuses'] = substatusesArray;
            }
            
            if (extensionsItem.instanceView.statuses !== null && extensionsItem.instanceView.statuses !== undefined) {
              var statusesArray = [];
              for (var loweredIndex12 = 0; loweredIndex12 < extensionsItem.instanceView.statuses.length; loweredIndex12 = loweredIndex12 + 1) {
                var statusesItem = extensionsItem.instanceView.statuses[loweredIndex12];
                
                if (statusesItem.code !== null && statusesItem.code !== undefined) {
                  requestDoc = statusesItem.code;
                }
                
                if (statusesItem.level !== null && statusesItem.level !== undefined) {
                  requestDoc = statusesItem.level;
                }
                
                if (statusesItem.displayStatus !== null && statusesItem.displayStatus !== undefined) {
                  requestDoc = statusesItem.displayStatus;
                }
                
                if (statusesItem.message !== null && statusesItem.message !== undefined) {
                  requestDoc = statusesItem.message;
                }
                
                if (statusesItem.time !== null && statusesItem.time !== undefined) {
                  requestDoc = statusesItem.time;
                }
              }
              instanceViewValue['statuses'] = statusesArray;
            }
          }
          
          if (extensionsItem.id !== null && extensionsItem.id !== undefined) {
            virtualMachineExtensionJsonValue['id'] = extensionsItem.id;
          }
          
          if (extensionsItem.name !== null && extensionsItem.name !== undefined) {
            virtualMachineExtensionJsonValue['name'] = extensionsItem.name;
          }
          
          if (extensionsItem.type !== null && extensionsItem.type !== undefined) {
            virtualMachineExtensionJsonValue['type'] = extensionsItem.type;
          }
          
          virtualMachineExtensionJsonValue['location'] = extensionsItem.location;
          
          if (extensionsItem.tags !== null && extensionsItem.tags !== undefined) {
            var tagsDictionary = {};
            for (var tagsKey in extensionsItem.tags) {
              var tagsValue = extensionsItem.tags[tagsKey];
              tagsDictionary[tagsKey] = tagsValue;
            }
            virtualMachineExtensionJsonValue['tags'] = tagsDictionary;
          }
        }
        resourcesValue['extensions'] = extensionsArray;
      }
    }
    
    if (parameters.id !== null && parameters.id !== undefined) {
      virtualMachineJsonValue['id'] = parameters.id;
    }
    
    if (parameters.name !== null && parameters.name !== undefined) {
      virtualMachineJsonValue['name'] = parameters.name;
    }
    
    if (parameters.type !== null && parameters.type !== undefined) {
      virtualMachineJsonValue['type'] = parameters.type;
    }
    
    virtualMachineJsonValue['location'] = parameters.location;
    
    if (parameters.tags !== null && parameters.tags !== undefined) {
      var tagsDictionary2 = {};
      for (var tagsKey2 in parameters.tags) {
        var tagsValue2 = parameters.tags[tagsKey2];
        tagsDictionary2[tagsKey2] = tagsValue2;
      }
      virtualMachineJsonValue['tags'] = tagsDictionary2;
    }
    
    requestContent = JSON.stringify(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200 && statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200 || statusCode === 201) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var virtualMachineInstance = { tags: {} };
          result.virtualMachine = virtualMachineInstance;
          
          var planValue2 = responseDoc['plan'];
          if (planValue2 !== null && planValue2 !== undefined) {
            var planInstance = {};
            virtualMachineInstance.plan = planInstance;
            
            var nameValue = planValue2['name'];
            if (nameValue !== null && nameValue !== undefined) {
              var nameInstance = nameValue;
              planInstance.name = nameInstance;
            }
            
            var publisherValue = planValue2['publisher'];
            if (publisherValue !== null && publisherValue !== undefined) {
              var publisherInstance = publisherValue;
              planInstance.publisher = publisherInstance;
            }
            
            var productValue = planValue2['product'];
            if (productValue !== null && productValue !== undefined) {
              var productInstance = productValue;
              planInstance.product = productInstance;
            }
            
            var promotionCodeValue = planValue2['promotionCode'];
            if (promotionCodeValue !== null && promotionCodeValue !== undefined) {
              var promotionCodeInstance = promotionCodeValue;
              planInstance.promotionCode = promotionCodeInstance;
            }
          }
          
          var propertiesValue4 = responseDoc['properties'];
          if (propertiesValue4 !== null && propertiesValue4 !== undefined) {
            var hardwareProfileValue2 = propertiesValue4['hardwareProfile'];
            if (hardwareProfileValue2 !== null && hardwareProfileValue2 !== undefined) {
              var hardwareProfileInstance = {};
              virtualMachineInstance.hardwareProfile = hardwareProfileInstance;
              
              var vmSizeValue = hardwareProfileValue2['vmSize'];
              if (vmSizeValue !== null && vmSizeValue !== undefined) {
                var vmSizeInstance = vmSizeValue;
                hardwareProfileInstance.virtualMachineSize = vmSizeInstance;
              }
            }
            
            var storageProfileValue2 = propertiesValue4['storageProfile'];
            if (storageProfileValue2 !== null && storageProfileValue2 !== undefined) {
              var storageProfileInstance = { dataDisks: [] };
              virtualMachineInstance.storageProfile = storageProfileInstance;
              
              var sourceImageValue2 = storageProfileValue2['sourceImage'];
              if (sourceImageValue2 !== null && sourceImageValue2 !== undefined) {
                var sourceImageInstance = {};
                storageProfileInstance.sourceImage = sourceImageInstance;
                
                var idValue = sourceImageValue2['id'];
                if (idValue !== null && idValue !== undefined) {
                  var idInstance = idValue;
                  sourceImageInstance.referenceUri = idInstance;
                }
              }
              
              var destinationVhdsContainerValue = storageProfileValue2['destinationVhdsContainer'];
              if (destinationVhdsContainerValue !== null && destinationVhdsContainerValue !== undefined) {
                var destinationVhdsContainerInstance = destinationVhdsContainerValue;
                storageProfileInstance.destinationVhdsContainer = destinationVhdsContainerInstance;
              }
              
              var osDiskValue2 = storageProfileValue2['osDisk'];
              if (osDiskValue2 !== null && osDiskValue2 !== undefined) {
                var osDiskInstance = {};
                storageProfileInstance.oSDisk = osDiskInstance;
                
                var osTypeValue = osDiskValue2['osType'];
                if (osTypeValue !== null && osTypeValue !== undefined) {
                  var osTypeInstance = osTypeValue;
                  osDiskInstance.operatingSystemType = osTypeInstance;
                }
                
                var nameValue2 = osDiskValue2['name'];
                if (nameValue2 !== null && nameValue2 !== undefined) {
                  var nameInstance2 = nameValue2;
                  osDiskInstance.name = nameInstance2;
                }
                
                var vhdValue3 = osDiskValue2['vhd'];
                if (vhdValue3 !== null && vhdValue3 !== undefined) {
                  var vhdInstance = {};
                  osDiskInstance.virtualHardDisk = vhdInstance;
                  
                  var uriValue = vhdValue3['uri'];
                  if (uriValue !== null && uriValue !== undefined) {
                    var uriInstance = uriValue;
                    vhdInstance.uri = uriInstance;
                  }
                }
                
                var imageValue3 = osDiskValue2['image'];
                if (imageValue3 !== null && imageValue3 !== undefined) {
                  var imageInstance = {};
                  osDiskInstance.sourceImage = imageInstance;
                  
                  var uriValue2 = imageValue3['uri'];
                  if (uriValue2 !== null && uriValue2 !== undefined) {
                    var uriInstance2 = uriValue2;
                    imageInstance.uri = uriInstance2;
                  }
                }
                
                var cachingValue = osDiskValue2['caching'];
                if (cachingValue !== null && cachingValue !== undefined) {
                  var cachingInstance = cachingValue;
                  osDiskInstance.caching = cachingInstance;
                }
              }
              
              var dataDisksArray2 = storageProfileValue2['dataDisks'];
              if (dataDisksArray2 !== null && dataDisksArray2 !== undefined) {
                for (var loweredIndex13 = 0; loweredIndex13 < dataDisksArray2.length; loweredIndex13 = loweredIndex13 + 1) {
                  var dataDisksValue = dataDisksArray2[loweredIndex13];
                  var dataDiskInstance = {};
                  storageProfileInstance.dataDisks.push(dataDiskInstance);
                  
                  var lunValue = dataDisksValue['lun'];
                  if (lunValue !== null && lunValue !== undefined) {
                    var lunInstance = lunValue;
                    dataDiskInstance.lun = lunInstance;
                  }
                  
                  var diskSizeGBValue = dataDisksValue['diskSizeGB'];
                  if (diskSizeGBValue !== null && diskSizeGBValue !== undefined) {
                    var diskSizeGBInstance = diskSizeGBValue;
                    dataDiskInstance.diskSizeGB = diskSizeGBInstance;
                  }
                  
                  var nameValue3 = dataDisksValue['name'];
                  if (nameValue3 !== null && nameValue3 !== undefined) {
                    var nameInstance3 = nameValue3;
                    dataDiskInstance.name = nameInstance3;
                  }
                  
                  var vhdValue4 = dataDisksValue['vhd'];
                  if (vhdValue4 !== null && vhdValue4 !== undefined) {
                    var vhdInstance2 = {};
                    dataDiskInstance.virtualHardDisk = vhdInstance2;
                    
                    var uriValue3 = vhdValue4['uri'];
                    if (uriValue3 !== null && uriValue3 !== undefined) {
                      var uriInstance3 = uriValue3;
                      vhdInstance2.uri = uriInstance3;
                    }
                  }
                  
                  var imageValue4 = dataDisksValue['image'];
                  if (imageValue4 !== null && imageValue4 !== undefined) {
                    var imageInstance2 = {};
                    dataDiskInstance.sourceImage = imageInstance2;
                    
                    var uriValue4 = imageValue4['uri'];
                    if (uriValue4 !== null && uriValue4 !== undefined) {
                      var uriInstance4 = uriValue4;
                      imageInstance2.uri = uriInstance4;
                    }
                  }
                  
                  var cachingValue2 = dataDisksValue['caching'];
                  if (cachingValue2 !== null && cachingValue2 !== undefined) {
                    var cachingInstance2 = cachingValue2;
                    dataDiskInstance.caching = cachingInstance2;
                  }
                }
              }
            }
            
            var osProfileValue2 = propertiesValue4['osProfile'];
            if (osProfileValue2 !== null && osProfileValue2 !== undefined) {
              var osProfileInstance = { secrets: [] };
              virtualMachineInstance.oSProfile = osProfileInstance;
              
              var computerNameValue = osProfileValue2['computerName'];
              if (computerNameValue !== null && computerNameValue !== undefined) {
                var computerNameInstance = computerNameValue;
                osProfileInstance.computerName = computerNameInstance;
              }
              
              var adminUsernameValue = osProfileValue2['adminUsername'];
              if (adminUsernameValue !== null && adminUsernameValue !== undefined) {
                var adminUsernameInstance = adminUsernameValue;
                osProfileInstance.adminUsername = adminUsernameInstance;
              }
              
              var adminPasswordValue = osProfileValue2['adminPassword'];
              if (adminPasswordValue !== null && adminPasswordValue !== undefined) {
                var adminPasswordInstance = adminPasswordValue;
                osProfileInstance.adminPassword = adminPasswordInstance;
              }
              
              var customDataValue = osProfileValue2['customData'];
              if (customDataValue !== null && customDataValue !== undefined) {
                var customDataInstance = customDataValue;
                osProfileInstance.customData = customDataInstance;
              }
              
              var windowsConfigurationValue2 = osProfileValue2['windowsConfiguration'];
              if (windowsConfigurationValue2 !== null && windowsConfigurationValue2 !== undefined) {
                var windowsConfigurationInstance = { additionalUnattendContents: [] };
                osProfileInstance.windowsConfiguration = windowsConfigurationInstance;
                
                var provisionVMAgentValue = windowsConfigurationValue2['provisionVMAgent'];
                if (provisionVMAgentValue !== null && provisionVMAgentValue !== undefined) {
                  var provisionVMAgentInstance = provisionVMAgentValue;
                  windowsConfigurationInstance.provisionVMAgent = provisionVMAgentInstance;
                }
                
                var enableAutomaticUpdatesValue = windowsConfigurationValue2['enableAutomaticUpdates'];
                if (enableAutomaticUpdatesValue !== null && enableAutomaticUpdatesValue !== undefined) {
                  var enableAutomaticUpdatesInstance = enableAutomaticUpdatesValue;
                  windowsConfigurationInstance.enableAutomaticUpdates = enableAutomaticUpdatesInstance;
                }
                
                var timeZoneValue = windowsConfigurationValue2['timeZone'];
                if (timeZoneValue !== null && timeZoneValue !== undefined) {
                  var timeZoneInstance = timeZoneValue;
                  windowsConfigurationInstance.timeZone = timeZoneInstance;
                }
                
                var additionalUnattendContentArray2 = windowsConfigurationValue2['additionalUnattendContent'];
                if (additionalUnattendContentArray2 !== null && additionalUnattendContentArray2 !== undefined) {
                  for (var loweredIndex14 = 0; loweredIndex14 < additionalUnattendContentArray2.length; loweredIndex14 = loweredIndex14 + 1) {
                    var additionalUnattendContentValue2 = additionalUnattendContentArray2[loweredIndex14];
                    var additionalUnattendContentInstance = {};
                    windowsConfigurationInstance.additionalUnattendContents.push(additionalUnattendContentInstance);
                    
                    var passNameValue = additionalUnattendContentValue2['passName'];
                    if (passNameValue !== null && passNameValue !== undefined) {
                      var passNameInstance = passNameValue;
                      additionalUnattendContentInstance.passName = passNameInstance;
                    }
                    
                    var componentNameValue = additionalUnattendContentValue2['componentName'];
                    if (componentNameValue !== null && componentNameValue !== undefined) {
                      var componentNameInstance = componentNameValue;
                      additionalUnattendContentInstance.componentName = componentNameInstance;
                    }
                    
                    var settingNameValue = additionalUnattendContentValue2['settingName'];
                    if (settingNameValue !== null && settingNameValue !== undefined) {
                      var settingNameInstance = settingNameValue;
                      additionalUnattendContentInstance.settingName = settingNameInstance;
                    }
                    
                    var contentValue = additionalUnattendContentValue2['content'];
                    if (contentValue !== null && contentValue !== undefined) {
                      var contentInstance = contentValue;
                      additionalUnattendContentInstance.content = contentInstance;
                    }
                  }
                }
                
                var winRMValue2 = windowsConfigurationValue2['winRM'];
                if (winRMValue2 !== null && winRMValue2 !== undefined) {
                  var winRMInstance = { listeners: [] };
                  windowsConfigurationInstance.winRMConfiguration = winRMInstance;
                  
                  var listenersArray2 = winRMValue2['listeners'];
                  if (listenersArray2 !== null && listenersArray2 !== undefined) {
                    for (var loweredIndex15 = 0; loweredIndex15 < listenersArray2.length; loweredIndex15 = loweredIndex15 + 1) {
                      var listenersValue = listenersArray2[loweredIndex15];
                      var winRMListenerInstance = {};
                      winRMInstance.listeners.push(winRMListenerInstance);
                      
                      var protocolValue = listenersValue['protocol'];
                      if (protocolValue !== null && protocolValue !== undefined) {
                        var protocolInstance = protocolValue;
                        winRMListenerInstance.protocol = protocolInstance;
                      }
                      
                      var certificateUrlValue = listenersValue['certificateUrl'];
                      if (certificateUrlValue !== null && certificateUrlValue !== undefined) {
                        var certificateUrlInstance = certificateUrlValue;
                        winRMListenerInstance.certificateUrl = certificateUrlInstance;
                      }
                    }
                  }
                }
              }
              
              var linuxConfigurationValue2 = osProfileValue2['linuxConfiguration'];
              if (linuxConfigurationValue2 !== null && linuxConfigurationValue2 !== undefined) {
                var linuxConfigurationInstance = {};
                osProfileInstance.linuxConfiguration = linuxConfigurationInstance;
                
                var disablePasswordAuthenticationValue = linuxConfigurationValue2['disablePasswordAuthentication'];
                if (disablePasswordAuthenticationValue !== null && disablePasswordAuthenticationValue !== undefined) {
                  var disablePasswordAuthenticationInstance = disablePasswordAuthenticationValue;
                  linuxConfigurationInstance.disablePasswordAuthentication = disablePasswordAuthenticationInstance;
                }
                
                var sshValue2 = linuxConfigurationValue2['ssh'];
                if (sshValue2 !== null && sshValue2 !== undefined) {
                  var sshInstance = { publicKeys: [] };
                  linuxConfigurationInstance.sshConfiguration = sshInstance;
                  
                  var publicKeysArray2 = sshValue2['publicKeys'];
                  if (publicKeysArray2 !== null && publicKeysArray2 !== undefined) {
                    for (var loweredIndex16 = 0; loweredIndex16 < publicKeysArray2.length; loweredIndex16 = loweredIndex16 + 1) {
                      var publicKeysValue = publicKeysArray2[loweredIndex16];
                      var sshPublicKeyInstance = {};
                      sshInstance.publicKeys.push(sshPublicKeyInstance);
                      
                      var pathValue = publicKeysValue['path'];
                      if (pathValue !== null && pathValue !== undefined) {
                        var pathInstance = pathValue;
                        sshPublicKeyInstance.path = pathInstance;
                      }
                      
                      var keyDataValue = publicKeysValue['keyData'];
                      if (keyDataValue !== null && keyDataValue !== undefined) {
                        var keyDataInstance = keyDataValue;
                        sshPublicKeyInstance.keyData = keyDataInstance;
                      }
                    }
                  }
                }
              }
              
              var secretsArray2 = osProfileValue2['secrets'];
              if (secretsArray2 !== null && secretsArray2 !== undefined) {
                for (var loweredIndex17 = 0; loweredIndex17 < secretsArray2.length; loweredIndex17 = loweredIndex17 + 1) {
                  var secretsValue = secretsArray2[loweredIndex17];
                  var vaultSecretGroupInstance = { vaultCertificates: [] };
                  osProfileInstance.secrets.push(vaultSecretGroupInstance);
                  
                  var sourceVaultValue2 = secretsValue['sourceVault'];
                  if (sourceVaultValue2 !== null && sourceVaultValue2 !== undefined) {
                    var sourceVaultInstance = {};
                    vaultSecretGroupInstance.sourceVault = sourceVaultInstance;
                    
                    var idValue2 = sourceVaultValue2['id'];
                    if (idValue2 !== null && idValue2 !== undefined) {
                      var idInstance2 = idValue2;
                      sourceVaultInstance.referenceUri = idInstance2;
                    }
                  }
                  
                  var vaultCertificatesArray2 = secretsValue['vaultCertificates'];
                  if (vaultCertificatesArray2 !== null && vaultCertificatesArray2 !== undefined) {
                    for (var loweredIndex18 = 0; loweredIndex18 < vaultCertificatesArray2.length; loweredIndex18 = loweredIndex18 + 1) {
                      var vaultCertificatesValue = vaultCertificatesArray2[loweredIndex18];
                      var vaultCertificateInstance = {};
                      vaultSecretGroupInstance.vaultCertificates.push(vaultCertificateInstance);
                      
                      var certificateUrlValue2 = vaultCertificatesValue['certificateUrl'];
                      if (certificateUrlValue2 !== null && certificateUrlValue2 !== undefined) {
                        var certificateUrlInstance2 = certificateUrlValue2;
                        vaultCertificateInstance.certificateUrl = certificateUrlInstance2;
                      }
                      
                      var certificateStoreValue = vaultCertificatesValue['certificateStore'];
                      if (certificateStoreValue !== null && certificateStoreValue !== undefined) {
                        var certificateStoreInstance = certificateStoreValue;
                        vaultCertificateInstance.certificateStore = certificateStoreInstance;
                      }
                    }
                  }
                }
              }
            }
            
            var networkProfileValue2 = propertiesValue4['networkProfile'];
            if (networkProfileValue2 !== null && networkProfileValue2 !== undefined) {
              var networkProfileInstance = { networkInterfaces: [] };
              virtualMachineInstance.networkProfile = networkProfileInstance;
              
              var networkInterfacesArray2 = networkProfileValue2['networkInterfaces'];
              if (networkInterfacesArray2 !== null && networkInterfacesArray2 !== undefined) {
                for (var loweredIndex19 = 0; loweredIndex19 < networkInterfacesArray2.length; loweredIndex19 = loweredIndex19 + 1) {
                  var networkInterfacesValue = networkInterfacesArray2[loweredIndex19];
                  var networkInterfaceReferenceJsonInstance = {};
                  networkProfileInstance.networkInterfaces.push(networkInterfaceReferenceJsonInstance);
                  
                  var propertiesValue5 = networkInterfacesValue['properties'];
                  if (propertiesValue5 !== null && propertiesValue5 !== undefined) {
                    var primaryValue = propertiesValue5['primary'];
                    if (primaryValue !== null && primaryValue !== undefined) {
                      var primaryInstance = primaryValue;
                      networkInterfaceReferenceJsonInstance.primary = primaryInstance;
                    }
                  }
                  
                  var idValue3 = networkInterfacesValue['id'];
                  if (idValue3 !== null && idValue3 !== undefined) {
                    var idInstance3 = idValue3;
                    networkInterfaceReferenceJsonInstance.referenceUri = idInstance3;
                  }
                }
              }
            }
            
            var availabilitySetValue2 = propertiesValue4['availabilitySet'];
            if (availabilitySetValue2 !== null && availabilitySetValue2 !== undefined) {
              var availabilitySetInstance = {};
              virtualMachineInstance.availabilitySetReference = availabilitySetInstance;
              
              var idValue4 = availabilitySetValue2['id'];
              if (idValue4 !== null && idValue4 !== undefined) {
                var idInstance4 = idValue4;
                availabilitySetInstance.referenceUri = idInstance4;
              }
            }
            
            var provisioningStateValue = propertiesValue4['provisioningState'];
            if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
              var provisioningStateInstance = provisioningStateValue;
              virtualMachineInstance.provisioningState = provisioningStateInstance;
            }
          }
          
          var resourcesValue2 = responseDoc['resources'];
          if (resourcesValue2 !== null && resourcesValue2 !== undefined) {
            var resourcesInstance = { extensions: [] };
            virtualMachineInstance.resources = resourcesInstance;
            
            var extensionsArray2 = resourcesValue2['extensions'];
            if (extensionsArray2 !== null && extensionsArray2 !== undefined) {
              for (var loweredIndex20 = 0; loweredIndex20 < extensionsArray2.length; loweredIndex20 = loweredIndex20 + 1) {
                var extensionsValue = extensionsArray2[loweredIndex20];
                var virtualMachineExtensionJsonInstance = { tags: {} };
                resourcesInstance.extensions.push(virtualMachineExtensionJsonInstance);
                
                var propertiesValue6 = extensionsValue['properties'];
                if (propertiesValue6 !== null && propertiesValue6 !== undefined) {
                  var publisherValue2 = propertiesValue6['publisher'];
                  if (publisherValue2 !== null && publisherValue2 !== undefined) {
                    var publisherInstance2 = publisherValue2;
                    virtualMachineExtensionJsonInstance.publisher = publisherInstance2;
                  }
                  
                  var typeValue = propertiesValue6['type'];
                  if (typeValue !== null && typeValue !== undefined) {
                    var typeInstance = typeValue;
                    virtualMachineExtensionJsonInstance.extensionType = typeInstance;
                  }
                  
                  var typeHandlerVersionValue = propertiesValue6['typeHandlerVersion'];
                  if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                    var typeHandlerVersionInstance = typeHandlerVersionValue;
                    virtualMachineExtensionJsonInstance.typeHandlerVersion = typeHandlerVersionInstance;
                  }
                  
                  var settingsValue = propertiesValue6['settings'];
                  if (settingsValue !== null && settingsValue !== undefined) {
                    var settingsInstance = settingsValue;
                    virtualMachineExtensionJsonInstance.settings = settingsInstance;
                  }
                  
                  var protectedSettingsValue = propertiesValue6['protectedSettings'];
                  if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
                    var protectedSettingsInstance = protectedSettingsValue;
                    virtualMachineExtensionJsonInstance.protectedSettings = protectedSettingsInstance;
                  }
                  
                  var provisioningStateValue2 = propertiesValue6['provisioningState'];
                  if (provisioningStateValue2 !== null && provisioningStateValue2 !== undefined) {
                    var provisioningStateInstance2 = provisioningStateValue2;
                    virtualMachineExtensionJsonInstance.provisioningState = provisioningStateInstance2;
                  }
                  
                  var instanceViewValue2 = propertiesValue6['instanceView'];
                  if (instanceViewValue2 !== null && instanceViewValue2 !== undefined) {
                    var instanceViewInstance = { statuses: [], subStatuses: [] };
                    virtualMachineExtensionJsonInstance.instanceView = instanceViewInstance;
                    
                    var nameValue4 = instanceViewValue2['name'];
                    if (nameValue4 !== null && nameValue4 !== undefined) {
                      var nameInstance4 = nameValue4;
                      instanceViewInstance.name = nameInstance4;
                    }
                    
                    var typeValue2 = instanceViewValue2['type'];
                    if (typeValue2 !== null && typeValue2 !== undefined) {
                      var typeInstance2 = typeValue2;
                      instanceViewInstance.extensionType = typeInstance2;
                    }
                    
                    var typeHandlerVersionValue2 = instanceViewValue2['typeHandlerVersion'];
                    if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                      var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                      instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
                    }
                    
                    var substatusesArray2 = instanceViewValue2['substatuses'];
                    if (substatusesArray2 !== null && substatusesArray2 !== undefined) {
                      for (var loweredIndex21 = 0; loweredIndex21 < substatusesArray2.length; loweredIndex21 = loweredIndex21 + 1) {
                        var substatusesValue = substatusesArray2[loweredIndex21];
                        var instanceViewStatusInstance = {};
                        instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                        
                        var codeValue = substatusesValue['code'];
                        if (codeValue !== null && codeValue !== undefined) {
                          var codeInstance = codeValue;
                          instanceViewStatusInstance.code = codeInstance;
                        }
                        
                        var levelValue = substatusesValue['level'];
                        if (levelValue !== null && levelValue !== undefined) {
                          var levelInstance = levelValue;
                          instanceViewStatusInstance.level = levelInstance;
                        }
                        
                        var displayStatusValue = substatusesValue['displayStatus'];
                        if (displayStatusValue !== null && displayStatusValue !== undefined) {
                          var displayStatusInstance = displayStatusValue;
                          instanceViewStatusInstance.displayStatus = displayStatusInstance;
                        }
                        
                        var messageValue = substatusesValue['message'];
                        if (messageValue !== null && messageValue !== undefined) {
                          var messageInstance = messageValue;
                          instanceViewStatusInstance.message = messageInstance;
                        }
                        
                        var timeValue = substatusesValue['time'];
                        if (timeValue !== null && timeValue !== undefined) {
                          var timeInstance = timeValue;
                          instanceViewStatusInstance.time = timeInstance;
                        }
                      }
                    }
                    
                    var statusesArray2 = instanceViewValue2['statuses'];
                    if (statusesArray2 !== null && statusesArray2 !== undefined) {
                      for (var loweredIndex22 = 0; loweredIndex22 < statusesArray2.length; loweredIndex22 = loweredIndex22 + 1) {
                        var statusesValue = statusesArray2[loweredIndex22];
                        var instanceViewStatusInstance2 = {};
                        instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                        
                        var codeValue2 = statusesValue['code'];
                        if (codeValue2 !== null && codeValue2 !== undefined) {
                          var codeInstance2 = codeValue2;
                          instanceViewStatusInstance2.code = codeInstance2;
                        }
                        
                        var levelValue2 = statusesValue['level'];
                        if (levelValue2 !== null && levelValue2 !== undefined) {
                          var levelInstance2 = levelValue2;
                          instanceViewStatusInstance2.level = levelInstance2;
                        }
                        
                        var displayStatusValue2 = statusesValue['displayStatus'];
                        if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                          var displayStatusInstance2 = displayStatusValue2;
                          instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                        }
                        
                        var messageValue2 = statusesValue['message'];
                        if (messageValue2 !== null && messageValue2 !== undefined) {
                          var messageInstance2 = messageValue2;
                          instanceViewStatusInstance2.message = messageInstance2;
                        }
                        
                        var timeValue2 = statusesValue['time'];
                        if (timeValue2 !== null && timeValue2 !== undefined) {
                          var timeInstance2 = timeValue2;
                          instanceViewStatusInstance2.time = timeInstance2;
                        }
                      }
                    }
                  }
                }
                
                var idValue5 = extensionsValue['id'];
                if (idValue5 !== null && idValue5 !== undefined) {
                  var idInstance5 = idValue5;
                  virtualMachineExtensionJsonInstance.id = idInstance5;
                }
                
                var nameValue5 = extensionsValue['name'];
                if (nameValue5 !== null && nameValue5 !== undefined) {
                  var nameInstance5 = nameValue5;
                  virtualMachineExtensionJsonInstance.name = nameInstance5;
                }
                
                var typeValue3 = extensionsValue['type'];
                if (typeValue3 !== null && typeValue3 !== undefined) {
                  var typeInstance3 = typeValue3;
                  virtualMachineExtensionJsonInstance.type = typeInstance3;
                }
                
                var locationValue = extensionsValue['location'];
                if (locationValue !== null && locationValue !== undefined) {
                  var locationInstance = locationValue;
                  virtualMachineExtensionJsonInstance.location = locationInstance;
                }
                
                var tagsSequenceElement = extensionsValue['tags'];
                if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                  for (var property in tagsSequenceElement) {
                    var tagsKey3 = property;
                    var tagsValue3 = tagsSequenceElement[property];
                    virtualMachineExtensionJsonInstance.tags[tagsKey3] = tagsValue3;
                  }
                }
              }
            }
          }
          
          var idValue6 = responseDoc['id'];
          if (idValue6 !== null && idValue6 !== undefined) {
            var idInstance6 = idValue6;
            virtualMachineInstance.id = idInstance6;
          }
          
          var nameValue6 = responseDoc['name'];
          if (nameValue6 !== null && nameValue6 !== undefined) {
            var nameInstance6 = nameValue6;
            virtualMachineInstance.name = nameInstance6;
          }
          
          var typeValue4 = responseDoc['type'];
          if (typeValue4 !== null && typeValue4 !== undefined) {
            var typeInstance4 = typeValue4;
            virtualMachineInstance.type = typeInstance4;
          }
          
          var locationValue2 = responseDoc['location'];
          if (locationValue2 !== null && locationValue2 !== undefined) {
            var locationInstance2 = locationValue2;
            virtualMachineInstance.location = locationInstance2;
          }
          
          var tagsSequenceElement2 = responseDoc['tags'];
          if (tagsSequenceElement2 !== null && tagsSequenceElement2 !== undefined) {
            for (var property2 in tagsSequenceElement2) {
              var tagsKey4 = property2;
              var tagsValue4 = tagsSequenceElement2[property2];
              virtualMachineInstance.tags[tagsKey4] = tagsValue4;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Shuts down the Virtual Machine and releases the compute resources. You are
   * not billed for the compute resources that this Virtual Machine uses.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginDeallocating = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/deallocate';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to delete a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginDeleting = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to restart a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginRestarting = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/restart';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to start a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginStarting = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/start';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to stop a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.beginStopping = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/stop';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 202) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.azureAsyncOperation = response.headers['azure-asyncoperation'];
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Captures the VM by copying VirtualHardDisks of the VM and outputs a
   * template that can be used to create similar VM.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {VirtualMachineCaptureParameters} parameters Parameters supplied to
   * the Capture Virtual Machine operation.
   * 
   * @param {string} parameters.virtualHardDiskNamePrefix Gets or sets the
   * captured VirtualHardDisk's name prefix.
   * 
   * @param {string} parameters.destinationContainerName Gets or sets the
   * destination container name.
   * 
   * @param {boolean} parameters.overwrite Gets or sets whether it overwrites
   * destination VirtualHardDisk if true, in case of conflict.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.capture = function(resourceGroupName, vmName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginCapturing(resourceGroupName, vmName, parameters, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * The operation to create a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {VirtualMachine} parameters Parameters supplied to the Create
   * Virtual Machine operation.
   * 
   * @param {Plan} [parameters.plan] Gets or sets the purchase plan when
   * deploying virtual machine from VM Marketplace images.
   * 
   * @param {string} [parameters.plan.name] Gets or sets the plan ID.
   * 
   * @param {string} [parameters.plan.publisher] Gets or sets the publisher ID.
   * 
   * @param {string} [parameters.plan.product] Gets or sets the offer ID.
   * 
   * @param {string} [parameters.plan.promotionCode] Gets or sets the promotion
   * code.
   * 
   * @param {HardwareProfile} [parameters.hardwareProfile] Gets or sets the
   * hardware profile.
   * 
   * @param {string} [parameters.hardwareProfile.virtualMachineSize] The
   * virtual machine size name.
   * 
   * @param {StorageProfile} [parameters.storageProfile] Gets or sets the
   * storage profile.
   * 
   * @param {SourceImageReference} [parameters.storageProfile.sourceImage] Gets
   * or sets the source image reference.
   * 
   * @param {string} [parameters.storageProfile.sourceImage.referenceUri] Gets
   * or sets the relative URL in the previous Service Management API's
   * namespace. For Example:
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   * 
   * @param {string} [parameters.storageProfile.destinationVhdsContainer] Gets
   * or sets the destination container for VHDs.
   * 
   * @param {OSDisk} [parameters.storageProfile.oSDisk] Gets or sets the OS
   * disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.operatingSystemType]
   * Gets or sets the Operating System type.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.name] Gets or sets the
   * disk name.
   * 
   * @param {VirtualHardDisk}
   * [parameters.storageProfile.oSDisk.virtualHardDisk] Gets or sets the
   * Virtual Hard Disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.virtualHardDisk.uri]
   * Gets or sets the virtual hard disk's uri. It should be a valid Uri to a
   * virtual hard disk.
   * 
   * @param {VirtualHardDisk} [parameters.storageProfile.oSDisk.sourceImage]
   * Gets or sets the Source User Image VirtualHardDisk. This VirtualHardDisk
   * will be copied before using it to attach to the Virtual Machine.If
   * SourceImage is provided, the destination VirtualHardDisk should not exist.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.sourceImage.uri] Gets or
   * sets the virtual hard disk's uri. It should be a valid Uri to a virtual
   * hard disk.
   * 
   * @param {string} [parameters.storageProfile.oSDisk.caching] Gets or sets
   * the caching type.
   * 
   * @param {array} [parameters.storageProfile.dataDisks] Gets or sets the data
   * disks.
   * 
   * @param {OSProfile} [parameters.oSProfile] Gets or sets the OS profile.
   * 
   * @param {string} [parameters.oSProfile.computerName] Gets or sets the
   * computer name.
   * 
   * @param {string} [parameters.oSProfile.adminUsername] Gets or sets the
   * admin user name.
   * 
   * @param {string} [parameters.oSProfile.adminPassword] Gets or sets the
   * admin user password.
   * 
   * @param {string} [parameters.oSProfile.customData] Gets or sets a base-64
   * encoded string of custom data.
   * 
   * @param {WindowsConfiguration} [parameters.oSProfile.windowsConfiguration]
   * Gets or sets the Windows Configuration of the OS profile.
   * 
   * @param {boolean}
   * [parameters.oSProfile.windowsConfiguration.provisionVMAgent] Gets or sets
   * whether VM Agent should be provisioned on the Virtual Machine.
   * 
   * @param {boolean}
   * [parameters.oSProfile.windowsConfiguration.enableAutomaticUpdates] Gets
   * or sets whether Windows updates are automatically installed on the VM
   * 
   * @param {string} [parameters.oSProfile.windowsConfiguration.timeZone] Gets
   * or sets the Time Zone of the VM
   * 
   * @param {array}
   * [parameters.oSProfile.windowsConfiguration.additionalUnattendContents]
   * Gets or sets the additional base-64 encoded XML formatted information
   * that can be included in the Unattend.xml file.
   * 
   * @param {WinRMConfiguration}
   * [parameters.oSProfile.windowsConfiguration.winRMConfiguration] Gets or
   * sets the Windows Remote Management configuration of the VM
   * 
   * @param {array}
   * [parameters.oSProfile.windowsConfiguration.winRMConfiguration.listeners]
   * Gets or sets the list of Windows Remote Management listeners
   * 
   * @param {LinuxConfiguration} [parameters.oSProfile.linuxConfiguration] Gets
   * or sets the Linux Configuration of the OS profile.
   * 
   * @param {boolean}
   * [parameters.oSProfile.linuxConfiguration.disablePasswordAuthentication]
   * Gets or sets whether Authentication using user name and password is
   * allowed or not
   * 
   * @param {SshConfiguration}
   * [parameters.oSProfile.linuxConfiguration.sshConfiguration] Gets or sets
   * the SSH configuration for linux VMs
   * 
   * @param {array}
   * [parameters.oSProfile.linuxConfiguration.sshConfiguration.publicKeys]
   * Gets or sets the list of SSH public keys used to authenticate with linux
   * based VMs
   * 
   * @param {array} [parameters.oSProfile.secrets] Gets or sets the List of
   * certificates for addition to the VM.
   * 
   * @param {NetworkProfile} [parameters.networkProfile] Gets or sets the
   * network profile.
   * 
   * @param {array} [parameters.networkProfile.networkInterfaces] Gets or sets
   * the network interfaces.
   * 
   * @param {AvailabilitySetReference} [parameters.availabilitySetReference]
   * Gets or sets the reference Id of the availailbity set to which this
   * virtual machine belongs.
   * 
   * @param {string} [parameters.availabilitySetReference.referenceUri] Gets or
   * sets the relative URL in the previous Service Management API's namespace.
   * For Example:
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   * 
   * @param {string} [parameters.provisioningState] Gets or sets the
   * provisioning state, which only appears in the response.
   * 
   * @param {VirtualMachineChildResources} [parameters.resources] Gets or sets
   * the virtual machine child resources.
   * 
   * @param {array} [parameters.resources.extensions] Gets or sets the virtual
   * machine child extension resources.
   * 
   * @param {string} [parameters.id] Gets or sets the ID of the resource.
   * 
   * @param {string} [parameters.name] Gets or sets the name of the resource.
   * 
   * @param {string} [parameters.type] Gets or sets the type of the resource.
   * 
   * @param {string} parameters.location Gets or sets the location of the
   * resource.
   * 
   * @param {object} [parameters.tags] Gets or sets the tags attached to the
   * resource.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.createOrUpdate = function(resourceGroupName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginCreatingOrUpdating(resourceGroupName, parameters, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * Shuts down the Virtual Machine and releases the compute resources. You are
   * not billed for the compute resources that this Virtual Machine uses.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.deallocate = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginDeallocating(resourceGroupName, vmName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * The operation to create a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.deleteMethod = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginDeleting(resourceGroupName, vmName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * Sets the state of the VM as Generalized.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.generalize = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/generalize';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to get a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.get = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var virtualMachineInstance = { tags: {} };
          result.virtualMachine = virtualMachineInstance;
          
          var planValue = responseDoc['plan'];
          if (planValue !== null && planValue !== undefined) {
            var planInstance = {};
            virtualMachineInstance.plan = planInstance;
            
            var nameValue = planValue['name'];
            if (nameValue !== null && nameValue !== undefined) {
              var nameInstance = nameValue;
              planInstance.name = nameInstance;
            }
            
            var publisherValue = planValue['publisher'];
            if (publisherValue !== null && publisherValue !== undefined) {
              var publisherInstance = publisherValue;
              planInstance.publisher = publisherInstance;
            }
            
            var productValue = planValue['product'];
            if (productValue !== null && productValue !== undefined) {
              var productInstance = productValue;
              planInstance.product = productInstance;
            }
            
            var promotionCodeValue = planValue['promotionCode'];
            if (promotionCodeValue !== null && promotionCodeValue !== undefined) {
              var promotionCodeInstance = promotionCodeValue;
              planInstance.promotionCode = promotionCodeInstance;
            }
          }
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var hardwareProfileValue = propertiesValue['hardwareProfile'];
            if (hardwareProfileValue !== null && hardwareProfileValue !== undefined) {
              var hardwareProfileInstance = {};
              virtualMachineInstance.hardwareProfile = hardwareProfileInstance;
              
              var vmSizeValue = hardwareProfileValue['vmSize'];
              if (vmSizeValue !== null && vmSizeValue !== undefined) {
                var vmSizeInstance = vmSizeValue;
                hardwareProfileInstance.virtualMachineSize = vmSizeInstance;
              }
            }
            
            var storageProfileValue = propertiesValue['storageProfile'];
            if (storageProfileValue !== null && storageProfileValue !== undefined) {
              var storageProfileInstance = { dataDisks: [] };
              virtualMachineInstance.storageProfile = storageProfileInstance;
              
              var sourceImageValue = storageProfileValue['sourceImage'];
              if (sourceImageValue !== null && sourceImageValue !== undefined) {
                var sourceImageInstance = {};
                storageProfileInstance.sourceImage = sourceImageInstance;
                
                var idValue = sourceImageValue['id'];
                if (idValue !== null && idValue !== undefined) {
                  var idInstance = idValue;
                  sourceImageInstance.referenceUri = idInstance;
                }
              }
              
              var destinationVhdsContainerValue = storageProfileValue['destinationVhdsContainer'];
              if (destinationVhdsContainerValue !== null && destinationVhdsContainerValue !== undefined) {
                var destinationVhdsContainerInstance = destinationVhdsContainerValue;
                storageProfileInstance.destinationVhdsContainer = destinationVhdsContainerInstance;
              }
              
              var osDiskValue = storageProfileValue['osDisk'];
              if (osDiskValue !== null && osDiskValue !== undefined) {
                var osDiskInstance = {};
                storageProfileInstance.oSDisk = osDiskInstance;
                
                var osTypeValue = osDiskValue['osType'];
                if (osTypeValue !== null && osTypeValue !== undefined) {
                  var osTypeInstance = osTypeValue;
                  osDiskInstance.operatingSystemType = osTypeInstance;
                }
                
                var nameValue2 = osDiskValue['name'];
                if (nameValue2 !== null && nameValue2 !== undefined) {
                  var nameInstance2 = nameValue2;
                  osDiskInstance.name = nameInstance2;
                }
                
                var vhdValue = osDiskValue['vhd'];
                if (vhdValue !== null && vhdValue !== undefined) {
                  var vhdInstance = {};
                  osDiskInstance.virtualHardDisk = vhdInstance;
                  
                  var uriValue = vhdValue['uri'];
                  if (uriValue !== null && uriValue !== undefined) {
                    var uriInstance = uriValue;
                    vhdInstance.uri = uriInstance;
                  }
                }
                
                var imageValue = osDiskValue['image'];
                if (imageValue !== null && imageValue !== undefined) {
                  var imageInstance = {};
                  osDiskInstance.sourceImage = imageInstance;
                  
                  var uriValue2 = imageValue['uri'];
                  if (uriValue2 !== null && uriValue2 !== undefined) {
                    var uriInstance2 = uriValue2;
                    imageInstance.uri = uriInstance2;
                  }
                }
                
                var cachingValue = osDiskValue['caching'];
                if (cachingValue !== null && cachingValue !== undefined) {
                  var cachingInstance = cachingValue;
                  osDiskInstance.caching = cachingInstance;
                }
              }
              
              var dataDisksArray = storageProfileValue['dataDisks'];
              if (dataDisksArray !== null && dataDisksArray !== undefined) {
                for (var loweredIndex1 = 0; loweredIndex1 < dataDisksArray.length; loweredIndex1 = loweredIndex1 + 1) {
                  var dataDisksValue = dataDisksArray[loweredIndex1];
                  var dataDiskInstance = {};
                  storageProfileInstance.dataDisks.push(dataDiskInstance);
                  
                  var lunValue = dataDisksValue['lun'];
                  if (lunValue !== null && lunValue !== undefined) {
                    var lunInstance = lunValue;
                    dataDiskInstance.lun = lunInstance;
                  }
                  
                  var diskSizeGBValue = dataDisksValue['diskSizeGB'];
                  if (diskSizeGBValue !== null && diskSizeGBValue !== undefined) {
                    var diskSizeGBInstance = diskSizeGBValue;
                    dataDiskInstance.diskSizeGB = diskSizeGBInstance;
                  }
                  
                  var nameValue3 = dataDisksValue['name'];
                  if (nameValue3 !== null && nameValue3 !== undefined) {
                    var nameInstance3 = nameValue3;
                    dataDiskInstance.name = nameInstance3;
                  }
                  
                  var vhdValue2 = dataDisksValue['vhd'];
                  if (vhdValue2 !== null && vhdValue2 !== undefined) {
                    var vhdInstance2 = {};
                    dataDiskInstance.virtualHardDisk = vhdInstance2;
                    
                    var uriValue3 = vhdValue2['uri'];
                    if (uriValue3 !== null && uriValue3 !== undefined) {
                      var uriInstance3 = uriValue3;
                      vhdInstance2.uri = uriInstance3;
                    }
                  }
                  
                  var imageValue2 = dataDisksValue['image'];
                  if (imageValue2 !== null && imageValue2 !== undefined) {
                    var imageInstance2 = {};
                    dataDiskInstance.sourceImage = imageInstance2;
                    
                    var uriValue4 = imageValue2['uri'];
                    if (uriValue4 !== null && uriValue4 !== undefined) {
                      var uriInstance4 = uriValue4;
                      imageInstance2.uri = uriInstance4;
                    }
                  }
                  
                  var cachingValue2 = dataDisksValue['caching'];
                  if (cachingValue2 !== null && cachingValue2 !== undefined) {
                    var cachingInstance2 = cachingValue2;
                    dataDiskInstance.caching = cachingInstance2;
                  }
                }
              }
            }
            
            var osProfileValue = propertiesValue['osProfile'];
            if (osProfileValue !== null && osProfileValue !== undefined) {
              var osProfileInstance = { secrets: [] };
              virtualMachineInstance.oSProfile = osProfileInstance;
              
              var computerNameValue = osProfileValue['computerName'];
              if (computerNameValue !== null && computerNameValue !== undefined) {
                var computerNameInstance = computerNameValue;
                osProfileInstance.computerName = computerNameInstance;
              }
              
              var adminUsernameValue = osProfileValue['adminUsername'];
              if (adminUsernameValue !== null && adminUsernameValue !== undefined) {
                var adminUsernameInstance = adminUsernameValue;
                osProfileInstance.adminUsername = adminUsernameInstance;
              }
              
              var adminPasswordValue = osProfileValue['adminPassword'];
              if (adminPasswordValue !== null && adminPasswordValue !== undefined) {
                var adminPasswordInstance = adminPasswordValue;
                osProfileInstance.adminPassword = adminPasswordInstance;
              }
              
              var customDataValue = osProfileValue['customData'];
              if (customDataValue !== null && customDataValue !== undefined) {
                var customDataInstance = customDataValue;
                osProfileInstance.customData = customDataInstance;
              }
              
              var windowsConfigurationValue = osProfileValue['windowsConfiguration'];
              if (windowsConfigurationValue !== null && windowsConfigurationValue !== undefined) {
                var windowsConfigurationInstance = { additionalUnattendContents: [] };
                osProfileInstance.windowsConfiguration = windowsConfigurationInstance;
                
                var provisionVMAgentValue = windowsConfigurationValue['provisionVMAgent'];
                if (provisionVMAgentValue !== null && provisionVMAgentValue !== undefined) {
                  var provisionVMAgentInstance = provisionVMAgentValue;
                  windowsConfigurationInstance.provisionVMAgent = provisionVMAgentInstance;
                }
                
                var enableAutomaticUpdatesValue = windowsConfigurationValue['enableAutomaticUpdates'];
                if (enableAutomaticUpdatesValue !== null && enableAutomaticUpdatesValue !== undefined) {
                  var enableAutomaticUpdatesInstance = enableAutomaticUpdatesValue;
                  windowsConfigurationInstance.enableAutomaticUpdates = enableAutomaticUpdatesInstance;
                }
                
                var timeZoneValue = windowsConfigurationValue['timeZone'];
                if (timeZoneValue !== null && timeZoneValue !== undefined) {
                  var timeZoneInstance = timeZoneValue;
                  windowsConfigurationInstance.timeZone = timeZoneInstance;
                }
                
                var additionalUnattendContentArray = windowsConfigurationValue['additionalUnattendContent'];
                if (additionalUnattendContentArray !== null && additionalUnattendContentArray !== undefined) {
                  for (var loweredIndex2 = 0; loweredIndex2 < additionalUnattendContentArray.length; loweredIndex2 = loweredIndex2 + 1) {
                    var additionalUnattendContentValue = additionalUnattendContentArray[loweredIndex2];
                    var additionalUnattendContentInstance = {};
                    windowsConfigurationInstance.additionalUnattendContents.push(additionalUnattendContentInstance);
                    
                    var passNameValue = additionalUnattendContentValue['passName'];
                    if (passNameValue !== null && passNameValue !== undefined) {
                      var passNameInstance = passNameValue;
                      additionalUnattendContentInstance.passName = passNameInstance;
                    }
                    
                    var componentNameValue = additionalUnattendContentValue['componentName'];
                    if (componentNameValue !== null && componentNameValue !== undefined) {
                      var componentNameInstance = componentNameValue;
                      additionalUnattendContentInstance.componentName = componentNameInstance;
                    }
                    
                    var settingNameValue = additionalUnattendContentValue['settingName'];
                    if (settingNameValue !== null && settingNameValue !== undefined) {
                      var settingNameInstance = settingNameValue;
                      additionalUnattendContentInstance.settingName = settingNameInstance;
                    }
                    
                    var contentValue = additionalUnattendContentValue['content'];
                    if (contentValue !== null && contentValue !== undefined) {
                      var contentInstance = contentValue;
                      additionalUnattendContentInstance.content = contentInstance;
                    }
                  }
                }
                
                var winRMValue = windowsConfigurationValue['winRM'];
                if (winRMValue !== null && winRMValue !== undefined) {
                  var winRMInstance = { listeners: [] };
                  windowsConfigurationInstance.winRMConfiguration = winRMInstance;
                  
                  var listenersArray = winRMValue['listeners'];
                  if (listenersArray !== null && listenersArray !== undefined) {
                    for (var loweredIndex3 = 0; loweredIndex3 < listenersArray.length; loweredIndex3 = loweredIndex3 + 1) {
                      var listenersValue = listenersArray[loweredIndex3];
                      var winRMListenerInstance = {};
                      winRMInstance.listeners.push(winRMListenerInstance);
                      
                      var protocolValue = listenersValue['protocol'];
                      if (protocolValue !== null && protocolValue !== undefined) {
                        var protocolInstance = protocolValue;
                        winRMListenerInstance.protocol = protocolInstance;
                      }
                      
                      var certificateUrlValue = listenersValue['certificateUrl'];
                      if (certificateUrlValue !== null && certificateUrlValue !== undefined) {
                        var certificateUrlInstance = certificateUrlValue;
                        winRMListenerInstance.certificateUrl = certificateUrlInstance;
                      }
                    }
                  }
                }
              }
              
              var linuxConfigurationValue = osProfileValue['linuxConfiguration'];
              if (linuxConfigurationValue !== null && linuxConfigurationValue !== undefined) {
                var linuxConfigurationInstance = {};
                osProfileInstance.linuxConfiguration = linuxConfigurationInstance;
                
                var disablePasswordAuthenticationValue = linuxConfigurationValue['disablePasswordAuthentication'];
                if (disablePasswordAuthenticationValue !== null && disablePasswordAuthenticationValue !== undefined) {
                  var disablePasswordAuthenticationInstance = disablePasswordAuthenticationValue;
                  linuxConfigurationInstance.disablePasswordAuthentication = disablePasswordAuthenticationInstance;
                }
                
                var sshValue = linuxConfigurationValue['ssh'];
                if (sshValue !== null && sshValue !== undefined) {
                  var sshInstance = { publicKeys: [] };
                  linuxConfigurationInstance.sshConfiguration = sshInstance;
                  
                  var publicKeysArray = sshValue['publicKeys'];
                  if (publicKeysArray !== null && publicKeysArray !== undefined) {
                    for (var loweredIndex4 = 0; loweredIndex4 < publicKeysArray.length; loweredIndex4 = loweredIndex4 + 1) {
                      var publicKeysValue = publicKeysArray[loweredIndex4];
                      var sshPublicKeyInstance = {};
                      sshInstance.publicKeys.push(sshPublicKeyInstance);
                      
                      var pathValue = publicKeysValue['path'];
                      if (pathValue !== null && pathValue !== undefined) {
                        var pathInstance = pathValue;
                        sshPublicKeyInstance.path = pathInstance;
                      }
                      
                      var keyDataValue = publicKeysValue['keyData'];
                      if (keyDataValue !== null && keyDataValue !== undefined) {
                        var keyDataInstance = keyDataValue;
                        sshPublicKeyInstance.keyData = keyDataInstance;
                      }
                    }
                  }
                }
              }
              
              var secretsArray = osProfileValue['secrets'];
              if (secretsArray !== null && secretsArray !== undefined) {
                for (var loweredIndex5 = 0; loweredIndex5 < secretsArray.length; loweredIndex5 = loweredIndex5 + 1) {
                  var secretsValue = secretsArray[loweredIndex5];
                  var vaultSecretGroupInstance = { vaultCertificates: [] };
                  osProfileInstance.secrets.push(vaultSecretGroupInstance);
                  
                  var sourceVaultValue = secretsValue['sourceVault'];
                  if (sourceVaultValue !== null && sourceVaultValue !== undefined) {
                    var sourceVaultInstance = {};
                    vaultSecretGroupInstance.sourceVault = sourceVaultInstance;
                    
                    var idValue2 = sourceVaultValue['id'];
                    if (idValue2 !== null && idValue2 !== undefined) {
                      var idInstance2 = idValue2;
                      sourceVaultInstance.referenceUri = idInstance2;
                    }
                  }
                  
                  var vaultCertificatesArray = secretsValue['vaultCertificates'];
                  if (vaultCertificatesArray !== null && vaultCertificatesArray !== undefined) {
                    for (var loweredIndex6 = 0; loweredIndex6 < vaultCertificatesArray.length; loweredIndex6 = loweredIndex6 + 1) {
                      var vaultCertificatesValue = vaultCertificatesArray[loweredIndex6];
                      var vaultCertificateInstance = {};
                      vaultSecretGroupInstance.vaultCertificates.push(vaultCertificateInstance);
                      
                      var certificateUrlValue2 = vaultCertificatesValue['certificateUrl'];
                      if (certificateUrlValue2 !== null && certificateUrlValue2 !== undefined) {
                        var certificateUrlInstance2 = certificateUrlValue2;
                        vaultCertificateInstance.certificateUrl = certificateUrlInstance2;
                      }
                      
                      var certificateStoreValue = vaultCertificatesValue['certificateStore'];
                      if (certificateStoreValue !== null && certificateStoreValue !== undefined) {
                        var certificateStoreInstance = certificateStoreValue;
                        vaultCertificateInstance.certificateStore = certificateStoreInstance;
                      }
                    }
                  }
                }
              }
            }
            
            var networkProfileValue = propertiesValue['networkProfile'];
            if (networkProfileValue !== null && networkProfileValue !== undefined) {
              var networkProfileInstance = { networkInterfaces: [] };
              virtualMachineInstance.networkProfile = networkProfileInstance;
              
              var networkInterfacesArray = networkProfileValue['networkInterfaces'];
              if (networkInterfacesArray !== null && networkInterfacesArray !== undefined) {
                for (var loweredIndex7 = 0; loweredIndex7 < networkInterfacesArray.length; loweredIndex7 = loweredIndex7 + 1) {
                  var networkInterfacesValue = networkInterfacesArray[loweredIndex7];
                  var networkInterfaceReferenceJsonInstance = {};
                  networkProfileInstance.networkInterfaces.push(networkInterfaceReferenceJsonInstance);
                  
                  var propertiesValue2 = networkInterfacesValue['properties'];
                  if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
                    var primaryValue = propertiesValue2['primary'];
                    if (primaryValue !== null && primaryValue !== undefined) {
                      var primaryInstance = primaryValue;
                      networkInterfaceReferenceJsonInstance.primary = primaryInstance;
                    }
                  }
                  
                  var idValue3 = networkInterfacesValue['id'];
                  if (idValue3 !== null && idValue3 !== undefined) {
                    var idInstance3 = idValue3;
                    networkInterfaceReferenceJsonInstance.referenceUri = idInstance3;
                  }
                }
              }
            }
            
            var availabilitySetValue = propertiesValue['availabilitySet'];
            if (availabilitySetValue !== null && availabilitySetValue !== undefined) {
              var availabilitySetInstance = {};
              virtualMachineInstance.availabilitySetReference = availabilitySetInstance;
              
              var idValue4 = availabilitySetValue['id'];
              if (idValue4 !== null && idValue4 !== undefined) {
                var idInstance4 = idValue4;
                availabilitySetInstance.referenceUri = idInstance4;
              }
            }
            
            var provisioningStateValue = propertiesValue['provisioningState'];
            if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
              var provisioningStateInstance = provisioningStateValue;
              virtualMachineInstance.provisioningState = provisioningStateInstance;
            }
          }
          
          var resourcesValue = responseDoc['resources'];
          if (resourcesValue !== null && resourcesValue !== undefined) {
            var resourcesInstance = { extensions: [] };
            virtualMachineInstance.resources = resourcesInstance;
            
            var extensionsArray = resourcesValue['extensions'];
            if (extensionsArray !== null && extensionsArray !== undefined) {
              for (var loweredIndex8 = 0; loweredIndex8 < extensionsArray.length; loweredIndex8 = loweredIndex8 + 1) {
                var extensionsValue = extensionsArray[loweredIndex8];
                var virtualMachineExtensionJsonInstance = { tags: {} };
                resourcesInstance.extensions.push(virtualMachineExtensionJsonInstance);
                
                var propertiesValue3 = extensionsValue['properties'];
                if (propertiesValue3 !== null && propertiesValue3 !== undefined) {
                  var publisherValue2 = propertiesValue3['publisher'];
                  if (publisherValue2 !== null && publisherValue2 !== undefined) {
                    var publisherInstance2 = publisherValue2;
                    virtualMachineExtensionJsonInstance.publisher = publisherInstance2;
                  }
                  
                  var typeValue = propertiesValue3['type'];
                  if (typeValue !== null && typeValue !== undefined) {
                    var typeInstance = typeValue;
                    virtualMachineExtensionJsonInstance.extensionType = typeInstance;
                  }
                  
                  var typeHandlerVersionValue = propertiesValue3['typeHandlerVersion'];
                  if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                    var typeHandlerVersionInstance = typeHandlerVersionValue;
                    virtualMachineExtensionJsonInstance.typeHandlerVersion = typeHandlerVersionInstance;
                  }
                  
                  var settingsValue = propertiesValue3['settings'];
                  if (settingsValue !== null && settingsValue !== undefined) {
                    var settingsInstance = settingsValue;
                    virtualMachineExtensionJsonInstance.settings = settingsInstance;
                  }
                  
                  var protectedSettingsValue = propertiesValue3['protectedSettings'];
                  if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
                    var protectedSettingsInstance = protectedSettingsValue;
                    virtualMachineExtensionJsonInstance.protectedSettings = protectedSettingsInstance;
                  }
                  
                  var provisioningStateValue2 = propertiesValue3['provisioningState'];
                  if (provisioningStateValue2 !== null && provisioningStateValue2 !== undefined) {
                    var provisioningStateInstance2 = provisioningStateValue2;
                    virtualMachineExtensionJsonInstance.provisioningState = provisioningStateInstance2;
                  }
                  
                  var instanceViewValue = propertiesValue3['instanceView'];
                  if (instanceViewValue !== null && instanceViewValue !== undefined) {
                    var instanceViewInstance = { statuses: [], subStatuses: [] };
                    virtualMachineExtensionJsonInstance.instanceView = instanceViewInstance;
                    
                    var nameValue4 = instanceViewValue['name'];
                    if (nameValue4 !== null && nameValue4 !== undefined) {
                      var nameInstance4 = nameValue4;
                      instanceViewInstance.name = nameInstance4;
                    }
                    
                    var typeValue2 = instanceViewValue['type'];
                    if (typeValue2 !== null && typeValue2 !== undefined) {
                      var typeInstance2 = typeValue2;
                      instanceViewInstance.extensionType = typeInstance2;
                    }
                    
                    var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
                    if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                      var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                      instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
                    }
                    
                    var substatusesArray = instanceViewValue['substatuses'];
                    if (substatusesArray !== null && substatusesArray !== undefined) {
                      for (var loweredIndex9 = 0; loweredIndex9 < substatusesArray.length; loweredIndex9 = loweredIndex9 + 1) {
                        var substatusesValue = substatusesArray[loweredIndex9];
                        var instanceViewStatusInstance = {};
                        instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                        
                        var codeValue = substatusesValue['code'];
                        if (codeValue !== null && codeValue !== undefined) {
                          var codeInstance = codeValue;
                          instanceViewStatusInstance.code = codeInstance;
                        }
                        
                        var levelValue = substatusesValue['level'];
                        if (levelValue !== null && levelValue !== undefined) {
                          var levelInstance = levelValue;
                          instanceViewStatusInstance.level = levelInstance;
                        }
                        
                        var displayStatusValue = substatusesValue['displayStatus'];
                        if (displayStatusValue !== null && displayStatusValue !== undefined) {
                          var displayStatusInstance = displayStatusValue;
                          instanceViewStatusInstance.displayStatus = displayStatusInstance;
                        }
                        
                        var messageValue = substatusesValue['message'];
                        if (messageValue !== null && messageValue !== undefined) {
                          var messageInstance = messageValue;
                          instanceViewStatusInstance.message = messageInstance;
                        }
                        
                        var timeValue = substatusesValue['time'];
                        if (timeValue !== null && timeValue !== undefined) {
                          var timeInstance = timeValue;
                          instanceViewStatusInstance.time = timeInstance;
                        }
                      }
                    }
                    
                    var statusesArray = instanceViewValue['statuses'];
                    if (statusesArray !== null && statusesArray !== undefined) {
                      for (var loweredIndex10 = 0; loweredIndex10 < statusesArray.length; loweredIndex10 = loweredIndex10 + 1) {
                        var statusesValue = statusesArray[loweredIndex10];
                        var instanceViewStatusInstance2 = {};
                        instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                        
                        var codeValue2 = statusesValue['code'];
                        if (codeValue2 !== null && codeValue2 !== undefined) {
                          var codeInstance2 = codeValue2;
                          instanceViewStatusInstance2.code = codeInstance2;
                        }
                        
                        var levelValue2 = statusesValue['level'];
                        if (levelValue2 !== null && levelValue2 !== undefined) {
                          var levelInstance2 = levelValue2;
                          instanceViewStatusInstance2.level = levelInstance2;
                        }
                        
                        var displayStatusValue2 = statusesValue['displayStatus'];
                        if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                          var displayStatusInstance2 = displayStatusValue2;
                          instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                        }
                        
                        var messageValue2 = statusesValue['message'];
                        if (messageValue2 !== null && messageValue2 !== undefined) {
                          var messageInstance2 = messageValue2;
                          instanceViewStatusInstance2.message = messageInstance2;
                        }
                        
                        var timeValue2 = statusesValue['time'];
                        if (timeValue2 !== null && timeValue2 !== undefined) {
                          var timeInstance2 = timeValue2;
                          instanceViewStatusInstance2.time = timeInstance2;
                        }
                      }
                    }
                  }
                }
                
                var idValue5 = extensionsValue['id'];
                if (idValue5 !== null && idValue5 !== undefined) {
                  var idInstance5 = idValue5;
                  virtualMachineExtensionJsonInstance.id = idInstance5;
                }
                
                var nameValue5 = extensionsValue['name'];
                if (nameValue5 !== null && nameValue5 !== undefined) {
                  var nameInstance5 = nameValue5;
                  virtualMachineExtensionJsonInstance.name = nameInstance5;
                }
                
                var typeValue3 = extensionsValue['type'];
                if (typeValue3 !== null && typeValue3 !== undefined) {
                  var typeInstance3 = typeValue3;
                  virtualMachineExtensionJsonInstance.type = typeInstance3;
                }
                
                var locationValue = extensionsValue['location'];
                if (locationValue !== null && locationValue !== undefined) {
                  var locationInstance = locationValue;
                  virtualMachineExtensionJsonInstance.location = locationInstance;
                }
                
                var tagsSequenceElement = extensionsValue['tags'];
                if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                  for (var property in tagsSequenceElement) {
                    var tagsKey = property;
                    var tagsValue = tagsSequenceElement[property];
                    virtualMachineExtensionJsonInstance.tags[tagsKey] = tagsValue;
                  }
                }
              }
            }
          }
          
          var idValue6 = responseDoc['id'];
          if (idValue6 !== null && idValue6 !== undefined) {
            var idInstance6 = idValue6;
            virtualMachineInstance.id = idInstance6;
          }
          
          var nameValue6 = responseDoc['name'];
          if (nameValue6 !== null && nameValue6 !== undefined) {
            var nameInstance6 = nameValue6;
            virtualMachineInstance.name = nameInstance6;
          }
          
          var typeValue4 = responseDoc['type'];
          if (typeValue4 !== null && typeValue4 !== undefined) {
            var typeInstance4 = typeValue4;
            virtualMachineInstance.type = typeInstance4;
          }
          
          var locationValue2 = responseDoc['location'];
          if (locationValue2 !== null && locationValue2 !== undefined) {
            var locationInstance2 = locationValue2;
            virtualMachineInstance.location = locationInstance2;
          }
          
          var tagsSequenceElement2 = responseDoc['tags'];
          if (tagsSequenceElement2 !== null && tagsSequenceElement2 !== undefined) {
            for (var property2 in tagsSequenceElement2) {
              var tagsKey2 = property2;
              var tagsValue2 = tagsSequenceElement2[property2];
              virtualMachineInstance.tags[tagsKey2] = tagsValue2;
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to get the instance view of a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.getInstanceView = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/InstanceView';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var instanceViewInstance = { disks: [], extensions: [], statuses: [] };
          result.instanceView = instanceViewInstance;
          
          var platformUpdateDomainValue = responseDoc['platformUpdateDomain'];
          if (platformUpdateDomainValue !== null && platformUpdateDomainValue !== undefined) {
            var platformUpdateDomainInstance = platformUpdateDomainValue;
            instanceViewInstance.platformUpdateDomain = platformUpdateDomainInstance;
          }
          
          var platformFaultDomainValue = responseDoc['platformFaultDomain'];
          if (platformFaultDomainValue !== null && platformFaultDomainValue !== undefined) {
            var platformFaultDomainInstance = platformFaultDomainValue;
            instanceViewInstance.platformFaultDomain = platformFaultDomainInstance;
          }
          
          var rdpThumbPrintValue = responseDoc['rdpThumbPrint'];
          if (rdpThumbPrintValue !== null && rdpThumbPrintValue !== undefined) {
            var rdpThumbPrintInstance = rdpThumbPrintValue;
            instanceViewInstance.remoteDesktopThumbprint = rdpThumbPrintInstance;
          }
          
          var vmAgentValue = responseDoc['vmAgent'];
          if (vmAgentValue !== null && vmAgentValue !== undefined) {
            var vmAgentInstance = { extensionHandlers: [], statuses: [] };
            instanceViewInstance.vMAgent = vmAgentInstance;
            
            var vmAgentVersionValue = vmAgentValue['vmAgentVersion'];
            if (vmAgentVersionValue !== null && vmAgentVersionValue !== undefined) {
              var vmAgentVersionInstance = vmAgentVersionValue;
              vmAgentInstance.vMAgentVersion = vmAgentVersionInstance;
            }
            
            var extensionHandlersArray = vmAgentValue['extensionHandlers'];
            if (extensionHandlersArray !== null && extensionHandlersArray !== undefined) {
              for (var loweredIndex1 = 0; loweredIndex1 < extensionHandlersArray.length; loweredIndex1 = loweredIndex1 + 1) {
                var extensionHandlersValue = extensionHandlersArray[loweredIndex1];
                var virtualMachineExtensionHandlerInstanceViewInstance = {};
                vmAgentInstance.extensionHandlers.push(virtualMachineExtensionHandlerInstanceViewInstance);
                
                var typeValue = extensionHandlersValue['type'];
                if (typeValue !== null && typeValue !== undefined) {
                  var typeInstance = typeValue;
                  virtualMachineExtensionHandlerInstanceViewInstance.type = typeInstance;
                }
                
                var typeHandlerVersionValue = extensionHandlersValue['typeHandlerVersion'];
                if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                  var typeHandlerVersionInstance = typeHandlerVersionValue;
                  virtualMachineExtensionHandlerInstanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance;
                }
                
                var statusValue = extensionHandlersValue['status'];
                if (statusValue !== null && statusValue !== undefined) {
                  var statusInstance = {};
                  virtualMachineExtensionHandlerInstanceViewInstance.status = statusInstance;
                  
                  var codeValue = statusValue['code'];
                  if (codeValue !== null && codeValue !== undefined) {
                    var codeInstance = codeValue;
                    statusInstance.code = codeInstance;
                  }
                  
                  var levelValue = statusValue['level'];
                  if (levelValue !== null && levelValue !== undefined) {
                    var levelInstance = levelValue;
                    statusInstance.level = levelInstance;
                  }
                  
                  var displayStatusValue = statusValue['displayStatus'];
                  if (displayStatusValue !== null && displayStatusValue !== undefined) {
                    var displayStatusInstance = displayStatusValue;
                    statusInstance.displayStatus = displayStatusInstance;
                  }
                  
                  var messageValue = statusValue['message'];
                  if (messageValue !== null && messageValue !== undefined) {
                    var messageInstance = messageValue;
                    statusInstance.message = messageInstance;
                  }
                  
                  var timeValue = statusValue['time'];
                  if (timeValue !== null && timeValue !== undefined) {
                    var timeInstance = timeValue;
                    statusInstance.time = timeInstance;
                  }
                }
              }
            }
            
            var statusesArray = vmAgentValue['statuses'];
            if (statusesArray !== null && statusesArray !== undefined) {
              for (var loweredIndex2 = 0; loweredIndex2 < statusesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                var statusesValue = statusesArray[loweredIndex2];
                var instanceViewStatusInstance = {};
                vmAgentInstance.statuses.push(instanceViewStatusInstance);
                
                var codeValue2 = statusesValue['code'];
                if (codeValue2 !== null && codeValue2 !== undefined) {
                  var codeInstance2 = codeValue2;
                  instanceViewStatusInstance.code = codeInstance2;
                }
                
                var levelValue2 = statusesValue['level'];
                if (levelValue2 !== null && levelValue2 !== undefined) {
                  var levelInstance2 = levelValue2;
                  instanceViewStatusInstance.level = levelInstance2;
                }
                
                var displayStatusValue2 = statusesValue['displayStatus'];
                if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                  var displayStatusInstance2 = displayStatusValue2;
                  instanceViewStatusInstance.displayStatus = displayStatusInstance2;
                }
                
                var messageValue2 = statusesValue['message'];
                if (messageValue2 !== null && messageValue2 !== undefined) {
                  var messageInstance2 = messageValue2;
                  instanceViewStatusInstance.message = messageInstance2;
                }
                
                var timeValue2 = statusesValue['time'];
                if (timeValue2 !== null && timeValue2 !== undefined) {
                  var timeInstance2 = timeValue2;
                  instanceViewStatusInstance.time = timeInstance2;
                }
              }
            }
          }
          
          var disksArray = responseDoc['disks'];
          if (disksArray !== null && disksArray !== undefined) {
            for (var loweredIndex3 = 0; loweredIndex3 < disksArray.length; loweredIndex3 = loweredIndex3 + 1) {
              var disksValue = disksArray[loweredIndex3];
              var diskInstanceViewInstance = { statuses: [] };
              instanceViewInstance.disks.push(diskInstanceViewInstance);
              
              var nameValue = disksValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                diskInstanceViewInstance.name = nameInstance;
              }
              
              var statusesArray2 = disksValue['statuses'];
              if (statusesArray2 !== null && statusesArray2 !== undefined) {
                for (var loweredIndex4 = 0; loweredIndex4 < statusesArray2.length; loweredIndex4 = loweredIndex4 + 1) {
                  var statusesValue2 = statusesArray2[loweredIndex4];
                  var instanceViewStatusInstance2 = {};
                  diskInstanceViewInstance.statuses.push(instanceViewStatusInstance2);
                  
                  var codeValue3 = statusesValue2['code'];
                  if (codeValue3 !== null && codeValue3 !== undefined) {
                    var codeInstance3 = codeValue3;
                    instanceViewStatusInstance2.code = codeInstance3;
                  }
                  
                  var levelValue3 = statusesValue2['level'];
                  if (levelValue3 !== null && levelValue3 !== undefined) {
                    var levelInstance3 = levelValue3;
                    instanceViewStatusInstance2.level = levelInstance3;
                  }
                  
                  var displayStatusValue3 = statusesValue2['displayStatus'];
                  if (displayStatusValue3 !== null && displayStatusValue3 !== undefined) {
                    var displayStatusInstance3 = displayStatusValue3;
                    instanceViewStatusInstance2.displayStatus = displayStatusInstance3;
                  }
                  
                  var messageValue3 = statusesValue2['message'];
                  if (messageValue3 !== null && messageValue3 !== undefined) {
                    var messageInstance3 = messageValue3;
                    instanceViewStatusInstance2.message = messageInstance3;
                  }
                  
                  var timeValue3 = statusesValue2['time'];
                  if (timeValue3 !== null && timeValue3 !== undefined) {
                    var timeInstance3 = timeValue3;
                    instanceViewStatusInstance2.time = timeInstance3;
                  }
                }
              }
            }
          }
          
          var extensionsArray = responseDoc['extensions'];
          if (extensionsArray !== null && extensionsArray !== undefined) {
            for (var loweredIndex5 = 0; loweredIndex5 < extensionsArray.length; loweredIndex5 = loweredIndex5 + 1) {
              var extensionsValue = extensionsArray[loweredIndex5];
              var virtualMachineExtensionInstanceViewInstance = { statuses: [], subStatuses: [] };
              instanceViewInstance.extensions.push(virtualMachineExtensionInstanceViewInstance);
              
              var nameValue2 = extensionsValue['name'];
              if (nameValue2 !== null && nameValue2 !== undefined) {
                var nameInstance2 = nameValue2;
                virtualMachineExtensionInstanceViewInstance.name = nameInstance2;
              }
              
              var typeValue2 = extensionsValue['type'];
              if (typeValue2 !== null && typeValue2 !== undefined) {
                var typeInstance2 = typeValue2;
                virtualMachineExtensionInstanceViewInstance.extensionType = typeInstance2;
              }
              
              var typeHandlerVersionValue2 = extensionsValue['typeHandlerVersion'];
              if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                virtualMachineExtensionInstanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
              }
              
              var substatusesArray = extensionsValue['substatuses'];
              if (substatusesArray !== null && substatusesArray !== undefined) {
                for (var loweredIndex6 = 0; loweredIndex6 < substatusesArray.length; loweredIndex6 = loweredIndex6 + 1) {
                  var substatusesValue = substatusesArray[loweredIndex6];
                  var instanceViewStatusInstance3 = {};
                  virtualMachineExtensionInstanceViewInstance.subStatuses.push(instanceViewStatusInstance3);
                  
                  var codeValue4 = substatusesValue['code'];
                  if (codeValue4 !== null && codeValue4 !== undefined) {
                    var codeInstance4 = codeValue4;
                    instanceViewStatusInstance3.code = codeInstance4;
                  }
                  
                  var levelValue4 = substatusesValue['level'];
                  if (levelValue4 !== null && levelValue4 !== undefined) {
                    var levelInstance4 = levelValue4;
                    instanceViewStatusInstance3.level = levelInstance4;
                  }
                  
                  var displayStatusValue4 = substatusesValue['displayStatus'];
                  if (displayStatusValue4 !== null && displayStatusValue4 !== undefined) {
                    var displayStatusInstance4 = displayStatusValue4;
                    instanceViewStatusInstance3.displayStatus = displayStatusInstance4;
                  }
                  
                  var messageValue4 = substatusesValue['message'];
                  if (messageValue4 !== null && messageValue4 !== undefined) {
                    var messageInstance4 = messageValue4;
                    instanceViewStatusInstance3.message = messageInstance4;
                  }
                  
                  var timeValue4 = substatusesValue['time'];
                  if (timeValue4 !== null && timeValue4 !== undefined) {
                    var timeInstance4 = timeValue4;
                    instanceViewStatusInstance3.time = timeInstance4;
                  }
                }
              }
              
              var statusesArray3 = extensionsValue['statuses'];
              if (statusesArray3 !== null && statusesArray3 !== undefined) {
                for (var loweredIndex7 = 0; loweredIndex7 < statusesArray3.length; loweredIndex7 = loweredIndex7 + 1) {
                  var statusesValue3 = statusesArray3[loweredIndex7];
                  var instanceViewStatusInstance4 = {};
                  virtualMachineExtensionInstanceViewInstance.statuses.push(instanceViewStatusInstance4);
                  
                  var codeValue5 = statusesValue3['code'];
                  if (codeValue5 !== null && codeValue5 !== undefined) {
                    var codeInstance5 = codeValue5;
                    instanceViewStatusInstance4.code = codeInstance5;
                  }
                  
                  var levelValue5 = statusesValue3['level'];
                  if (levelValue5 !== null && levelValue5 !== undefined) {
                    var levelInstance5 = levelValue5;
                    instanceViewStatusInstance4.level = levelInstance5;
                  }
                  
                  var displayStatusValue5 = statusesValue3['displayStatus'];
                  if (displayStatusValue5 !== null && displayStatusValue5 !== undefined) {
                    var displayStatusInstance5 = displayStatusValue5;
                    instanceViewStatusInstance4.displayStatus = displayStatusInstance5;
                  }
                  
                  var messageValue5 = statusesValue3['message'];
                  if (messageValue5 !== null && messageValue5 !== undefined) {
                    var messageInstance5 = messageValue5;
                    instanceViewStatusInstance4.message = messageInstance5;
                  }
                  
                  var timeValue5 = statusesValue3['time'];
                  if (timeValue5 !== null && timeValue5 !== undefined) {
                    var timeInstance5 = timeValue5;
                    instanceViewStatusInstance4.time = timeInstance5;
                  }
                }
              }
            }
          }
          
          var statusesArray4 = responseDoc['statuses'];
          if (statusesArray4 !== null && statusesArray4 !== undefined) {
            for (var loweredIndex8 = 0; loweredIndex8 < statusesArray4.length; loweredIndex8 = loweredIndex8 + 1) {
              var statusesValue4 = statusesArray4[loweredIndex8];
              var instanceViewStatusInstance5 = {};
              instanceViewInstance.statuses.push(instanceViewStatusInstance5);
              
              var codeValue6 = statusesValue4['code'];
              if (codeValue6 !== null && codeValue6 !== undefined) {
                var codeInstance6 = codeValue6;
                instanceViewStatusInstance5.code = codeInstance6;
              }
              
              var levelValue6 = statusesValue4['level'];
              if (levelValue6 !== null && levelValue6 !== undefined) {
                var levelInstance6 = levelValue6;
                instanceViewStatusInstance5.level = levelInstance6;
              }
              
              var displayStatusValue6 = statusesValue4['displayStatus'];
              if (displayStatusValue6 !== null && displayStatusValue6 !== undefined) {
                var displayStatusInstance6 = displayStatusValue6;
                instanceViewStatusInstance5.displayStatus = displayStatusInstance6;
              }
              
              var messageValue6 = statusesValue4['message'];
              if (messageValue6 !== null && messageValue6 !== undefined) {
                var messageInstance6 = messageValue6;
                instanceViewStatusInstance5.message = messageInstance6;
              }
              
              var timeValue6 = statusesValue4['time'];
              if (timeValue6 !== null && timeValue6 !== undefined) {
                var timeInstance6 = timeValue6;
                instanceViewStatusInstance5.time = timeInstance6;
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The operation to list virtual machines under a resource group.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.list = function(resourceGroupName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachines: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineJsonInstance = { tags: {} };
              result.virtualMachines.push(virtualMachineJsonInstance);
              
              var planValue = valueValue['plan'];
              if (planValue !== null && planValue !== undefined) {
                var planInstance = {};
                virtualMachineJsonInstance.plan = planInstance;
                
                var nameValue = planValue['name'];
                if (nameValue !== null && nameValue !== undefined) {
                  var nameInstance = nameValue;
                  planInstance.name = nameInstance;
                }
                
                var publisherValue = planValue['publisher'];
                if (publisherValue !== null && publisherValue !== undefined) {
                  var publisherInstance = publisherValue;
                  planInstance.publisher = publisherInstance;
                }
                
                var productValue = planValue['product'];
                if (productValue !== null && productValue !== undefined) {
                  var productInstance = productValue;
                  planInstance.product = productInstance;
                }
                
                var promotionCodeValue = planValue['promotionCode'];
                if (promotionCodeValue !== null && promotionCodeValue !== undefined) {
                  var promotionCodeInstance = promotionCodeValue;
                  planInstance.promotionCode = promotionCodeInstance;
                }
              }
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var hardwareProfileValue = propertiesValue['hardwareProfile'];
                if (hardwareProfileValue !== null && hardwareProfileValue !== undefined) {
                  var hardwareProfileInstance = {};
                  virtualMachineJsonInstance.hardwareProfile = hardwareProfileInstance;
                  
                  var vmSizeValue = hardwareProfileValue['vmSize'];
                  if (vmSizeValue !== null && vmSizeValue !== undefined) {
                    var vmSizeInstance = vmSizeValue;
                    hardwareProfileInstance.virtualMachineSize = vmSizeInstance;
                  }
                }
                
                var storageProfileValue = propertiesValue['storageProfile'];
                if (storageProfileValue !== null && storageProfileValue !== undefined) {
                  var storageProfileInstance = { dataDisks: [] };
                  virtualMachineJsonInstance.storageProfile = storageProfileInstance;
                  
                  var sourceImageValue = storageProfileValue['sourceImage'];
                  if (sourceImageValue !== null && sourceImageValue !== undefined) {
                    var sourceImageInstance = {};
                    storageProfileInstance.sourceImage = sourceImageInstance;
                    
                    var idValue = sourceImageValue['id'];
                    if (idValue !== null && idValue !== undefined) {
                      var idInstance = idValue;
                      sourceImageInstance.referenceUri = idInstance;
                    }
                  }
                  
                  var destinationVhdsContainerValue = storageProfileValue['destinationVhdsContainer'];
                  if (destinationVhdsContainerValue !== null && destinationVhdsContainerValue !== undefined) {
                    var destinationVhdsContainerInstance = destinationVhdsContainerValue;
                    storageProfileInstance.destinationVhdsContainer = destinationVhdsContainerInstance;
                  }
                  
                  var osDiskValue = storageProfileValue['osDisk'];
                  if (osDiskValue !== null && osDiskValue !== undefined) {
                    var osDiskInstance = {};
                    storageProfileInstance.oSDisk = osDiskInstance;
                    
                    var osTypeValue = osDiskValue['osType'];
                    if (osTypeValue !== null && osTypeValue !== undefined) {
                      var osTypeInstance = osTypeValue;
                      osDiskInstance.operatingSystemType = osTypeInstance;
                    }
                    
                    var nameValue2 = osDiskValue['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      osDiskInstance.name = nameInstance2;
                    }
                    
                    var vhdValue = osDiskValue['vhd'];
                    if (vhdValue !== null && vhdValue !== undefined) {
                      var vhdInstance = {};
                      osDiskInstance.virtualHardDisk = vhdInstance;
                      
                      var uriValue = vhdValue['uri'];
                      if (uriValue !== null && uriValue !== undefined) {
                        var uriInstance = uriValue;
                        vhdInstance.uri = uriInstance;
                      }
                    }
                    
                    var imageValue = osDiskValue['image'];
                    if (imageValue !== null && imageValue !== undefined) {
                      var imageInstance = {};
                      osDiskInstance.sourceImage = imageInstance;
                      
                      var uriValue2 = imageValue['uri'];
                      if (uriValue2 !== null && uriValue2 !== undefined) {
                        var uriInstance2 = uriValue2;
                        imageInstance.uri = uriInstance2;
                      }
                    }
                    
                    var cachingValue = osDiskValue['caching'];
                    if (cachingValue !== null && cachingValue !== undefined) {
                      var cachingInstance = cachingValue;
                      osDiskInstance.caching = cachingInstance;
                    }
                  }
                  
                  var dataDisksArray = storageProfileValue['dataDisks'];
                  if (dataDisksArray !== null && dataDisksArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < dataDisksArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var dataDisksValue = dataDisksArray[loweredIndex2];
                      var dataDiskInstance = {};
                      storageProfileInstance.dataDisks.push(dataDiskInstance);
                      
                      var lunValue = dataDisksValue['lun'];
                      if (lunValue !== null && lunValue !== undefined) {
                        var lunInstance = lunValue;
                        dataDiskInstance.lun = lunInstance;
                      }
                      
                      var diskSizeGBValue = dataDisksValue['diskSizeGB'];
                      if (diskSizeGBValue !== null && diskSizeGBValue !== undefined) {
                        var diskSizeGBInstance = diskSizeGBValue;
                        dataDiskInstance.diskSizeGB = diskSizeGBInstance;
                      }
                      
                      var nameValue3 = dataDisksValue['name'];
                      if (nameValue3 !== null && nameValue3 !== undefined) {
                        var nameInstance3 = nameValue3;
                        dataDiskInstance.name = nameInstance3;
                      }
                      
                      var vhdValue2 = dataDisksValue['vhd'];
                      if (vhdValue2 !== null && vhdValue2 !== undefined) {
                        var vhdInstance2 = {};
                        dataDiskInstance.virtualHardDisk = vhdInstance2;
                        
                        var uriValue3 = vhdValue2['uri'];
                        if (uriValue3 !== null && uriValue3 !== undefined) {
                          var uriInstance3 = uriValue3;
                          vhdInstance2.uri = uriInstance3;
                        }
                      }
                      
                      var imageValue2 = dataDisksValue['image'];
                      if (imageValue2 !== null && imageValue2 !== undefined) {
                        var imageInstance2 = {};
                        dataDiskInstance.sourceImage = imageInstance2;
                        
                        var uriValue4 = imageValue2['uri'];
                        if (uriValue4 !== null && uriValue4 !== undefined) {
                          var uriInstance4 = uriValue4;
                          imageInstance2.uri = uriInstance4;
                        }
                      }
                      
                      var cachingValue2 = dataDisksValue['caching'];
                      if (cachingValue2 !== null && cachingValue2 !== undefined) {
                        var cachingInstance2 = cachingValue2;
                        dataDiskInstance.caching = cachingInstance2;
                      }
                    }
                  }
                }
                
                var osProfileValue = propertiesValue['osProfile'];
                if (osProfileValue !== null && osProfileValue !== undefined) {
                  var osProfileInstance = { secrets: [] };
                  virtualMachineJsonInstance.oSProfile = osProfileInstance;
                  
                  var computerNameValue = osProfileValue['computerName'];
                  if (computerNameValue !== null && computerNameValue !== undefined) {
                    var computerNameInstance = computerNameValue;
                    osProfileInstance.computerName = computerNameInstance;
                  }
                  
                  var adminUsernameValue = osProfileValue['adminUsername'];
                  if (adminUsernameValue !== null && adminUsernameValue !== undefined) {
                    var adminUsernameInstance = adminUsernameValue;
                    osProfileInstance.adminUsername = adminUsernameInstance;
                  }
                  
                  var adminPasswordValue = osProfileValue['adminPassword'];
                  if (adminPasswordValue !== null && adminPasswordValue !== undefined) {
                    var adminPasswordInstance = adminPasswordValue;
                    osProfileInstance.adminPassword = adminPasswordInstance;
                  }
                  
                  var customDataValue = osProfileValue['customData'];
                  if (customDataValue !== null && customDataValue !== undefined) {
                    var customDataInstance = customDataValue;
                    osProfileInstance.customData = customDataInstance;
                  }
                  
                  var windowsConfigurationValue = osProfileValue['windowsConfiguration'];
                  if (windowsConfigurationValue !== null && windowsConfigurationValue !== undefined) {
                    var windowsConfigurationInstance = { additionalUnattendContents: [] };
                    osProfileInstance.windowsConfiguration = windowsConfigurationInstance;
                    
                    var provisionVMAgentValue = windowsConfigurationValue['provisionVMAgent'];
                    if (provisionVMAgentValue !== null && provisionVMAgentValue !== undefined) {
                      var provisionVMAgentInstance = provisionVMAgentValue;
                      windowsConfigurationInstance.provisionVMAgent = provisionVMAgentInstance;
                    }
                    
                    var enableAutomaticUpdatesValue = windowsConfigurationValue['enableAutomaticUpdates'];
                    if (enableAutomaticUpdatesValue !== null && enableAutomaticUpdatesValue !== undefined) {
                      var enableAutomaticUpdatesInstance = enableAutomaticUpdatesValue;
                      windowsConfigurationInstance.enableAutomaticUpdates = enableAutomaticUpdatesInstance;
                    }
                    
                    var timeZoneValue = windowsConfigurationValue['timeZone'];
                    if (timeZoneValue !== null && timeZoneValue !== undefined) {
                      var timeZoneInstance = timeZoneValue;
                      windowsConfigurationInstance.timeZone = timeZoneInstance;
                    }
                    
                    var additionalUnattendContentArray = windowsConfigurationValue['additionalUnattendContent'];
                    if (additionalUnattendContentArray !== null && additionalUnattendContentArray !== undefined) {
                      for (var loweredIndex3 = 0; loweredIndex3 < additionalUnattendContentArray.length; loweredIndex3 = loweredIndex3 + 1) {
                        var additionalUnattendContentValue = additionalUnattendContentArray[loweredIndex3];
                        var additionalUnattendContentInstance = {};
                        windowsConfigurationInstance.additionalUnattendContents.push(additionalUnattendContentInstance);
                        
                        var passNameValue = additionalUnattendContentValue['passName'];
                        if (passNameValue !== null && passNameValue !== undefined) {
                          var passNameInstance = passNameValue;
                          additionalUnattendContentInstance.passName = passNameInstance;
                        }
                        
                        var componentNameValue = additionalUnattendContentValue['componentName'];
                        if (componentNameValue !== null && componentNameValue !== undefined) {
                          var componentNameInstance = componentNameValue;
                          additionalUnattendContentInstance.componentName = componentNameInstance;
                        }
                        
                        var settingNameValue = additionalUnattendContentValue['settingName'];
                        if (settingNameValue !== null && settingNameValue !== undefined) {
                          var settingNameInstance = settingNameValue;
                          additionalUnattendContentInstance.settingName = settingNameInstance;
                        }
                        
                        var contentValue = additionalUnattendContentValue['content'];
                        if (contentValue !== null && contentValue !== undefined) {
                          var contentInstance = contentValue;
                          additionalUnattendContentInstance.content = contentInstance;
                        }
                      }
                    }
                    
                    var winRMValue = windowsConfigurationValue['winRM'];
                    if (winRMValue !== null && winRMValue !== undefined) {
                      var winRMInstance = { listeners: [] };
                      windowsConfigurationInstance.winRMConfiguration = winRMInstance;
                      
                      var listenersArray = winRMValue['listeners'];
                      if (listenersArray !== null && listenersArray !== undefined) {
                        for (var loweredIndex4 = 0; loweredIndex4 < listenersArray.length; loweredIndex4 = loweredIndex4 + 1) {
                          var listenersValue = listenersArray[loweredIndex4];
                          var winRMListenerInstance = {};
                          winRMInstance.listeners.push(winRMListenerInstance);
                          
                          var protocolValue = listenersValue['protocol'];
                          if (protocolValue !== null && protocolValue !== undefined) {
                            var protocolInstance = protocolValue;
                            winRMListenerInstance.protocol = protocolInstance;
                          }
                          
                          var certificateUrlValue = listenersValue['certificateUrl'];
                          if (certificateUrlValue !== null && certificateUrlValue !== undefined) {
                            var certificateUrlInstance = certificateUrlValue;
                            winRMListenerInstance.certificateUrl = certificateUrlInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var linuxConfigurationValue = osProfileValue['linuxConfiguration'];
                  if (linuxConfigurationValue !== null && linuxConfigurationValue !== undefined) {
                    var linuxConfigurationInstance = {};
                    osProfileInstance.linuxConfiguration = linuxConfigurationInstance;
                    
                    var disablePasswordAuthenticationValue = linuxConfigurationValue['disablePasswordAuthentication'];
                    if (disablePasswordAuthenticationValue !== null && disablePasswordAuthenticationValue !== undefined) {
                      var disablePasswordAuthenticationInstance = disablePasswordAuthenticationValue;
                      linuxConfigurationInstance.disablePasswordAuthentication = disablePasswordAuthenticationInstance;
                    }
                    
                    var sshValue = linuxConfigurationValue['ssh'];
                    if (sshValue !== null && sshValue !== undefined) {
                      var sshInstance = { publicKeys: [] };
                      linuxConfigurationInstance.sshConfiguration = sshInstance;
                      
                      var publicKeysArray = sshValue['publicKeys'];
                      if (publicKeysArray !== null && publicKeysArray !== undefined) {
                        for (var loweredIndex5 = 0; loweredIndex5 < publicKeysArray.length; loweredIndex5 = loweredIndex5 + 1) {
                          var publicKeysValue = publicKeysArray[loweredIndex5];
                          var sshPublicKeyInstance = {};
                          sshInstance.publicKeys.push(sshPublicKeyInstance);
                          
                          var pathValue = publicKeysValue['path'];
                          if (pathValue !== null && pathValue !== undefined) {
                            var pathInstance = pathValue;
                            sshPublicKeyInstance.path = pathInstance;
                          }
                          
                          var keyDataValue = publicKeysValue['keyData'];
                          if (keyDataValue !== null && keyDataValue !== undefined) {
                            var keyDataInstance = keyDataValue;
                            sshPublicKeyInstance.keyData = keyDataInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var secretsArray = osProfileValue['secrets'];
                  if (secretsArray !== null && secretsArray !== undefined) {
                    for (var loweredIndex6 = 0; loweredIndex6 < secretsArray.length; loweredIndex6 = loweredIndex6 + 1) {
                      var secretsValue = secretsArray[loweredIndex6];
                      var vaultSecretGroupInstance = { vaultCertificates: [] };
                      osProfileInstance.secrets.push(vaultSecretGroupInstance);
                      
                      var sourceVaultValue = secretsValue['sourceVault'];
                      if (sourceVaultValue !== null && sourceVaultValue !== undefined) {
                        var sourceVaultInstance = {};
                        vaultSecretGroupInstance.sourceVault = sourceVaultInstance;
                        
                        var idValue2 = sourceVaultValue['id'];
                        if (idValue2 !== null && idValue2 !== undefined) {
                          var idInstance2 = idValue2;
                          sourceVaultInstance.referenceUri = idInstance2;
                        }
                      }
                      
                      var vaultCertificatesArray = secretsValue['vaultCertificates'];
                      if (vaultCertificatesArray !== null && vaultCertificatesArray !== undefined) {
                        for (var loweredIndex7 = 0; loweredIndex7 < vaultCertificatesArray.length; loweredIndex7 = loweredIndex7 + 1) {
                          var vaultCertificatesValue = vaultCertificatesArray[loweredIndex7];
                          var vaultCertificateInstance = {};
                          vaultSecretGroupInstance.vaultCertificates.push(vaultCertificateInstance);
                          
                          var certificateUrlValue2 = vaultCertificatesValue['certificateUrl'];
                          if (certificateUrlValue2 !== null && certificateUrlValue2 !== undefined) {
                            var certificateUrlInstance2 = certificateUrlValue2;
                            vaultCertificateInstance.certificateUrl = certificateUrlInstance2;
                          }
                          
                          var certificateStoreValue = vaultCertificatesValue['certificateStore'];
                          if (certificateStoreValue !== null && certificateStoreValue !== undefined) {
                            var certificateStoreInstance = certificateStoreValue;
                            vaultCertificateInstance.certificateStore = certificateStoreInstance;
                          }
                        }
                      }
                    }
                  }
                }
                
                var networkProfileValue = propertiesValue['networkProfile'];
                if (networkProfileValue !== null && networkProfileValue !== undefined) {
                  var networkProfileInstance = { networkInterfaces: [] };
                  virtualMachineJsonInstance.networkProfile = networkProfileInstance;
                  
                  var networkInterfacesArray = networkProfileValue['networkInterfaces'];
                  if (networkInterfacesArray !== null && networkInterfacesArray !== undefined) {
                    for (var loweredIndex8 = 0; loweredIndex8 < networkInterfacesArray.length; loweredIndex8 = loweredIndex8 + 1) {
                      var networkInterfacesValue = networkInterfacesArray[loweredIndex8];
                      var networkInterfaceReferenceJsonInstance = {};
                      networkProfileInstance.networkInterfaces.push(networkInterfaceReferenceJsonInstance);
                      
                      var propertiesValue2 = networkInterfacesValue['properties'];
                      if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
                        var primaryValue = propertiesValue2['primary'];
                        if (primaryValue !== null && primaryValue !== undefined) {
                          var primaryInstance = primaryValue;
                          networkInterfaceReferenceJsonInstance.primary = primaryInstance;
                        }
                      }
                      
                      var idValue3 = networkInterfacesValue['id'];
                      if (idValue3 !== null && idValue3 !== undefined) {
                        var idInstance3 = idValue3;
                        networkInterfaceReferenceJsonInstance.referenceUri = idInstance3;
                      }
                    }
                  }
                }
                
                var availabilitySetValue = propertiesValue['availabilitySet'];
                if (availabilitySetValue !== null && availabilitySetValue !== undefined) {
                  var availabilitySetInstance = {};
                  virtualMachineJsonInstance.availabilitySetReference = availabilitySetInstance;
                  
                  var idValue4 = availabilitySetValue['id'];
                  if (idValue4 !== null && idValue4 !== undefined) {
                    var idInstance4 = idValue4;
                    availabilitySetInstance.referenceUri = idInstance4;
                  }
                }
                
                var provisioningStateValue = propertiesValue['provisioningState'];
                if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
                  var provisioningStateInstance = provisioningStateValue;
                  virtualMachineJsonInstance.provisioningState = provisioningStateInstance;
                }
              }
              
              var resourcesValue = valueValue['resources'];
              if (resourcesValue !== null && resourcesValue !== undefined) {
                var resourcesInstance = { extensions: [] };
                virtualMachineJsonInstance.resources = resourcesInstance;
                
                var extensionsArray = resourcesValue['extensions'];
                if (extensionsArray !== null && extensionsArray !== undefined) {
                  for (var loweredIndex9 = 0; loweredIndex9 < extensionsArray.length; loweredIndex9 = loweredIndex9 + 1) {
                    var extensionsValue = extensionsArray[loweredIndex9];
                    var virtualMachineExtensionJsonInstance = { tags: {} };
                    resourcesInstance.extensions.push(virtualMachineExtensionJsonInstance);
                    
                    var propertiesValue3 = extensionsValue['properties'];
                    if (propertiesValue3 !== null && propertiesValue3 !== undefined) {
                      var publisherValue2 = propertiesValue3['publisher'];
                      if (publisherValue2 !== null && publisherValue2 !== undefined) {
                        var publisherInstance2 = publisherValue2;
                        virtualMachineExtensionJsonInstance.publisher = publisherInstance2;
                      }
                      
                      var typeValue = propertiesValue3['type'];
                      if (typeValue !== null && typeValue !== undefined) {
                        var typeInstance = typeValue;
                        virtualMachineExtensionJsonInstance.extensionType = typeInstance;
                      }
                      
                      var typeHandlerVersionValue = propertiesValue3['typeHandlerVersion'];
                      if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                        var typeHandlerVersionInstance = typeHandlerVersionValue;
                        virtualMachineExtensionJsonInstance.typeHandlerVersion = typeHandlerVersionInstance;
                      }
                      
                      var settingsValue = propertiesValue3['settings'];
                      if (settingsValue !== null && settingsValue !== undefined) {
                        var settingsInstance = settingsValue;
                        virtualMachineExtensionJsonInstance.settings = settingsInstance;
                      }
                      
                      var protectedSettingsValue = propertiesValue3['protectedSettings'];
                      if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
                        var protectedSettingsInstance = protectedSettingsValue;
                        virtualMachineExtensionJsonInstance.protectedSettings = protectedSettingsInstance;
                      }
                      
                      var provisioningStateValue2 = propertiesValue3['provisioningState'];
                      if (provisioningStateValue2 !== null && provisioningStateValue2 !== undefined) {
                        var provisioningStateInstance2 = provisioningStateValue2;
                        virtualMachineExtensionJsonInstance.provisioningState = provisioningStateInstance2;
                      }
                      
                      var instanceViewValue = propertiesValue3['instanceView'];
                      if (instanceViewValue !== null && instanceViewValue !== undefined) {
                        var instanceViewInstance = { statuses: [], subStatuses: [] };
                        virtualMachineExtensionJsonInstance.instanceView = instanceViewInstance;
                        
                        var nameValue4 = instanceViewValue['name'];
                        if (nameValue4 !== null && nameValue4 !== undefined) {
                          var nameInstance4 = nameValue4;
                          instanceViewInstance.name = nameInstance4;
                        }
                        
                        var typeValue2 = instanceViewValue['type'];
                        if (typeValue2 !== null && typeValue2 !== undefined) {
                          var typeInstance2 = typeValue2;
                          instanceViewInstance.extensionType = typeInstance2;
                        }
                        
                        var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
                        if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                          var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                          instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
                        }
                        
                        var substatusesArray = instanceViewValue['substatuses'];
                        if (substatusesArray !== null && substatusesArray !== undefined) {
                          for (var loweredIndex10 = 0; loweredIndex10 < substatusesArray.length; loweredIndex10 = loweredIndex10 + 1) {
                            var substatusesValue = substatusesArray[loweredIndex10];
                            var instanceViewStatusInstance = {};
                            instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                            
                            var codeValue = substatusesValue['code'];
                            if (codeValue !== null && codeValue !== undefined) {
                              var codeInstance = codeValue;
                              instanceViewStatusInstance.code = codeInstance;
                            }
                            
                            var levelValue = substatusesValue['level'];
                            if (levelValue !== null && levelValue !== undefined) {
                              var levelInstance = levelValue;
                              instanceViewStatusInstance.level = levelInstance;
                            }
                            
                            var displayStatusValue = substatusesValue['displayStatus'];
                            if (displayStatusValue !== null && displayStatusValue !== undefined) {
                              var displayStatusInstance = displayStatusValue;
                              instanceViewStatusInstance.displayStatus = displayStatusInstance;
                            }
                            
                            var messageValue = substatusesValue['message'];
                            if (messageValue !== null && messageValue !== undefined) {
                              var messageInstance = messageValue;
                              instanceViewStatusInstance.message = messageInstance;
                            }
                            
                            var timeValue = substatusesValue['time'];
                            if (timeValue !== null && timeValue !== undefined) {
                              var timeInstance = timeValue;
                              instanceViewStatusInstance.time = timeInstance;
                            }
                          }
                        }
                        
                        var statusesArray = instanceViewValue['statuses'];
                        if (statusesArray !== null && statusesArray !== undefined) {
                          for (var loweredIndex11 = 0; loweredIndex11 < statusesArray.length; loweredIndex11 = loweredIndex11 + 1) {
                            var statusesValue = statusesArray[loweredIndex11];
                            var instanceViewStatusInstance2 = {};
                            instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                            
                            var codeValue2 = statusesValue['code'];
                            if (codeValue2 !== null && codeValue2 !== undefined) {
                              var codeInstance2 = codeValue2;
                              instanceViewStatusInstance2.code = codeInstance2;
                            }
                            
                            var levelValue2 = statusesValue['level'];
                            if (levelValue2 !== null && levelValue2 !== undefined) {
                              var levelInstance2 = levelValue2;
                              instanceViewStatusInstance2.level = levelInstance2;
                            }
                            
                            var displayStatusValue2 = statusesValue['displayStatus'];
                            if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                              var displayStatusInstance2 = displayStatusValue2;
                              instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                            }
                            
                            var messageValue2 = statusesValue['message'];
                            if (messageValue2 !== null && messageValue2 !== undefined) {
                              var messageInstance2 = messageValue2;
                              instanceViewStatusInstance2.message = messageInstance2;
                            }
                            
                            var timeValue2 = statusesValue['time'];
                            if (timeValue2 !== null && timeValue2 !== undefined) {
                              var timeInstance2 = timeValue2;
                              instanceViewStatusInstance2.time = timeInstance2;
                            }
                          }
                        }
                      }
                    }
                    
                    var idValue5 = extensionsValue['id'];
                    if (idValue5 !== null && idValue5 !== undefined) {
                      var idInstance5 = idValue5;
                      virtualMachineExtensionJsonInstance.id = idInstance5;
                    }
                    
                    var nameValue5 = extensionsValue['name'];
                    if (nameValue5 !== null && nameValue5 !== undefined) {
                      var nameInstance5 = nameValue5;
                      virtualMachineExtensionJsonInstance.name = nameInstance5;
                    }
                    
                    var typeValue3 = extensionsValue['type'];
                    if (typeValue3 !== null && typeValue3 !== undefined) {
                      var typeInstance3 = typeValue3;
                      virtualMachineExtensionJsonInstance.type = typeInstance3;
                    }
                    
                    var locationValue = extensionsValue['location'];
                    if (locationValue !== null && locationValue !== undefined) {
                      var locationInstance = locationValue;
                      virtualMachineExtensionJsonInstance.location = locationInstance;
                    }
                    
                    var tagsSequenceElement = extensionsValue['tags'];
                    if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                      for (var property in tagsSequenceElement) {
                        var tagsKey = property;
                        var tagsValue = tagsSequenceElement[property];
                        virtualMachineExtensionJsonInstance.tags[tagsKey] = tagsValue;
                      }
                    }
                  }
                }
              }
              
              var idValue6 = valueValue['id'];
              if (idValue6 !== null && idValue6 !== undefined) {
                var idInstance6 = idValue6;
                virtualMachineJsonInstance.id = idInstance6;
              }
              
              var nameValue6 = valueValue['name'];
              if (nameValue6 !== null && nameValue6 !== undefined) {
                var nameInstance6 = nameValue6;
                virtualMachineJsonInstance.name = nameInstance6;
              }
              
              var typeValue4 = valueValue['type'];
              if (typeValue4 !== null && typeValue4 !== undefined) {
                var typeInstance4 = typeValue4;
                virtualMachineJsonInstance.type = typeInstance4;
              }
              
              var locationValue2 = valueValue['location'];
              if (locationValue2 !== null && locationValue2 !== undefined) {
                var locationInstance2 = locationValue2;
                virtualMachineJsonInstance.location = locationInstance2;
              }
              
              var tagsSequenceElement2 = valueValue['tags'];
              if (tagsSequenceElement2 !== null && tagsSequenceElement2 !== undefined) {
                for (var property2 in tagsSequenceElement2) {
                  var tagsKey2 = property2;
                  var tagsValue2 = tagsSequenceElement2[property2];
                  virtualMachineJsonInstance.tags[tagsKey2] = tagsValue2;
                }
              }
            }
          }
          
          var odatanextLinkValue = responseDoc['@odata.nextLink'];
          if (odatanextLinkValue !== null && odatanextLinkValue !== undefined) {
            var odatanextLinkInstance = odatanextLinkValue;
            result.nextLink = odatanextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Gets the list of Virtual Machines in the subscription. Use nextLink
   * property in the response to get the next page of Virtual Machines. Do
   * this till nextLink is not null to fetch all the Virtual Machines.
   * 
   * @param {ListParameters} [parametersOrCallback]
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.listAll = function(parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachines: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineJsonInstance = { tags: {} };
              result.virtualMachines.push(virtualMachineJsonInstance);
              
              var planValue = valueValue['plan'];
              if (planValue !== null && planValue !== undefined) {
                var planInstance = {};
                virtualMachineJsonInstance.plan = planInstance;
                
                var nameValue = planValue['name'];
                if (nameValue !== null && nameValue !== undefined) {
                  var nameInstance = nameValue;
                  planInstance.name = nameInstance;
                }
                
                var publisherValue = planValue['publisher'];
                if (publisherValue !== null && publisherValue !== undefined) {
                  var publisherInstance = publisherValue;
                  planInstance.publisher = publisherInstance;
                }
                
                var productValue = planValue['product'];
                if (productValue !== null && productValue !== undefined) {
                  var productInstance = productValue;
                  planInstance.product = productInstance;
                }
                
                var promotionCodeValue = planValue['promotionCode'];
                if (promotionCodeValue !== null && promotionCodeValue !== undefined) {
                  var promotionCodeInstance = promotionCodeValue;
                  planInstance.promotionCode = promotionCodeInstance;
                }
              }
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var hardwareProfileValue = propertiesValue['hardwareProfile'];
                if (hardwareProfileValue !== null && hardwareProfileValue !== undefined) {
                  var hardwareProfileInstance = {};
                  virtualMachineJsonInstance.hardwareProfile = hardwareProfileInstance;
                  
                  var vmSizeValue = hardwareProfileValue['vmSize'];
                  if (vmSizeValue !== null && vmSizeValue !== undefined) {
                    var vmSizeInstance = vmSizeValue;
                    hardwareProfileInstance.virtualMachineSize = vmSizeInstance;
                  }
                }
                
                var storageProfileValue = propertiesValue['storageProfile'];
                if (storageProfileValue !== null && storageProfileValue !== undefined) {
                  var storageProfileInstance = { dataDisks: [] };
                  virtualMachineJsonInstance.storageProfile = storageProfileInstance;
                  
                  var sourceImageValue = storageProfileValue['sourceImage'];
                  if (sourceImageValue !== null && sourceImageValue !== undefined) {
                    var sourceImageInstance = {};
                    storageProfileInstance.sourceImage = sourceImageInstance;
                    
                    var idValue = sourceImageValue['id'];
                    if (idValue !== null && idValue !== undefined) {
                      var idInstance = idValue;
                      sourceImageInstance.referenceUri = idInstance;
                    }
                  }
                  
                  var destinationVhdsContainerValue = storageProfileValue['destinationVhdsContainer'];
                  if (destinationVhdsContainerValue !== null && destinationVhdsContainerValue !== undefined) {
                    var destinationVhdsContainerInstance = destinationVhdsContainerValue;
                    storageProfileInstance.destinationVhdsContainer = destinationVhdsContainerInstance;
                  }
                  
                  var osDiskValue = storageProfileValue['osDisk'];
                  if (osDiskValue !== null && osDiskValue !== undefined) {
                    var osDiskInstance = {};
                    storageProfileInstance.oSDisk = osDiskInstance;
                    
                    var osTypeValue = osDiskValue['osType'];
                    if (osTypeValue !== null && osTypeValue !== undefined) {
                      var osTypeInstance = osTypeValue;
                      osDiskInstance.operatingSystemType = osTypeInstance;
                    }
                    
                    var nameValue2 = osDiskValue['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      osDiskInstance.name = nameInstance2;
                    }
                    
                    var vhdValue = osDiskValue['vhd'];
                    if (vhdValue !== null && vhdValue !== undefined) {
                      var vhdInstance = {};
                      osDiskInstance.virtualHardDisk = vhdInstance;
                      
                      var uriValue = vhdValue['uri'];
                      if (uriValue !== null && uriValue !== undefined) {
                        var uriInstance = uriValue;
                        vhdInstance.uri = uriInstance;
                      }
                    }
                    
                    var imageValue = osDiskValue['image'];
                    if (imageValue !== null && imageValue !== undefined) {
                      var imageInstance = {};
                      osDiskInstance.sourceImage = imageInstance;
                      
                      var uriValue2 = imageValue['uri'];
                      if (uriValue2 !== null && uriValue2 !== undefined) {
                        var uriInstance2 = uriValue2;
                        imageInstance.uri = uriInstance2;
                      }
                    }
                    
                    var cachingValue = osDiskValue['caching'];
                    if (cachingValue !== null && cachingValue !== undefined) {
                      var cachingInstance = cachingValue;
                      osDiskInstance.caching = cachingInstance;
                    }
                  }
                  
                  var dataDisksArray = storageProfileValue['dataDisks'];
                  if (dataDisksArray !== null && dataDisksArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < dataDisksArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var dataDisksValue = dataDisksArray[loweredIndex2];
                      var dataDiskInstance = {};
                      storageProfileInstance.dataDisks.push(dataDiskInstance);
                      
                      var lunValue = dataDisksValue['lun'];
                      if (lunValue !== null && lunValue !== undefined) {
                        var lunInstance = lunValue;
                        dataDiskInstance.lun = lunInstance;
                      }
                      
                      var diskSizeGBValue = dataDisksValue['diskSizeGB'];
                      if (diskSizeGBValue !== null && diskSizeGBValue !== undefined) {
                        var diskSizeGBInstance = diskSizeGBValue;
                        dataDiskInstance.diskSizeGB = diskSizeGBInstance;
                      }
                      
                      var nameValue3 = dataDisksValue['name'];
                      if (nameValue3 !== null && nameValue3 !== undefined) {
                        var nameInstance3 = nameValue3;
                        dataDiskInstance.name = nameInstance3;
                      }
                      
                      var vhdValue2 = dataDisksValue['vhd'];
                      if (vhdValue2 !== null && vhdValue2 !== undefined) {
                        var vhdInstance2 = {};
                        dataDiskInstance.virtualHardDisk = vhdInstance2;
                        
                        var uriValue3 = vhdValue2['uri'];
                        if (uriValue3 !== null && uriValue3 !== undefined) {
                          var uriInstance3 = uriValue3;
                          vhdInstance2.uri = uriInstance3;
                        }
                      }
                      
                      var imageValue2 = dataDisksValue['image'];
                      if (imageValue2 !== null && imageValue2 !== undefined) {
                        var imageInstance2 = {};
                        dataDiskInstance.sourceImage = imageInstance2;
                        
                        var uriValue4 = imageValue2['uri'];
                        if (uriValue4 !== null && uriValue4 !== undefined) {
                          var uriInstance4 = uriValue4;
                          imageInstance2.uri = uriInstance4;
                        }
                      }
                      
                      var cachingValue2 = dataDisksValue['caching'];
                      if (cachingValue2 !== null && cachingValue2 !== undefined) {
                        var cachingInstance2 = cachingValue2;
                        dataDiskInstance.caching = cachingInstance2;
                      }
                    }
                  }
                }
                
                var osProfileValue = propertiesValue['osProfile'];
                if (osProfileValue !== null && osProfileValue !== undefined) {
                  var osProfileInstance = { secrets: [] };
                  virtualMachineJsonInstance.oSProfile = osProfileInstance;
                  
                  var computerNameValue = osProfileValue['computerName'];
                  if (computerNameValue !== null && computerNameValue !== undefined) {
                    var computerNameInstance = computerNameValue;
                    osProfileInstance.computerName = computerNameInstance;
                  }
                  
                  var adminUsernameValue = osProfileValue['adminUsername'];
                  if (adminUsernameValue !== null && adminUsernameValue !== undefined) {
                    var adminUsernameInstance = adminUsernameValue;
                    osProfileInstance.adminUsername = adminUsernameInstance;
                  }
                  
                  var adminPasswordValue = osProfileValue['adminPassword'];
                  if (adminPasswordValue !== null && adminPasswordValue !== undefined) {
                    var adminPasswordInstance = adminPasswordValue;
                    osProfileInstance.adminPassword = adminPasswordInstance;
                  }
                  
                  var customDataValue = osProfileValue['customData'];
                  if (customDataValue !== null && customDataValue !== undefined) {
                    var customDataInstance = customDataValue;
                    osProfileInstance.customData = customDataInstance;
                  }
                  
                  var windowsConfigurationValue = osProfileValue['windowsConfiguration'];
                  if (windowsConfigurationValue !== null && windowsConfigurationValue !== undefined) {
                    var windowsConfigurationInstance = { additionalUnattendContents: [] };
                    osProfileInstance.windowsConfiguration = windowsConfigurationInstance;
                    
                    var provisionVMAgentValue = windowsConfigurationValue['provisionVMAgent'];
                    if (provisionVMAgentValue !== null && provisionVMAgentValue !== undefined) {
                      var provisionVMAgentInstance = provisionVMAgentValue;
                      windowsConfigurationInstance.provisionVMAgent = provisionVMAgentInstance;
                    }
                    
                    var enableAutomaticUpdatesValue = windowsConfigurationValue['enableAutomaticUpdates'];
                    if (enableAutomaticUpdatesValue !== null && enableAutomaticUpdatesValue !== undefined) {
                      var enableAutomaticUpdatesInstance = enableAutomaticUpdatesValue;
                      windowsConfigurationInstance.enableAutomaticUpdates = enableAutomaticUpdatesInstance;
                    }
                    
                    var timeZoneValue = windowsConfigurationValue['timeZone'];
                    if (timeZoneValue !== null && timeZoneValue !== undefined) {
                      var timeZoneInstance = timeZoneValue;
                      windowsConfigurationInstance.timeZone = timeZoneInstance;
                    }
                    
                    var additionalUnattendContentArray = windowsConfigurationValue['additionalUnattendContent'];
                    if (additionalUnattendContentArray !== null && additionalUnattendContentArray !== undefined) {
                      for (var loweredIndex3 = 0; loweredIndex3 < additionalUnattendContentArray.length; loweredIndex3 = loweredIndex3 + 1) {
                        var additionalUnattendContentValue = additionalUnattendContentArray[loweredIndex3];
                        var additionalUnattendContentInstance = {};
                        windowsConfigurationInstance.additionalUnattendContents.push(additionalUnattendContentInstance);
                        
                        var passNameValue = additionalUnattendContentValue['passName'];
                        if (passNameValue !== null && passNameValue !== undefined) {
                          var passNameInstance = passNameValue;
                          additionalUnattendContentInstance.passName = passNameInstance;
                        }
                        
                        var componentNameValue = additionalUnattendContentValue['componentName'];
                        if (componentNameValue !== null && componentNameValue !== undefined) {
                          var componentNameInstance = componentNameValue;
                          additionalUnattendContentInstance.componentName = componentNameInstance;
                        }
                        
                        var settingNameValue = additionalUnattendContentValue['settingName'];
                        if (settingNameValue !== null && settingNameValue !== undefined) {
                          var settingNameInstance = settingNameValue;
                          additionalUnattendContentInstance.settingName = settingNameInstance;
                        }
                        
                        var contentValue = additionalUnattendContentValue['content'];
                        if (contentValue !== null && contentValue !== undefined) {
                          var contentInstance = contentValue;
                          additionalUnattendContentInstance.content = contentInstance;
                        }
                      }
                    }
                    
                    var winRMValue = windowsConfigurationValue['winRM'];
                    if (winRMValue !== null && winRMValue !== undefined) {
                      var winRMInstance = { listeners: [] };
                      windowsConfigurationInstance.winRMConfiguration = winRMInstance;
                      
                      var listenersArray = winRMValue['listeners'];
                      if (listenersArray !== null && listenersArray !== undefined) {
                        for (var loweredIndex4 = 0; loweredIndex4 < listenersArray.length; loweredIndex4 = loweredIndex4 + 1) {
                          var listenersValue = listenersArray[loweredIndex4];
                          var winRMListenerInstance = {};
                          winRMInstance.listeners.push(winRMListenerInstance);
                          
                          var protocolValue = listenersValue['protocol'];
                          if (protocolValue !== null && protocolValue !== undefined) {
                            var protocolInstance = protocolValue;
                            winRMListenerInstance.protocol = protocolInstance;
                          }
                          
                          var certificateUrlValue = listenersValue['certificateUrl'];
                          if (certificateUrlValue !== null && certificateUrlValue !== undefined) {
                            var certificateUrlInstance = certificateUrlValue;
                            winRMListenerInstance.certificateUrl = certificateUrlInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var linuxConfigurationValue = osProfileValue['linuxConfiguration'];
                  if (linuxConfigurationValue !== null && linuxConfigurationValue !== undefined) {
                    var linuxConfigurationInstance = {};
                    osProfileInstance.linuxConfiguration = linuxConfigurationInstance;
                    
                    var disablePasswordAuthenticationValue = linuxConfigurationValue['disablePasswordAuthentication'];
                    if (disablePasswordAuthenticationValue !== null && disablePasswordAuthenticationValue !== undefined) {
                      var disablePasswordAuthenticationInstance = disablePasswordAuthenticationValue;
                      linuxConfigurationInstance.disablePasswordAuthentication = disablePasswordAuthenticationInstance;
                    }
                    
                    var sshValue = linuxConfigurationValue['ssh'];
                    if (sshValue !== null && sshValue !== undefined) {
                      var sshInstance = { publicKeys: [] };
                      linuxConfigurationInstance.sshConfiguration = sshInstance;
                      
                      var publicKeysArray = sshValue['publicKeys'];
                      if (publicKeysArray !== null && publicKeysArray !== undefined) {
                        for (var loweredIndex5 = 0; loweredIndex5 < publicKeysArray.length; loweredIndex5 = loweredIndex5 + 1) {
                          var publicKeysValue = publicKeysArray[loweredIndex5];
                          var sshPublicKeyInstance = {};
                          sshInstance.publicKeys.push(sshPublicKeyInstance);
                          
                          var pathValue = publicKeysValue['path'];
                          if (pathValue !== null && pathValue !== undefined) {
                            var pathInstance = pathValue;
                            sshPublicKeyInstance.path = pathInstance;
                          }
                          
                          var keyDataValue = publicKeysValue['keyData'];
                          if (keyDataValue !== null && keyDataValue !== undefined) {
                            var keyDataInstance = keyDataValue;
                            sshPublicKeyInstance.keyData = keyDataInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var secretsArray = osProfileValue['secrets'];
                  if (secretsArray !== null && secretsArray !== undefined) {
                    for (var loweredIndex6 = 0; loweredIndex6 < secretsArray.length; loweredIndex6 = loweredIndex6 + 1) {
                      var secretsValue = secretsArray[loweredIndex6];
                      var vaultSecretGroupInstance = { vaultCertificates: [] };
                      osProfileInstance.secrets.push(vaultSecretGroupInstance);
                      
                      var sourceVaultValue = secretsValue['sourceVault'];
                      if (sourceVaultValue !== null && sourceVaultValue !== undefined) {
                        var sourceVaultInstance = {};
                        vaultSecretGroupInstance.sourceVault = sourceVaultInstance;
                        
                        var idValue2 = sourceVaultValue['id'];
                        if (idValue2 !== null && idValue2 !== undefined) {
                          var idInstance2 = idValue2;
                          sourceVaultInstance.referenceUri = idInstance2;
                        }
                      }
                      
                      var vaultCertificatesArray = secretsValue['vaultCertificates'];
                      if (vaultCertificatesArray !== null && vaultCertificatesArray !== undefined) {
                        for (var loweredIndex7 = 0; loweredIndex7 < vaultCertificatesArray.length; loweredIndex7 = loweredIndex7 + 1) {
                          var vaultCertificatesValue = vaultCertificatesArray[loweredIndex7];
                          var vaultCertificateInstance = {};
                          vaultSecretGroupInstance.vaultCertificates.push(vaultCertificateInstance);
                          
                          var certificateUrlValue2 = vaultCertificatesValue['certificateUrl'];
                          if (certificateUrlValue2 !== null && certificateUrlValue2 !== undefined) {
                            var certificateUrlInstance2 = certificateUrlValue2;
                            vaultCertificateInstance.certificateUrl = certificateUrlInstance2;
                          }
                          
                          var certificateStoreValue = vaultCertificatesValue['certificateStore'];
                          if (certificateStoreValue !== null && certificateStoreValue !== undefined) {
                            var certificateStoreInstance = certificateStoreValue;
                            vaultCertificateInstance.certificateStore = certificateStoreInstance;
                          }
                        }
                      }
                    }
                  }
                }
                
                var networkProfileValue = propertiesValue['networkProfile'];
                if (networkProfileValue !== null && networkProfileValue !== undefined) {
                  var networkProfileInstance = { networkInterfaces: [] };
                  virtualMachineJsonInstance.networkProfile = networkProfileInstance;
                  
                  var networkInterfacesArray = networkProfileValue['networkInterfaces'];
                  if (networkInterfacesArray !== null && networkInterfacesArray !== undefined) {
                    for (var loweredIndex8 = 0; loweredIndex8 < networkInterfacesArray.length; loweredIndex8 = loweredIndex8 + 1) {
                      var networkInterfacesValue = networkInterfacesArray[loweredIndex8];
                      var networkInterfaceReferenceJsonInstance = {};
                      networkProfileInstance.networkInterfaces.push(networkInterfaceReferenceJsonInstance);
                      
                      var propertiesValue2 = networkInterfacesValue['properties'];
                      if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
                        var primaryValue = propertiesValue2['primary'];
                        if (primaryValue !== null && primaryValue !== undefined) {
                          var primaryInstance = primaryValue;
                          networkInterfaceReferenceJsonInstance.primary = primaryInstance;
                        }
                      }
                      
                      var idValue3 = networkInterfacesValue['id'];
                      if (idValue3 !== null && idValue3 !== undefined) {
                        var idInstance3 = idValue3;
                        networkInterfaceReferenceJsonInstance.referenceUri = idInstance3;
                      }
                    }
                  }
                }
                
                var availabilitySetValue = propertiesValue['availabilitySet'];
                if (availabilitySetValue !== null && availabilitySetValue !== undefined) {
                  var availabilitySetInstance = {};
                  virtualMachineJsonInstance.availabilitySetReference = availabilitySetInstance;
                  
                  var idValue4 = availabilitySetValue['id'];
                  if (idValue4 !== null && idValue4 !== undefined) {
                    var idInstance4 = idValue4;
                    availabilitySetInstance.referenceUri = idInstance4;
                  }
                }
                
                var provisioningStateValue = propertiesValue['provisioningState'];
                if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
                  var provisioningStateInstance = provisioningStateValue;
                  virtualMachineJsonInstance.provisioningState = provisioningStateInstance;
                }
              }
              
              var resourcesValue = valueValue['resources'];
              if (resourcesValue !== null && resourcesValue !== undefined) {
                var resourcesInstance = { extensions: [] };
                virtualMachineJsonInstance.resources = resourcesInstance;
                
                var extensionsArray = resourcesValue['extensions'];
                if (extensionsArray !== null && extensionsArray !== undefined) {
                  for (var loweredIndex9 = 0; loweredIndex9 < extensionsArray.length; loweredIndex9 = loweredIndex9 + 1) {
                    var extensionsValue = extensionsArray[loweredIndex9];
                    var virtualMachineExtensionJsonInstance = { tags: {} };
                    resourcesInstance.extensions.push(virtualMachineExtensionJsonInstance);
                    
                    var propertiesValue3 = extensionsValue['properties'];
                    if (propertiesValue3 !== null && propertiesValue3 !== undefined) {
                      var publisherValue2 = propertiesValue3['publisher'];
                      if (publisherValue2 !== null && publisherValue2 !== undefined) {
                        var publisherInstance2 = publisherValue2;
                        virtualMachineExtensionJsonInstance.publisher = publisherInstance2;
                      }
                      
                      var typeValue = propertiesValue3['type'];
                      if (typeValue !== null && typeValue !== undefined) {
                        var typeInstance = typeValue;
                        virtualMachineExtensionJsonInstance.extensionType = typeInstance;
                      }
                      
                      var typeHandlerVersionValue = propertiesValue3['typeHandlerVersion'];
                      if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                        var typeHandlerVersionInstance = typeHandlerVersionValue;
                        virtualMachineExtensionJsonInstance.typeHandlerVersion = typeHandlerVersionInstance;
                      }
                      
                      var settingsValue = propertiesValue3['settings'];
                      if (settingsValue !== null && settingsValue !== undefined) {
                        var settingsInstance = settingsValue;
                        virtualMachineExtensionJsonInstance.settings = settingsInstance;
                      }
                      
                      var protectedSettingsValue = propertiesValue3['protectedSettings'];
                      if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
                        var protectedSettingsInstance = protectedSettingsValue;
                        virtualMachineExtensionJsonInstance.protectedSettings = protectedSettingsInstance;
                      }
                      
                      var provisioningStateValue2 = propertiesValue3['provisioningState'];
                      if (provisioningStateValue2 !== null && provisioningStateValue2 !== undefined) {
                        var provisioningStateInstance2 = provisioningStateValue2;
                        virtualMachineExtensionJsonInstance.provisioningState = provisioningStateInstance2;
                      }
                      
                      var instanceViewValue = propertiesValue3['instanceView'];
                      if (instanceViewValue !== null && instanceViewValue !== undefined) {
                        var instanceViewInstance = { statuses: [], subStatuses: [] };
                        virtualMachineExtensionJsonInstance.instanceView = instanceViewInstance;
                        
                        var nameValue4 = instanceViewValue['name'];
                        if (nameValue4 !== null && nameValue4 !== undefined) {
                          var nameInstance4 = nameValue4;
                          instanceViewInstance.name = nameInstance4;
                        }
                        
                        var typeValue2 = instanceViewValue['type'];
                        if (typeValue2 !== null && typeValue2 !== undefined) {
                          var typeInstance2 = typeValue2;
                          instanceViewInstance.extensionType = typeInstance2;
                        }
                        
                        var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
                        if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                          var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                          instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
                        }
                        
                        var substatusesArray = instanceViewValue['substatuses'];
                        if (substatusesArray !== null && substatusesArray !== undefined) {
                          for (var loweredIndex10 = 0; loweredIndex10 < substatusesArray.length; loweredIndex10 = loweredIndex10 + 1) {
                            var substatusesValue = substatusesArray[loweredIndex10];
                            var instanceViewStatusInstance = {};
                            instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                            
                            var codeValue = substatusesValue['code'];
                            if (codeValue !== null && codeValue !== undefined) {
                              var codeInstance = codeValue;
                              instanceViewStatusInstance.code = codeInstance;
                            }
                            
                            var levelValue = substatusesValue['level'];
                            if (levelValue !== null && levelValue !== undefined) {
                              var levelInstance = levelValue;
                              instanceViewStatusInstance.level = levelInstance;
                            }
                            
                            var displayStatusValue = substatusesValue['displayStatus'];
                            if (displayStatusValue !== null && displayStatusValue !== undefined) {
                              var displayStatusInstance = displayStatusValue;
                              instanceViewStatusInstance.displayStatus = displayStatusInstance;
                            }
                            
                            var messageValue = substatusesValue['message'];
                            if (messageValue !== null && messageValue !== undefined) {
                              var messageInstance = messageValue;
                              instanceViewStatusInstance.message = messageInstance;
                            }
                            
                            var timeValue = substatusesValue['time'];
                            if (timeValue !== null && timeValue !== undefined) {
                              var timeInstance = timeValue;
                              instanceViewStatusInstance.time = timeInstance;
                            }
                          }
                        }
                        
                        var statusesArray = instanceViewValue['statuses'];
                        if (statusesArray !== null && statusesArray !== undefined) {
                          for (var loweredIndex11 = 0; loweredIndex11 < statusesArray.length; loweredIndex11 = loweredIndex11 + 1) {
                            var statusesValue = statusesArray[loweredIndex11];
                            var instanceViewStatusInstance2 = {};
                            instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                            
                            var codeValue2 = statusesValue['code'];
                            if (codeValue2 !== null && codeValue2 !== undefined) {
                              var codeInstance2 = codeValue2;
                              instanceViewStatusInstance2.code = codeInstance2;
                            }
                            
                            var levelValue2 = statusesValue['level'];
                            if (levelValue2 !== null && levelValue2 !== undefined) {
                              var levelInstance2 = levelValue2;
                              instanceViewStatusInstance2.level = levelInstance2;
                            }
                            
                            var displayStatusValue2 = statusesValue['displayStatus'];
                            if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                              var displayStatusInstance2 = displayStatusValue2;
                              instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                            }
                            
                            var messageValue2 = statusesValue['message'];
                            if (messageValue2 !== null && messageValue2 !== undefined) {
                              var messageInstance2 = messageValue2;
                              instanceViewStatusInstance2.message = messageInstance2;
                            }
                            
                            var timeValue2 = statusesValue['time'];
                            if (timeValue2 !== null && timeValue2 !== undefined) {
                              var timeInstance2 = timeValue2;
                              instanceViewStatusInstance2.time = timeInstance2;
                            }
                          }
                        }
                      }
                    }
                    
                    var idValue5 = extensionsValue['id'];
                    if (idValue5 !== null && idValue5 !== undefined) {
                      var idInstance5 = idValue5;
                      virtualMachineExtensionJsonInstance.id = idInstance5;
                    }
                    
                    var nameValue5 = extensionsValue['name'];
                    if (nameValue5 !== null && nameValue5 !== undefined) {
                      var nameInstance5 = nameValue5;
                      virtualMachineExtensionJsonInstance.name = nameInstance5;
                    }
                    
                    var typeValue3 = extensionsValue['type'];
                    if (typeValue3 !== null && typeValue3 !== undefined) {
                      var typeInstance3 = typeValue3;
                      virtualMachineExtensionJsonInstance.type = typeInstance3;
                    }
                    
                    var locationValue = extensionsValue['location'];
                    if (locationValue !== null && locationValue !== undefined) {
                      var locationInstance = locationValue;
                      virtualMachineExtensionJsonInstance.location = locationInstance;
                    }
                    
                    var tagsSequenceElement = extensionsValue['tags'];
                    if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                      for (var property in tagsSequenceElement) {
                        var tagsKey = property;
                        var tagsValue = tagsSequenceElement[property];
                        virtualMachineExtensionJsonInstance.tags[tagsKey] = tagsValue;
                      }
                    }
                  }
                }
              }
              
              var idValue6 = valueValue['id'];
              if (idValue6 !== null && idValue6 !== undefined) {
                var idInstance6 = idValue6;
                virtualMachineJsonInstance.id = idInstance6;
              }
              
              var nameValue6 = valueValue['name'];
              if (nameValue6 !== null && nameValue6 !== undefined) {
                var nameInstance6 = nameValue6;
                virtualMachineJsonInstance.name = nameInstance6;
              }
              
              var typeValue4 = valueValue['type'];
              if (typeValue4 !== null && typeValue4 !== undefined) {
                var typeInstance4 = typeValue4;
                virtualMachineJsonInstance.type = typeInstance4;
              }
              
              var locationValue2 = valueValue['location'];
              if (locationValue2 !== null && locationValue2 !== undefined) {
                var locationInstance2 = locationValue2;
                virtualMachineJsonInstance.location = locationInstance2;
              }
              
              var tagsSequenceElement2 = valueValue['tags'];
              if (tagsSequenceElement2 !== null && tagsSequenceElement2 !== undefined) {
                for (var property2 in tagsSequenceElement2) {
                  var tagsKey2 = property2;
                  var tagsValue2 = tagsSequenceElement2[property2];
                  virtualMachineJsonInstance.tags[tagsKey2] = tagsValue2;
                }
              }
            }
          }
          
          var odatanextLinkValue = responseDoc['@odata.nextLink'];
          if (odatanextLinkValue !== null && odatanextLinkValue !== undefined) {
            var odatanextLinkInstance = odatanextLinkValue;
            result.nextLink = odatanextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Lists virtual-machine-sizes available to be used for a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.listAvailableSizes = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroupName === null || resourceGroupName === undefined) {
      return callback(new Error('resourceGroupName cannot be null.'));
    }
    if (vmName === null || vmName === undefined) {
      return callback(new Error('vmName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourceGroups/';
    url2 = url2 + encodeURIComponent(resourceGroupName);
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/';
    url2 = url2 + 'virtualMachines';
    url2 = url2 + '/';
    url2 = url2 + encodeURIComponent(vmName);
    url2 = url2 + '/vmSizes';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachineSizes: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineSizeInstance = {};
              result.virtualMachineSizes.push(virtualMachineSizeInstance);
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                virtualMachineSizeInstance.name = nameInstance;
              }
              
              var numberOfCoresValue = valueValue['numberOfCores'];
              if (numberOfCoresValue !== null && numberOfCoresValue !== undefined) {
                var numberOfCoresInstance = numberOfCoresValue;
                virtualMachineSizeInstance.numberOfCores = numberOfCoresInstance;
              }
              
              var osDiskSizeInMBValue = valueValue['osDiskSizeInMB'];
              if (osDiskSizeInMBValue !== null && osDiskSizeInMBValue !== undefined) {
                var osDiskSizeInMBInstance = osDiskSizeInMBValue;
                virtualMachineSizeInstance.oSDiskSizeInMB = osDiskSizeInMBInstance;
              }
              
              var resourceDiskSizeInMBValue = valueValue['resourceDiskSizeInMB'];
              if (resourceDiskSizeInMBValue !== null && resourceDiskSizeInMBValue !== undefined) {
                var resourceDiskSizeInMBInstance = resourceDiskSizeInMBValue;
                virtualMachineSizeInstance.resourceDiskSizeInMB = resourceDiskSizeInMBInstance;
              }
              
              var memoryInMBValue = valueValue['memoryInMB'];
              if (memoryInMBValue !== null && memoryInMBValue !== undefined) {
                var memoryInMBInstance = memoryInMBValue;
                virtualMachineSizeInstance.memoryInMB = memoryInMBInstance;
              }
              
              var maxDataDiskCountValue = valueValue['maxDataDiskCount'];
              if (maxDataDiskCountValue !== null && maxDataDiskCountValue !== undefined) {
                var maxDataDiskCountInstance = maxDataDiskCountValue;
                virtualMachineSizeInstance.maxDataDiskCount = maxDataDiskCountInstance;
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Gets the next page of Virtual Machines. NextLink is obtained by making a
   * ListAll() callwhich fetches the first page of Virtual Machines and a link
   * to fetch the next page.
   * 
   * @param {string} nextLink NextLink from the previous successful call to
   * ListVirtualMachines operation.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.listNext = function(nextLink, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (nextLink === null || nextLink === undefined) {
      return callback(new Error('nextLink cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + nextLink;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachines: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineJsonInstance = { tags: {} };
              result.virtualMachines.push(virtualMachineJsonInstance);
              
              var planValue = valueValue['plan'];
              if (planValue !== null && planValue !== undefined) {
                var planInstance = {};
                virtualMachineJsonInstance.plan = planInstance;
                
                var nameValue = planValue['name'];
                if (nameValue !== null && nameValue !== undefined) {
                  var nameInstance = nameValue;
                  planInstance.name = nameInstance;
                }
                
                var publisherValue = planValue['publisher'];
                if (publisherValue !== null && publisherValue !== undefined) {
                  var publisherInstance = publisherValue;
                  planInstance.publisher = publisherInstance;
                }
                
                var productValue = planValue['product'];
                if (productValue !== null && productValue !== undefined) {
                  var productInstance = productValue;
                  planInstance.product = productInstance;
                }
                
                var promotionCodeValue = planValue['promotionCode'];
                if (promotionCodeValue !== null && promotionCodeValue !== undefined) {
                  var promotionCodeInstance = promotionCodeValue;
                  planInstance.promotionCode = promotionCodeInstance;
                }
              }
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var hardwareProfileValue = propertiesValue['hardwareProfile'];
                if (hardwareProfileValue !== null && hardwareProfileValue !== undefined) {
                  var hardwareProfileInstance = {};
                  virtualMachineJsonInstance.hardwareProfile = hardwareProfileInstance;
                  
                  var vmSizeValue = hardwareProfileValue['vmSize'];
                  if (vmSizeValue !== null && vmSizeValue !== undefined) {
                    var vmSizeInstance = vmSizeValue;
                    hardwareProfileInstance.virtualMachineSize = vmSizeInstance;
                  }
                }
                
                var storageProfileValue = propertiesValue['storageProfile'];
                if (storageProfileValue !== null && storageProfileValue !== undefined) {
                  var storageProfileInstance = { dataDisks: [] };
                  virtualMachineJsonInstance.storageProfile = storageProfileInstance;
                  
                  var sourceImageValue = storageProfileValue['sourceImage'];
                  if (sourceImageValue !== null && sourceImageValue !== undefined) {
                    var sourceImageInstance = {};
                    storageProfileInstance.sourceImage = sourceImageInstance;
                    
                    var idValue = sourceImageValue['id'];
                    if (idValue !== null && idValue !== undefined) {
                      var idInstance = idValue;
                      sourceImageInstance.referenceUri = idInstance;
                    }
                  }
                  
                  var destinationVhdsContainerValue = storageProfileValue['destinationVhdsContainer'];
                  if (destinationVhdsContainerValue !== null && destinationVhdsContainerValue !== undefined) {
                    var destinationVhdsContainerInstance = destinationVhdsContainerValue;
                    storageProfileInstance.destinationVhdsContainer = destinationVhdsContainerInstance;
                  }
                  
                  var osDiskValue = storageProfileValue['osDisk'];
                  if (osDiskValue !== null && osDiskValue !== undefined) {
                    var osDiskInstance = {};
                    storageProfileInstance.oSDisk = osDiskInstance;
                    
                    var osTypeValue = osDiskValue['osType'];
                    if (osTypeValue !== null && osTypeValue !== undefined) {
                      var osTypeInstance = osTypeValue;
                      osDiskInstance.operatingSystemType = osTypeInstance;
                    }
                    
                    var nameValue2 = osDiskValue['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      osDiskInstance.name = nameInstance2;
                    }
                    
                    var vhdValue = osDiskValue['vhd'];
                    if (vhdValue !== null && vhdValue !== undefined) {
                      var vhdInstance = {};
                      osDiskInstance.virtualHardDisk = vhdInstance;
                      
                      var uriValue = vhdValue['uri'];
                      if (uriValue !== null && uriValue !== undefined) {
                        var uriInstance = uriValue;
                        vhdInstance.uri = uriInstance;
                      }
                    }
                    
                    var imageValue = osDiskValue['image'];
                    if (imageValue !== null && imageValue !== undefined) {
                      var imageInstance = {};
                      osDiskInstance.sourceImage = imageInstance;
                      
                      var uriValue2 = imageValue['uri'];
                      if (uriValue2 !== null && uriValue2 !== undefined) {
                        var uriInstance2 = uriValue2;
                        imageInstance.uri = uriInstance2;
                      }
                    }
                    
                    var cachingValue = osDiskValue['caching'];
                    if (cachingValue !== null && cachingValue !== undefined) {
                      var cachingInstance = cachingValue;
                      osDiskInstance.caching = cachingInstance;
                    }
                  }
                  
                  var dataDisksArray = storageProfileValue['dataDisks'];
                  if (dataDisksArray !== null && dataDisksArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < dataDisksArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var dataDisksValue = dataDisksArray[loweredIndex2];
                      var dataDiskInstance = {};
                      storageProfileInstance.dataDisks.push(dataDiskInstance);
                      
                      var lunValue = dataDisksValue['lun'];
                      if (lunValue !== null && lunValue !== undefined) {
                        var lunInstance = lunValue;
                        dataDiskInstance.lun = lunInstance;
                      }
                      
                      var diskSizeGBValue = dataDisksValue['diskSizeGB'];
                      if (diskSizeGBValue !== null && diskSizeGBValue !== undefined) {
                        var diskSizeGBInstance = diskSizeGBValue;
                        dataDiskInstance.diskSizeGB = diskSizeGBInstance;
                      }
                      
                      var nameValue3 = dataDisksValue['name'];
                      if (nameValue3 !== null && nameValue3 !== undefined) {
                        var nameInstance3 = nameValue3;
                        dataDiskInstance.name = nameInstance3;
                      }
                      
                      var vhdValue2 = dataDisksValue['vhd'];
                      if (vhdValue2 !== null && vhdValue2 !== undefined) {
                        var vhdInstance2 = {};
                        dataDiskInstance.virtualHardDisk = vhdInstance2;
                        
                        var uriValue3 = vhdValue2['uri'];
                        if (uriValue3 !== null && uriValue3 !== undefined) {
                          var uriInstance3 = uriValue3;
                          vhdInstance2.uri = uriInstance3;
                        }
                      }
                      
                      var imageValue2 = dataDisksValue['image'];
                      if (imageValue2 !== null && imageValue2 !== undefined) {
                        var imageInstance2 = {};
                        dataDiskInstance.sourceImage = imageInstance2;
                        
                        var uriValue4 = imageValue2['uri'];
                        if (uriValue4 !== null && uriValue4 !== undefined) {
                          var uriInstance4 = uriValue4;
                          imageInstance2.uri = uriInstance4;
                        }
                      }
                      
                      var cachingValue2 = dataDisksValue['caching'];
                      if (cachingValue2 !== null && cachingValue2 !== undefined) {
                        var cachingInstance2 = cachingValue2;
                        dataDiskInstance.caching = cachingInstance2;
                      }
                    }
                  }
                }
                
                var osProfileValue = propertiesValue['osProfile'];
                if (osProfileValue !== null && osProfileValue !== undefined) {
                  var osProfileInstance = { secrets: [] };
                  virtualMachineJsonInstance.oSProfile = osProfileInstance;
                  
                  var computerNameValue = osProfileValue['computerName'];
                  if (computerNameValue !== null && computerNameValue !== undefined) {
                    var computerNameInstance = computerNameValue;
                    osProfileInstance.computerName = computerNameInstance;
                  }
                  
                  var adminUsernameValue = osProfileValue['adminUsername'];
                  if (adminUsernameValue !== null && adminUsernameValue !== undefined) {
                    var adminUsernameInstance = adminUsernameValue;
                    osProfileInstance.adminUsername = adminUsernameInstance;
                  }
                  
                  var adminPasswordValue = osProfileValue['adminPassword'];
                  if (adminPasswordValue !== null && adminPasswordValue !== undefined) {
                    var adminPasswordInstance = adminPasswordValue;
                    osProfileInstance.adminPassword = adminPasswordInstance;
                  }
                  
                  var customDataValue = osProfileValue['customData'];
                  if (customDataValue !== null && customDataValue !== undefined) {
                    var customDataInstance = customDataValue;
                    osProfileInstance.customData = customDataInstance;
                  }
                  
                  var windowsConfigurationValue = osProfileValue['windowsConfiguration'];
                  if (windowsConfigurationValue !== null && windowsConfigurationValue !== undefined) {
                    var windowsConfigurationInstance = { additionalUnattendContents: [] };
                    osProfileInstance.windowsConfiguration = windowsConfigurationInstance;
                    
                    var provisionVMAgentValue = windowsConfigurationValue['provisionVMAgent'];
                    if (provisionVMAgentValue !== null && provisionVMAgentValue !== undefined) {
                      var provisionVMAgentInstance = provisionVMAgentValue;
                      windowsConfigurationInstance.provisionVMAgent = provisionVMAgentInstance;
                    }
                    
                    var enableAutomaticUpdatesValue = windowsConfigurationValue['enableAutomaticUpdates'];
                    if (enableAutomaticUpdatesValue !== null && enableAutomaticUpdatesValue !== undefined) {
                      var enableAutomaticUpdatesInstance = enableAutomaticUpdatesValue;
                      windowsConfigurationInstance.enableAutomaticUpdates = enableAutomaticUpdatesInstance;
                    }
                    
                    var timeZoneValue = windowsConfigurationValue['timeZone'];
                    if (timeZoneValue !== null && timeZoneValue !== undefined) {
                      var timeZoneInstance = timeZoneValue;
                      windowsConfigurationInstance.timeZone = timeZoneInstance;
                    }
                    
                    var additionalUnattendContentArray = windowsConfigurationValue['additionalUnattendContent'];
                    if (additionalUnattendContentArray !== null && additionalUnattendContentArray !== undefined) {
                      for (var loweredIndex3 = 0; loweredIndex3 < additionalUnattendContentArray.length; loweredIndex3 = loweredIndex3 + 1) {
                        var additionalUnattendContentValue = additionalUnattendContentArray[loweredIndex3];
                        var additionalUnattendContentInstance = {};
                        windowsConfigurationInstance.additionalUnattendContents.push(additionalUnattendContentInstance);
                        
                        var passNameValue = additionalUnattendContentValue['passName'];
                        if (passNameValue !== null && passNameValue !== undefined) {
                          var passNameInstance = passNameValue;
                          additionalUnattendContentInstance.passName = passNameInstance;
                        }
                        
                        var componentNameValue = additionalUnattendContentValue['componentName'];
                        if (componentNameValue !== null && componentNameValue !== undefined) {
                          var componentNameInstance = componentNameValue;
                          additionalUnattendContentInstance.componentName = componentNameInstance;
                        }
                        
                        var settingNameValue = additionalUnattendContentValue['settingName'];
                        if (settingNameValue !== null && settingNameValue !== undefined) {
                          var settingNameInstance = settingNameValue;
                          additionalUnattendContentInstance.settingName = settingNameInstance;
                        }
                        
                        var contentValue = additionalUnattendContentValue['content'];
                        if (contentValue !== null && contentValue !== undefined) {
                          var contentInstance = contentValue;
                          additionalUnattendContentInstance.content = contentInstance;
                        }
                      }
                    }
                    
                    var winRMValue = windowsConfigurationValue['winRM'];
                    if (winRMValue !== null && winRMValue !== undefined) {
                      var winRMInstance = { listeners: [] };
                      windowsConfigurationInstance.winRMConfiguration = winRMInstance;
                      
                      var listenersArray = winRMValue['listeners'];
                      if (listenersArray !== null && listenersArray !== undefined) {
                        for (var loweredIndex4 = 0; loweredIndex4 < listenersArray.length; loweredIndex4 = loweredIndex4 + 1) {
                          var listenersValue = listenersArray[loweredIndex4];
                          var winRMListenerInstance = {};
                          winRMInstance.listeners.push(winRMListenerInstance);
                          
                          var protocolValue = listenersValue['protocol'];
                          if (protocolValue !== null && protocolValue !== undefined) {
                            var protocolInstance = protocolValue;
                            winRMListenerInstance.protocol = protocolInstance;
                          }
                          
                          var certificateUrlValue = listenersValue['certificateUrl'];
                          if (certificateUrlValue !== null && certificateUrlValue !== undefined) {
                            var certificateUrlInstance = certificateUrlValue;
                            winRMListenerInstance.certificateUrl = certificateUrlInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var linuxConfigurationValue = osProfileValue['linuxConfiguration'];
                  if (linuxConfigurationValue !== null && linuxConfigurationValue !== undefined) {
                    var linuxConfigurationInstance = {};
                    osProfileInstance.linuxConfiguration = linuxConfigurationInstance;
                    
                    var disablePasswordAuthenticationValue = linuxConfigurationValue['disablePasswordAuthentication'];
                    if (disablePasswordAuthenticationValue !== null && disablePasswordAuthenticationValue !== undefined) {
                      var disablePasswordAuthenticationInstance = disablePasswordAuthenticationValue;
                      linuxConfigurationInstance.disablePasswordAuthentication = disablePasswordAuthenticationInstance;
                    }
                    
                    var sshValue = linuxConfigurationValue['ssh'];
                    if (sshValue !== null && sshValue !== undefined) {
                      var sshInstance = { publicKeys: [] };
                      linuxConfigurationInstance.sshConfiguration = sshInstance;
                      
                      var publicKeysArray = sshValue['publicKeys'];
                      if (publicKeysArray !== null && publicKeysArray !== undefined) {
                        for (var loweredIndex5 = 0; loweredIndex5 < publicKeysArray.length; loweredIndex5 = loweredIndex5 + 1) {
                          var publicKeysValue = publicKeysArray[loweredIndex5];
                          var sshPublicKeyInstance = {};
                          sshInstance.publicKeys.push(sshPublicKeyInstance);
                          
                          var pathValue = publicKeysValue['path'];
                          if (pathValue !== null && pathValue !== undefined) {
                            var pathInstance = pathValue;
                            sshPublicKeyInstance.path = pathInstance;
                          }
                          
                          var keyDataValue = publicKeysValue['keyData'];
                          if (keyDataValue !== null && keyDataValue !== undefined) {
                            var keyDataInstance = keyDataValue;
                            sshPublicKeyInstance.keyData = keyDataInstance;
                          }
                        }
                      }
                    }
                  }
                  
                  var secretsArray = osProfileValue['secrets'];
                  if (secretsArray !== null && secretsArray !== undefined) {
                    for (var loweredIndex6 = 0; loweredIndex6 < secretsArray.length; loweredIndex6 = loweredIndex6 + 1) {
                      var secretsValue = secretsArray[loweredIndex6];
                      var vaultSecretGroupInstance = { vaultCertificates: [] };
                      osProfileInstance.secrets.push(vaultSecretGroupInstance);
                      
                      var sourceVaultValue = secretsValue['sourceVault'];
                      if (sourceVaultValue !== null && sourceVaultValue !== undefined) {
                        var sourceVaultInstance = {};
                        vaultSecretGroupInstance.sourceVault = sourceVaultInstance;
                        
                        var idValue2 = sourceVaultValue['id'];
                        if (idValue2 !== null && idValue2 !== undefined) {
                          var idInstance2 = idValue2;
                          sourceVaultInstance.referenceUri = idInstance2;
                        }
                      }
                      
                      var vaultCertificatesArray = secretsValue['vaultCertificates'];
                      if (vaultCertificatesArray !== null && vaultCertificatesArray !== undefined) {
                        for (var loweredIndex7 = 0; loweredIndex7 < vaultCertificatesArray.length; loweredIndex7 = loweredIndex7 + 1) {
                          var vaultCertificatesValue = vaultCertificatesArray[loweredIndex7];
                          var vaultCertificateInstance = {};
                          vaultSecretGroupInstance.vaultCertificates.push(vaultCertificateInstance);
                          
                          var certificateUrlValue2 = vaultCertificatesValue['certificateUrl'];
                          if (certificateUrlValue2 !== null && certificateUrlValue2 !== undefined) {
                            var certificateUrlInstance2 = certificateUrlValue2;
                            vaultCertificateInstance.certificateUrl = certificateUrlInstance2;
                          }
                          
                          var certificateStoreValue = vaultCertificatesValue['certificateStore'];
                          if (certificateStoreValue !== null && certificateStoreValue !== undefined) {
                            var certificateStoreInstance = certificateStoreValue;
                            vaultCertificateInstance.certificateStore = certificateStoreInstance;
                          }
                        }
                      }
                    }
                  }
                }
                
                var networkProfileValue = propertiesValue['networkProfile'];
                if (networkProfileValue !== null && networkProfileValue !== undefined) {
                  var networkProfileInstance = { networkInterfaces: [] };
                  virtualMachineJsonInstance.networkProfile = networkProfileInstance;
                  
                  var networkInterfacesArray = networkProfileValue['networkInterfaces'];
                  if (networkInterfacesArray !== null && networkInterfacesArray !== undefined) {
                    for (var loweredIndex8 = 0; loweredIndex8 < networkInterfacesArray.length; loweredIndex8 = loweredIndex8 + 1) {
                      var networkInterfacesValue = networkInterfacesArray[loweredIndex8];
                      var networkInterfaceReferenceJsonInstance = {};
                      networkProfileInstance.networkInterfaces.push(networkInterfaceReferenceJsonInstance);
                      
                      var propertiesValue2 = networkInterfacesValue['properties'];
                      if (propertiesValue2 !== null && propertiesValue2 !== undefined) {
                        var primaryValue = propertiesValue2['primary'];
                        if (primaryValue !== null && primaryValue !== undefined) {
                          var primaryInstance = primaryValue;
                          networkInterfaceReferenceJsonInstance.primary = primaryInstance;
                        }
                      }
                      
                      var idValue3 = networkInterfacesValue['id'];
                      if (idValue3 !== null && idValue3 !== undefined) {
                        var idInstance3 = idValue3;
                        networkInterfaceReferenceJsonInstance.referenceUri = idInstance3;
                      }
                    }
                  }
                }
                
                var availabilitySetValue = propertiesValue['availabilitySet'];
                if (availabilitySetValue !== null && availabilitySetValue !== undefined) {
                  var availabilitySetInstance = {};
                  virtualMachineJsonInstance.availabilitySetReference = availabilitySetInstance;
                  
                  var idValue4 = availabilitySetValue['id'];
                  if (idValue4 !== null && idValue4 !== undefined) {
                    var idInstance4 = idValue4;
                    availabilitySetInstance.referenceUri = idInstance4;
                  }
                }
                
                var provisioningStateValue = propertiesValue['provisioningState'];
                if (provisioningStateValue !== null && provisioningStateValue !== undefined) {
                  var provisioningStateInstance = provisioningStateValue;
                  virtualMachineJsonInstance.provisioningState = provisioningStateInstance;
                }
              }
              
              var resourcesValue = valueValue['resources'];
              if (resourcesValue !== null && resourcesValue !== undefined) {
                var resourcesInstance = { extensions: [] };
                virtualMachineJsonInstance.resources = resourcesInstance;
                
                var extensionsArray = resourcesValue['extensions'];
                if (extensionsArray !== null && extensionsArray !== undefined) {
                  for (var loweredIndex9 = 0; loweredIndex9 < extensionsArray.length; loweredIndex9 = loweredIndex9 + 1) {
                    var extensionsValue = extensionsArray[loweredIndex9];
                    var virtualMachineExtensionJsonInstance = { tags: {} };
                    resourcesInstance.extensions.push(virtualMachineExtensionJsonInstance);
                    
                    var propertiesValue3 = extensionsValue['properties'];
                    if (propertiesValue3 !== null && propertiesValue3 !== undefined) {
                      var publisherValue2 = propertiesValue3['publisher'];
                      if (publisherValue2 !== null && publisherValue2 !== undefined) {
                        var publisherInstance2 = publisherValue2;
                        virtualMachineExtensionJsonInstance.publisher = publisherInstance2;
                      }
                      
                      var typeValue = propertiesValue3['type'];
                      if (typeValue !== null && typeValue !== undefined) {
                        var typeInstance = typeValue;
                        virtualMachineExtensionJsonInstance.extensionType = typeInstance;
                      }
                      
                      var typeHandlerVersionValue = propertiesValue3['typeHandlerVersion'];
                      if (typeHandlerVersionValue !== null && typeHandlerVersionValue !== undefined) {
                        var typeHandlerVersionInstance = typeHandlerVersionValue;
                        virtualMachineExtensionJsonInstance.typeHandlerVersion = typeHandlerVersionInstance;
                      }
                      
                      var settingsValue = propertiesValue3['settings'];
                      if (settingsValue !== null && settingsValue !== undefined) {
                        var settingsInstance = settingsValue;
                        virtualMachineExtensionJsonInstance.settings = settingsInstance;
                      }
                      
                      var protectedSettingsValue = propertiesValue3['protectedSettings'];
                      if (protectedSettingsValue !== null && protectedSettingsValue !== undefined) {
                        var protectedSettingsInstance = protectedSettingsValue;
                        virtualMachineExtensionJsonInstance.protectedSettings = protectedSettingsInstance;
                      }
                      
                      var provisioningStateValue2 = propertiesValue3['provisioningState'];
                      if (provisioningStateValue2 !== null && provisioningStateValue2 !== undefined) {
                        var provisioningStateInstance2 = provisioningStateValue2;
                        virtualMachineExtensionJsonInstance.provisioningState = provisioningStateInstance2;
                      }
                      
                      var instanceViewValue = propertiesValue3['instanceView'];
                      if (instanceViewValue !== null && instanceViewValue !== undefined) {
                        var instanceViewInstance = { statuses: [], subStatuses: [] };
                        virtualMachineExtensionJsonInstance.instanceView = instanceViewInstance;
                        
                        var nameValue4 = instanceViewValue['name'];
                        if (nameValue4 !== null && nameValue4 !== undefined) {
                          var nameInstance4 = nameValue4;
                          instanceViewInstance.name = nameInstance4;
                        }
                        
                        var typeValue2 = instanceViewValue['type'];
                        if (typeValue2 !== null && typeValue2 !== undefined) {
                          var typeInstance2 = typeValue2;
                          instanceViewInstance.extensionType = typeInstance2;
                        }
                        
                        var typeHandlerVersionValue2 = instanceViewValue['typeHandlerVersion'];
                        if (typeHandlerVersionValue2 !== null && typeHandlerVersionValue2 !== undefined) {
                          var typeHandlerVersionInstance2 = typeHandlerVersionValue2;
                          instanceViewInstance.typeHandlerVersion = typeHandlerVersionInstance2;
                        }
                        
                        var substatusesArray = instanceViewValue['substatuses'];
                        if (substatusesArray !== null && substatusesArray !== undefined) {
                          for (var loweredIndex10 = 0; loweredIndex10 < substatusesArray.length; loweredIndex10 = loweredIndex10 + 1) {
                            var substatusesValue = substatusesArray[loweredIndex10];
                            var instanceViewStatusInstance = {};
                            instanceViewInstance.subStatuses.push(instanceViewStatusInstance);
                            
                            var codeValue = substatusesValue['code'];
                            if (codeValue !== null && codeValue !== undefined) {
                              var codeInstance = codeValue;
                              instanceViewStatusInstance.code = codeInstance;
                            }
                            
                            var levelValue = substatusesValue['level'];
                            if (levelValue !== null && levelValue !== undefined) {
                              var levelInstance = levelValue;
                              instanceViewStatusInstance.level = levelInstance;
                            }
                            
                            var displayStatusValue = substatusesValue['displayStatus'];
                            if (displayStatusValue !== null && displayStatusValue !== undefined) {
                              var displayStatusInstance = displayStatusValue;
                              instanceViewStatusInstance.displayStatus = displayStatusInstance;
                            }
                            
                            var messageValue = substatusesValue['message'];
                            if (messageValue !== null && messageValue !== undefined) {
                              var messageInstance = messageValue;
                              instanceViewStatusInstance.message = messageInstance;
                            }
                            
                            var timeValue = substatusesValue['time'];
                            if (timeValue !== null && timeValue !== undefined) {
                              var timeInstance = timeValue;
                              instanceViewStatusInstance.time = timeInstance;
                            }
                          }
                        }
                        
                        var statusesArray = instanceViewValue['statuses'];
                        if (statusesArray !== null && statusesArray !== undefined) {
                          for (var loweredIndex11 = 0; loweredIndex11 < statusesArray.length; loweredIndex11 = loweredIndex11 + 1) {
                            var statusesValue = statusesArray[loweredIndex11];
                            var instanceViewStatusInstance2 = {};
                            instanceViewInstance.statuses.push(instanceViewStatusInstance2);
                            
                            var codeValue2 = statusesValue['code'];
                            if (codeValue2 !== null && codeValue2 !== undefined) {
                              var codeInstance2 = codeValue2;
                              instanceViewStatusInstance2.code = codeInstance2;
                            }
                            
                            var levelValue2 = statusesValue['level'];
                            if (levelValue2 !== null && levelValue2 !== undefined) {
                              var levelInstance2 = levelValue2;
                              instanceViewStatusInstance2.level = levelInstance2;
                            }
                            
                            var displayStatusValue2 = statusesValue['displayStatus'];
                            if (displayStatusValue2 !== null && displayStatusValue2 !== undefined) {
                              var displayStatusInstance2 = displayStatusValue2;
                              instanceViewStatusInstance2.displayStatus = displayStatusInstance2;
                            }
                            
                            var messageValue2 = statusesValue['message'];
                            if (messageValue2 !== null && messageValue2 !== undefined) {
                              var messageInstance2 = messageValue2;
                              instanceViewStatusInstance2.message = messageInstance2;
                            }
                            
                            var timeValue2 = statusesValue['time'];
                            if (timeValue2 !== null && timeValue2 !== undefined) {
                              var timeInstance2 = timeValue2;
                              instanceViewStatusInstance2.time = timeInstance2;
                            }
                          }
                        }
                      }
                    }
                    
                    var idValue5 = extensionsValue['id'];
                    if (idValue5 !== null && idValue5 !== undefined) {
                      var idInstance5 = idValue5;
                      virtualMachineExtensionJsonInstance.id = idInstance5;
                    }
                    
                    var nameValue5 = extensionsValue['name'];
                    if (nameValue5 !== null && nameValue5 !== undefined) {
                      var nameInstance5 = nameValue5;
                      virtualMachineExtensionJsonInstance.name = nameInstance5;
                    }
                    
                    var typeValue3 = extensionsValue['type'];
                    if (typeValue3 !== null && typeValue3 !== undefined) {
                      var typeInstance3 = typeValue3;
                      virtualMachineExtensionJsonInstance.type = typeInstance3;
                    }
                    
                    var locationValue = extensionsValue['location'];
                    if (locationValue !== null && locationValue !== undefined) {
                      var locationInstance = locationValue;
                      virtualMachineExtensionJsonInstance.location = locationInstance;
                    }
                    
                    var tagsSequenceElement = extensionsValue['tags'];
                    if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                      for (var property in tagsSequenceElement) {
                        var tagsKey = property;
                        var tagsValue = tagsSequenceElement[property];
                        virtualMachineExtensionJsonInstance.tags[tagsKey] = tagsValue;
                      }
                    }
                  }
                }
              }
              
              var idValue6 = valueValue['id'];
              if (idValue6 !== null && idValue6 !== undefined) {
                var idInstance6 = idValue6;
                virtualMachineJsonInstance.id = idInstance6;
              }
              
              var nameValue6 = valueValue['name'];
              if (nameValue6 !== null && nameValue6 !== undefined) {
                var nameInstance6 = nameValue6;
                virtualMachineJsonInstance.name = nameInstance6;
              }
              
              var typeValue4 = valueValue['type'];
              if (typeValue4 !== null && typeValue4 !== undefined) {
                var typeInstance4 = typeValue4;
                virtualMachineJsonInstance.type = typeInstance4;
              }
              
              var locationValue2 = valueValue['location'];
              if (locationValue2 !== null && locationValue2 !== undefined) {
                var locationInstance2 = locationValue2;
                virtualMachineJsonInstance.location = locationInstance2;
              }
              
              var tagsSequenceElement2 = valueValue['tags'];
              if (tagsSequenceElement2 !== null && tagsSequenceElement2 !== undefined) {
                for (var property2 in tagsSequenceElement2) {
                  var tagsKey2 = property2;
                  var tagsValue2 = tagsSequenceElement2[property2];
                  virtualMachineJsonInstance.tags[tagsKey2] = tagsValue2;
                }
              }
            }
          }
          
          var odatanextLinkValue = responseDoc['@odata.nextLink'];
          if (odatanextLinkValue !== null && odatanextLinkValue !== undefined) {
            var odatanextLinkInstance = odatanextLinkValue;
            result.nextLink = odatanextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.restart = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginRestarting(resourceGroupName, vmName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.start = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginStarting(resourceGroupName, vmName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  /**
   * The operation to stop a virtual machine.
   * 
   * @param {string} resourceGroupName The name of the resource group.
   * 
   * @param {string} vmName The name of the virtual machine.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineOperations.prototype.stop = function(resourceGroupName, vmName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    var client2 = this.client;
    
    return client2.virtualMachines.beginStopping(resourceGroupName, vmName, function (err, result) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var delayInSeconds = 30;
      if (client2.longRunningOperationInitialTimeout >= 0) {
        delayInSeconds = client2.longRunningOperationInitialTimeout;
      }
      
      var pollOperation = function () {
        setTimeout(function () {
          return client2.getLongRunningOperationStatus(result.azureAsyncOperation, function (err2, result2) {
            if (err2 !== null && err2 !== undefined) {
              return callback(err2);
            } else {
              if (result2.status !== 'InProgress') {
                return callback(null, result2);
              } else {
                delayInSeconds = 30;
                if (client2.longRunningOperationRetryTimeout >= 0) {
                  delayInSeconds = client2.longRunningOperationRetryTimeout;
                }
                pollOperation();
              }
            }
          });
        }, delayInSeconds * 1000);
      };
      
      pollOperation();
    });
  };
  
  return VirtualMachineOperations;
})();

var VirtualMachineSizeOperations = ( /** @lends VirtualMachineSizeOperations */ function() {
  /**
   * @class
   * Operations for listing virtual machine sizes available in a region.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ComputeManagementClient] {@link
   * ComputeManagementClient~ComputeManagementClient}.
   * See [virtualMachineSizes] {@link
   * ComputeManagementClient~ComputeManagementClient#virtualMachineSizes}.
   * Initializes a new instance of the VirtualMachineSizeOperations class.
   * @constructor
   * 
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  function VirtualMachineSizeOperations(client) {
    this.client = client;
  }
  
  /**
   * Lists virtual-machine-sizes available in a location for a subscription.
   * 
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  VirtualMachineSizeOperations.prototype.list = function(location, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (location === null || location === undefined) {
      return callback(new Error('location cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/providers/';
    url2 = url2 + 'Microsoft.Compute';
    url2 = url2 + '/locations/';
    url2 = url2 + encodeURIComponent(location);
    url2 = url2 + '/vmSizes';
    var queryParameters = [];
    queryParameters.push('api-version=2014-12-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { virtualMachineSizes: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var virtualMachineSizeInstance = {};
              result.virtualMachineSizes.push(virtualMachineSizeInstance);
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                virtualMachineSizeInstance.name = nameInstance;
              }
              
              var numberOfCoresValue = valueValue['numberOfCores'];
              if (numberOfCoresValue !== null && numberOfCoresValue !== undefined) {
                var numberOfCoresInstance = numberOfCoresValue;
                virtualMachineSizeInstance.numberOfCores = numberOfCoresInstance;
              }
              
              var osDiskSizeInMBValue = valueValue['osDiskSizeInMB'];
              if (osDiskSizeInMBValue !== null && osDiskSizeInMBValue !== undefined) {
                var osDiskSizeInMBInstance = osDiskSizeInMBValue;
                virtualMachineSizeInstance.oSDiskSizeInMB = osDiskSizeInMBInstance;
              }
              
              var resourceDiskSizeInMBValue = valueValue['resourceDiskSizeInMB'];
              if (resourceDiskSizeInMBValue !== null && resourceDiskSizeInMBValue !== undefined) {
                var resourceDiskSizeInMBInstance = resourceDiskSizeInMBValue;
                virtualMachineSizeInstance.resourceDiskSizeInMB = resourceDiskSizeInMBInstance;
              }
              
              var memoryInMBValue = valueValue['memoryInMB'];
              if (memoryInMBValue !== null && memoryInMBValue !== undefined) {
                var memoryInMBInstance = memoryInMBValue;
                virtualMachineSizeInstance.memoryInMB = memoryInMBInstance;
              }
              
              var maxDataDiskCountValue = valueValue['maxDataDiskCount'];
              if (maxDataDiskCountValue !== null && maxDataDiskCountValue !== undefined) {
                var maxDataDiskCountInstance = maxDataDiskCountValue;
                virtualMachineSizeInstance.maxDataDiskCount = maxDataDiskCountInstance;
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return VirtualMachineSizeOperations;
})();
